@@ -13,7 +13,7 @@
 #include <kmainwindow.h>
 
 #include <QJSEngine>
-#include <QtTestWidgets>
+#include <QTest>
 
 #include "testutils.h"
 
@@ -64,7 +64,7 @@ void BugTest::tryCrash()
     view->show();
     view->resize(900, 800);
     view->setCursorPosition(Cursor(0, 0));
-    doc->editBegin();
+    doc->editStart();
     // QTest::qWait(200);
 
     // evaluate test-script
@@ -72,7 +72,6 @@ void BugTest::tryCrash()
     QFile sourceFile(QLatin1String(TEST_DATA_DIR "bug313759.js"));
     QVERIFY(sourceFile.open(QFile::ReadOnly));
     QTextStream stream(&sourceFile);
-    stream.setCodec("UTF8");
     QString code = stream.readAll();
     sourceFile.close();
     // execute script
