@@ -19,11 +19,11 @@
 #include <ktexteditor/movingrange.h>
 
 #include <QStringListModel>
-#include <QtTestWidgets>
+#include <QTest>
 
 QTEST_MAIN(SearchBarTest)
 
-#define testNewRow() (QTest::newRow(QString("line %1").arg(__LINE__).toLatin1().data()))
+#define testNewRow() (QTest::newRow(QStringLiteral("line %1").arg(__LINE__).toLatin1().data()))
 
 using namespace KTextEditor;
 
@@ -39,7 +39,7 @@ SearchBarTest::~SearchBarTest()
 void SearchBarTest::initTestCase()
 {
     KTextEditor::EditorPrivate::enableUnitTestMode();
-    KMessageBox::saveDontShowAgainYesNo(QLatin1String("DoNotShowAgainContinueSearchDialog"), KMessageBox::Yes);
+    KMessageBox::saveDontShowAgainTwoActions(QLatin1String("DoNotShowAgainContinueSearchDialog"), KMessageBox::PrimaryAction);
 }
 
 void SearchBarTest::cleanupTestCase()
@@ -49,14 +49,14 @@ void SearchBarTest::cleanupTestCase()
 void SearchBarTest::testFindNextIncremental()
 {
     KTextEditor::DocumentPrivate doc;
-    doc.setText("a a a b b");
+    doc.setText(QStringLiteral("a a a b b"));
 
     KTextEditor::ViewPrivate view(&doc, nullptr);
     KateViewConfig config(&view);
 
     KateSearchBar bar(false, &view, &config);
 
-    bar.setSearchPattern("b");
+    bar.setSearchPattern(QStringLiteral("b"));
 
     QCOMPARE(view.selectionRange(), Range(0, 6, 0, 7));
 
@@ -64,7 +64,7 @@ void SearchBarTest::testFindNextIncremental()
 
     QCOMPARE(view.selectionRange(), Range(0, 8, 0, 9));
 
-    bar.setSearchPattern("a");
+    bar.setSearchPattern(QStringLiteral("a"));
 
     QCOMPARE(view.selectionRange(), Range(0, 0, 0, 1));
 
@@ -81,35 +81,104 @@ void SearchBarTest::testFindNextIncremental()
     QCOMPARE(view.selectionRange(), Range(0, 0, 0, 1));
 }
 
-
 void SearchBarTest::testFindNextZeroLengthMatch()
 {
     // Some regularexpression matches are zero length assertions
     // e.g. '$', '^', '\b', test that the cursor won't be stuck
     // on one match when using FindNext
     KTextEditor::DocumentPrivate doc;
-    doc.setText("a\nb \nc\n\n");
+    doc.setText(QStringLiteral("a\nb \nc\n\n"));
 
     KTextEditor::ViewPrivate view(&doc, nullptr);
     KateViewConfig config(&view);
     KateSearchBar bar(true, &view, &config);
     bar.setSearchMode(KateSearchBar::MODE_REGEX);
 
-    bar.setSearchPattern("$");
+    bar.setSearchPattern(QStringLiteral("$"));
 
     QVERIFY(bar.isPower());
 
     bar.findNext();
-    QCOMPARE(view.cursorPosition(), Cursor(0,1));
+    QCOMPARE(view.cursorPosition(), Cursor(0, 1));
 
     bar.findNext();
-    QCOMPARE(view.cursorPosition(), Cursor(1,2));
+    QCOMPARE(view.cursorPosition(), Cursor(1, 2));
 
     bar.findNext();
-    QCOMPARE(view.cursorPosition(), Cursor(2,1));
+    QCOMPARE(view.cursorPosition(), Cursor(2, 1));
 
     bar.findNext();
-    QCOMPARE(view.cursorPosition(), Cursor(3,0));
+    QCOMPARE(view.cursorPosition(), Cursor(3, 0));
+
+    // Test Unicode
+    doc.setText(QStringLiteral("aéöz\n"));
+    bar.setSearchPattern(QStringLiteral("\\w"));
+    view.setCursorPosition(Cursor(0, 0));
+
+    bar.findNext();
+    QCOMPARE(view.cursorPosition(), Cursor(0, 1));
+
+    bar.findNext();
+    QCOMPARE(view.cursorPosition(), Cursor(0, 2));
+
+    bar.findNext();
+    QCOMPARE(view.cursorPosition(), Cursor(0, 3));
+
+    bar.findNext();
+    QCOMPARE(view.cursorPosition(), Cursor(0, 4));
+
+    doc.setText(QStringLiteral("aé ö z\n"));
+    bar.setSearchPattern(QStringLiteral("\\b"));
+    view.setCursorPosition(Cursor(0, 0));
+
+    bar.findNext();
+    QCOMPARE(view.cursorPosition(), Cursor(0, 2));
+    QCOMPARE(doc.text(Range(0, 1, 0, 2)), QStringLiteral("é"));
+
+    bar.findNext();
+    QCOMPARE(view.cursorPosition(), Cursor(0, 3));
+    QCOMPARE(doc.text(Range(0, 3, 0, 4)), QStringLiteral("ö"));
+
+    bar.findNext();
+    QCOMPARE(view.cursorPosition(), Cursor(0, 4));
+    QCOMPARE(doc.text(Range(0, 3, 0, 4)), QStringLiteral("ö"));
+
+    bar.findNext();
+    QCOMPARE(view.cursorPosition(), Cursor(0, 5));
+    QCOMPARE(doc.text(Range(0, 5, 0, 6)), QStringLiteral("z"));
+
+    bar.findNext();
+    QCOMPARE(view.cursorPosition(), Cursor(0, 6));
+    QCOMPARE(doc.text(Range(0, 5, 0, 6)), QStringLiteral("z"));
+
+    bar.findNext();
+    // Search wraps, back to before first char
+    QCOMPARE(view.cursorPosition(), Cursor(0, 0));
+    QCOMPARE(doc.text(Range(0, 0, 0, 1)), QStringLiteral("a"));
+}
+
+void SearchBarTest::testFindNextNoNewLineAtEnd()
+{
+    // Testing with a document that doesn't end with a new line
+    KTextEditor::DocumentPrivate doc;
+    doc.setText(QStringLiteral(" \n \n "));
+
+    KTextEditor::ViewPrivate view(&doc, nullptr);
+    KateViewConfig config(&view);
+    KateSearchBar bar(true, &view, &config);
+    QVERIFY(bar.isPower());
+    bar.setSearchMode(KateSearchBar::MODE_REGEX);
+    bar.setSearchPattern(QStringLiteral("^ *\\n"));
+
+    bar.findNext();
+    QCOMPARE(view.selectionRange(), Range(0, 0, 1, 0));
+
+    bar.findNext();
+    QCOMPARE(view.selectionRange(), Range(1, 0, 2, 0));
+
+    bar.findNext();
+    // Search wraps
+    QCOMPARE(view.selectionRange(), Range(0, 0, 1, 0));
 }
 
 void SearchBarTest::testSetMatchCaseIncremental()
@@ -118,14 +187,14 @@ void SearchBarTest::testSetMatchCaseIncremental()
     KTextEditor::ViewPrivate view(&doc, nullptr);
     KateViewConfig config(&view);
 
-    doc.setText("a A a");
+    doc.setText(QStringLiteral("a A a"));
     KateSearchBar bar(false, &view, &config);
 
     QVERIFY(!bar.isPower());
     QVERIFY(!view.selection());
 
     bar.setMatchCase(false);
-    bar.setSearchPattern("A");
+    bar.setSearchPattern(QStringLiteral("A"));
 
     QVERIFY(!bar.matchCase());
     QCOMPARE(view.selectionRange(), Range(0, 0, 0, 1));
@@ -152,7 +221,7 @@ void SearchBarTest::testSetMatchCasePower()
     KTextEditor::ViewPrivate view(&doc, nullptr);
     KateViewConfig config(&view);
 
-    doc.setText("a A a");
+    doc.setText(QStringLiteral("a A a"));
     view.setCursorPosition(Cursor(0, 0));
 
     KateSearchBar bar(true, &view, &config);
@@ -161,10 +230,10 @@ void SearchBarTest::testSetMatchCasePower()
     QVERIFY(!view.selection());
 
     bar.setMatchCase(false);
-    bar.setSearchPattern("A");
+    bar.setSearchPattern(QStringLiteral("A"));
     bar.findNext();
 
-    QCOMPARE(bar.searchPattern(), QString("A"));
+    QCOMPARE(bar.searchPattern(), QStringLiteral("A"));
     QVERIFY(!bar.matchCase());
     QCOMPARE(view.selectionRange(), Range(0, 0, 0, 1));
 
@@ -193,10 +262,10 @@ void SearchBarTest::testSetSelectionOnlyPower()
     KTextEditor::ViewPrivate view(&doc, nullptr);
     KateViewConfig config(&view);
 
-    doc.setText("a a a");
+    doc.setText(QStringLiteral("a a a a"));
     KateSearchBar bar(true, &view, &config);
 
-    bar.setSearchPattern("a");
+    bar.setSearchPattern(QStringLiteral("a"));
 
     QVERIFY(bar.isPower());
     QVERIFY(!view.selection());
@@ -217,10 +286,26 @@ void SearchBarTest::testSetSelectionOnlyPower()
     QCOMPARE(view.selectionRange(), Range(0, 2, 0, 3));
     QVERIFY(bar.selectionOnly());
 
-    bar.setSelectionOnly(false);
     bar.findNext();
 
     QCOMPARE(view.selectionRange(), Range(0, 4, 0, 5));
+    QVERIFY(bar.selectionOnly());
+
+    // Test Search wrap for selection only
+    bar.findNext();
+
+    QCOMPARE(view.selectionRange(), Range(0, 2, 0, 3));
+    QVERIFY(bar.selectionOnly());
+
+    bar.findPrevious();
+
+    QCOMPARE(view.selectionRange(), Range(0, 4, 0, 5));
+    QVERIFY(bar.selectionOnly());
+
+    bar.setSelectionOnly(false);
+    bar.findNext();
+
+    QCOMPARE(view.selectionRange(), Range(0, 6, 0, 7));
     QVERIFY(!bar.selectionOnly());
 }
 
@@ -242,16 +327,16 @@ void SearchBarTest::testSetSearchPattern()
     KTextEditor::ViewPrivate view(&doc, nullptr);
     KateViewConfig config(&view);
 
-    doc.setText("a a a");
+    doc.setText(QStringLiteral("a a a"));
 
     KateSearchBar bar(power, &view, &config);
 
-    bar.setSearchPattern("a");
+    bar.setSearchPattern(QStringLiteral("a"));
     bar.findAll();
 
     QCOMPARE(bar.m_hlRanges.size(), 3);
 
-    bar.setSearchPattern("a ");
+    bar.setSearchPattern(QStringLiteral("a "));
 
     QCOMPARE(bar.m_hlRanges.size(), numMatches2);
 
@@ -266,13 +351,13 @@ void SearchBarTest::testSetSelectionOnly()
     KTextEditor::ViewPrivate view(&doc, nullptr);
     KateViewConfig config(&view);
 
-    doc.setText("a a a");
+    doc.setText(QStringLiteral("a a a"));
     view.setSelection(Range(0, 0, 0, 3));
 
     KateSearchBar bar(false, &view, &config);
 
     bar.setSelectionOnly(false);
-    bar.setSearchPattern("a");
+    bar.setSearchPattern(QStringLiteral("a"));
     bar.findAll();
 
     QCOMPARE(bar.m_hlRanges.size(), 3);
@@ -302,12 +387,12 @@ void SearchBarTest::testFindAll()
     KTextEditor::ViewPrivate view(&doc, nullptr);
     KateViewConfig config(&view);
 
-    doc.setText("a a a");
+    doc.setText(QStringLiteral("a a a"));
     KateSearchBar bar(power, &view, &config);
 
     QCOMPARE(bar.isPower(), power);
 
-    bar.setSearchPattern("a");
+    bar.setSearchPattern(QStringLiteral("a"));
     bar.findAll();
 
     QCOMPARE(bar.m_hlRanges.size(), 3);
@@ -315,7 +400,7 @@ void SearchBarTest::testFindAll()
     QCOMPARE(bar.m_hlRanges.at(1)->toRange(), Range(0, 2, 0, 3));
     QCOMPARE(bar.m_hlRanges.at(2)->toRange(), Range(0, 4, 0, 5));
 
-    bar.setSearchPattern("a ");
+    bar.setSearchPattern(QStringLiteral("a "));
 
     QCOMPARE(bar.m_hlRanges.size(), numMatches2);
 
@@ -323,7 +408,7 @@ void SearchBarTest::testFindAll()
 
     QCOMPARE(bar.m_hlRanges.size(), 2);
 
-    bar.setSearchPattern("a  ");
+    bar.setSearchPattern(QStringLiteral("a  "));
 
     QCOMPARE(bar.m_hlRanges.size(), numMatches4);
 
@@ -332,17 +417,74 @@ void SearchBarTest::testFindAll()
     QCOMPARE(bar.m_hlRanges.size(), 0);
 }
 
+void SearchBarTest::testReplaceInSelectionOnly()
+{
+    KTextEditor::DocumentPrivate doc;
+    KTextEditor::ViewPrivate view(&doc, nullptr);
+    KateViewConfig config(&view);
+
+    doc.setText(QStringLiteral("a\na\na\na\na"));
+    KateSearchBar bar(true, &view, &config);
+
+    bar.setSearchPattern(QStringLiteral("a\n"));
+
+    view.setSelection(Range(1, 0, 4, 0));
+    bar.setSelectionOnly(true);
+
+    QVERIFY(bar.selectionOnly());
+
+    bar.replaceNext();
+
+    QCOMPARE(view.selectionRange(), Range(1, 0, 2, 0));
+    QVERIFY(bar.selectionOnly());
+
+    bar.replaceNext();
+
+    QCOMPARE(view.selectionRange(), Range(1, 0, 2, 0));
+    QVERIFY(bar.selectionOnly());
+
+    bar.replaceNext();
+
+    QCOMPARE(view.selectionRange(), Range(1, 0, 2, 0));
+    QVERIFY(bar.selectionOnly());
+
+    bar.replaceNext();
+
+    QCOMPARE(view.selectionRange(), Range(1, 0, 1, 0));
+    QCOMPARE(doc.text(), QStringLiteral("a\na"));
+    QVERIFY(bar.selectionOnly());
+
+    // Test undo (search selection range should still be preserved)
+    doc.undo();
+    doc.undo();
+    doc.undo();
+
+    QCOMPARE(view.selectionRange(), Range(1, 0, 2, 0));
+    QVERIFY(bar.selectionOnly());
+
+    bar.findPrevious();
+
+    QCOMPARE(view.selectionRange(), Range(3, 0, 4, 0));
+    QVERIFY(bar.selectionOnly());
+
+    // TODO: Make sure deleted parts of the selection range are added back on undo (currently, the MovingRange just moves forward and does not add back the
+    // deleted range) bar.findNext();
+    //
+    // QCOMPARE(view.selectionRange(), Range(1, 0, 2, 0));
+    // QVERIFY(bar.selectionOnly());
+}
+
 void SearchBarTest::testReplaceAll()
 {
     KTextEditor::DocumentPrivate doc;
     KTextEditor::ViewPrivate view(&doc, nullptr);
     KateViewConfig config(&view);
 
-    doc.setText("a a a");
+    doc.setText(QStringLiteral("a a a"));
     KateSearchBar bar(true, &view, &config);
 
-    bar.setSearchPattern("a");
-    bar.setReplacementPattern("");
+    bar.setSearchPattern(QStringLiteral("a"));
+    bar.setReplacementPattern(QString());
     bar.replaceAll();
 
     QCOMPARE(bar.m_hlRanges.size(), 3);
@@ -350,8 +492,8 @@ void SearchBarTest::testReplaceAll()
     QCOMPARE(bar.m_hlRanges.at(1)->toRange(), Range(0, 1, 0, 1));
     QCOMPARE(bar.m_hlRanges.at(2)->toRange(), Range(0, 2, 0, 2));
 
-    bar.setSearchPattern(" ");
-    bar.setReplacementPattern("b");
+    bar.setSearchPattern(QStringLiteral(" "));
+    bar.setReplacementPattern(QStringLiteral("b"));
     bar.replaceAll();
 
     QCOMPARE(bar.m_hlRanges.size(), 2);
@@ -366,20 +508,20 @@ void SearchBarTest::testFindSelectionForward_data()
     QTest::addColumn<Range>("selectionRange");
     QTest::addColumn<Range>("match");
 
-    testNewRow() << "a a a" << false << Range(0, 0, 0, 1) << Range(0, 0, 0, 2);
-    testNewRow() << "a a a" << true << Range(0, 0, 0, 1) << Range(0, 0, 0, 1);
+    testNewRow() << QStringLiteral("a a a") << false << Range(0, 0, 0, 1) << Range(0, 0, 0, 2);
+    testNewRow() << QStringLiteral("a a a") << true << Range(0, 0, 0, 1) << Range(0, 0, 0, 1);
 
-    testNewRow() << "a a a" << false << Range(0, 0, 0, 2) << Range(0, 2, 0, 4);
-    testNewRow() << "a a a" << true << Range(0, 0, 0, 2) << Range(0, 0, 0, 2);
+    testNewRow() << QStringLiteral("a a a") << false << Range(0, 0, 0, 2) << Range(0, 2, 0, 4);
+    testNewRow() << QStringLiteral("a a a") << true << Range(0, 0, 0, 2) << Range(0, 0, 0, 2);
 
-    testNewRow() << "a a a" << false << Range(0, 0, 0, 3) << Range(0, 0, 0, 2);
-    testNewRow() << "a a a" << true << Range(0, 0, 0, 3) << Range(0, 0, 0, 2);
+    testNewRow() << QStringLiteral("a a a") << false << Range(0, 0, 0, 3) << Range(0, 0, 0, 2);
+    testNewRow() << QStringLiteral("a a a") << true << Range(0, 0, 0, 3) << Range(0, 0, 0, 2);
 
-    testNewRow() << "a a a" << false << Range(0, 2, 0, 4) << Range(0, 0, 0, 2);
-    testNewRow() << "a a a" << true << Range(0, 2, 0, 4) << Range(0, 2, 0, 4);
+    testNewRow() << QStringLiteral("a a a") << false << Range(0, 2, 0, 4) << Range(0, 0, 0, 2);
+    testNewRow() << QStringLiteral("a a a") << true << Range(0, 2, 0, 4) << Range(0, 2, 0, 4);
 
-    testNewRow() << "a a a" << false << Range(0, 3, 0, 4) << Range(0, 0, 0, 2);
-    testNewRow() << "a a a" << true << Range(0, 3, 0, 4) << Range(0, 3, 0, 4);
+    testNewRow() << QStringLiteral("a a a") << false << Range(0, 3, 0, 4) << Range(0, 0, 0, 2);
+    testNewRow() << QStringLiteral("a a a") << true << Range(0, 3, 0, 4) << Range(0, 3, 0, 4);
 }
 
 void SearchBarTest::testFindSelectionForward()
@@ -398,7 +540,7 @@ void SearchBarTest::testFindSelectionForward()
     view.setSelection(Range(0, 0, 0, 2));
 
     KateSearchBar bar(true, &view, &config);
-    QVERIFY(bar.searchPattern() == QString("a "));
+    QVERIFY(bar.searchPattern() == QStringLiteral("a "));
 
     view.setSelection(selectionRange);
     QCOMPARE(view.selectionRange(), selectionRange);
@@ -430,11 +572,11 @@ void SearchBarTest::testRemoveWithSelectionForward()
     KTextEditor::ViewPrivate view(&doc, nullptr);
     KateViewConfig config(&view);
 
-    doc.setText("a a a");
+    doc.setText(QStringLiteral("a a a"));
     view.setSelection(selectionRange);
 
     KateSearchBar bar(true, &view, &config);
-    bar.setSearchPattern("a ");
+    bar.setSearchPattern(QStringLiteral("a "));
     bar.setSelectionOnly(false);
 
     bar.replaceNext();
@@ -464,11 +606,11 @@ void SearchBarTest::testRemoveInSelectionForward()
     KTextEditor::ViewPrivate view(&doc, nullptr);
     KateViewConfig config(&view);
 
-    doc.setText("a a a");
+    doc.setText(QStringLiteral("a a a"));
     view.setSelection(selectionRange);
 
     KateSearchBar bar(true, &view, &config);
-    bar.setSearchPattern("a ");
+    bar.setSearchPattern(QStringLiteral("a "));
     bar.setSelectionOnly(true);
 
     QVERIFY(bar.replacementPattern().isEmpty());
@@ -486,16 +628,16 @@ void SearchBarTest::testReplaceWithDoubleSelecion_data()
     QTest::addColumn<Range>("match");
 
     //  testNewRow() << "a"  << Range(0, 0, 0, 1) << "aa" << Range(?, ?, ?, ?);
-    testNewRow() << "aa" << Range(0, 1, 0, 2) << "aaa" << Range(0, 0, 0, 1);
-    testNewRow() << "aa" << Range(0, 0, 0, 1) << "aaa" << Range(0, 2, 0, 3);
+    testNewRow() << QStringLiteral("aa") << Range(0, 1, 0, 2) << QStringLiteral("aaa") << Range(0, 0, 0, 1);
+    testNewRow() << QStringLiteral("aa") << Range(0, 0, 0, 1) << QStringLiteral("aaa") << Range(0, 2, 0, 3);
 
     //  testNewRow() << "ab"  << Range(0, 0, 0, 1) << "aab"  << Range(?, ?, ?, ?);
-    testNewRow() << "aab" << Range(0, 0, 0, 1) << "aaab" << Range(0, 2, 0, 3);
-    testNewRow() << "aba" << Range(0, 0, 0, 1) << "aaba" << Range(0, 3, 0, 4);
+    testNewRow() << QStringLiteral("aab") << Range(0, 0, 0, 1) << QStringLiteral("aaab") << Range(0, 2, 0, 3);
+    testNewRow() << QStringLiteral("aba") << Range(0, 0, 0, 1) << QStringLiteral("aaba") << Range(0, 3, 0, 4);
 
     //  testNewRow() << "ab"   << Range(0, 0, 0, 2) << "abab"   << Range(?, ?, ?, ?);
-    testNewRow() << "abab" << Range(0, 0, 0, 2) << "ababab" << Range(0, 4, 0, 6);
-    testNewRow() << "abab" << Range(0, 2, 0, 4) << "ababab" << Range(0, 0, 0, 2);
+    testNewRow() << QStringLiteral("abab") << Range(0, 0, 0, 2) << QStringLiteral("ababab") << Range(0, 4, 0, 6);
+    testNewRow() << QStringLiteral("abab") << Range(0, 2, 0, 4) << QStringLiteral("ababab") << Range(0, 0, 0, 2);
 }
 
 void SearchBarTest::testReplaceWithDoubleSelecion()
@@ -528,15 +670,15 @@ void SearchBarTest::testReplaceDollar()
     KTextEditor::ViewPrivate view(&doc, nullptr);
     KateViewConfig config(&view);
 
-    doc.setText("aaa\nbbb\nccc\n\n\naaa\nbbb\nccc\nddd\n");
+    doc.setText(QStringLiteral("aaa\nbbb\nccc\n\n\naaa\nbbb\nccc\nddd\n"));
 
     KateSearchBar bar(true, &view, &config);
 
     bar.setSearchPattern(QStringLiteral("$"));
     bar.setSearchMode(KateSearchBar::MODE_REGEX);
-    bar.setReplacementPattern("D");
+    bar.setReplacementPattern(QStringLiteral("D"));
     bar.replaceAll();
-    QCOMPARE(doc.text(), QString("aaaD\nbbbD\ncccD\nD\nD\naaaD\nbbbD\ncccD\ndddD\n"));
+    QCOMPARE(doc.text(), QStringLiteral("aaaD\nbbbD\ncccD\nD\nD\naaaD\nbbbD\ncccD\ndddD\n"));
 }
 
 void SearchBarTest::testSearchHistoryIncremental()
@@ -546,28 +688,28 @@ void SearchBarTest::testSearchHistoryIncremental()
     KateViewConfig *const config = view.config();
     KTextEditor::EditorPrivate::self()->searchHistoryModel()->setStringList(QStringList());
 
-    doc.setText("foo bar");
+    doc.setText(QStringLiteral("foo bar"));
 
     KateSearchBar bar(false, &view, config);
 
-    bar.setSearchPattern("foo");
+    bar.setSearchPattern(QStringLiteral("foo"));
     bar.findNext();
 
-    QCOMPARE(bar.m_incUi->pattern->findText("foo"), 0);
+    QCOMPARE(bar.m_incUi->pattern->findText(QStringLiteral("foo")), 0);
 
-    bar.setSearchPattern("bar");
+    bar.setSearchPattern(QStringLiteral("bar"));
     bar.findNext();
 
-    QCOMPARE(bar.m_incUi->pattern->findText("bar"), 0);
-    QCOMPARE(bar.m_incUi->pattern->findText("foo"), 1);
+    QCOMPARE(bar.m_incUi->pattern->findText(QStringLiteral("bar")), 0);
+    QCOMPARE(bar.m_incUi->pattern->findText(QStringLiteral("foo")), 1);
 
     KTextEditor::DocumentPrivate doc2;
     KTextEditor::ViewPrivate view2(&doc2, nullptr);
     KateViewConfig *const config2 = view2.config();
     KateSearchBar bar2(false, &view2, config2);
 
-    QCOMPARE(bar2.m_incUi->pattern->findText("bar"), 0);
-    QCOMPARE(bar2.m_incUi->pattern->findText("foo"), 1);
+    QCOMPARE(bar2.m_incUi->pattern->findText(QStringLiteral("bar")), 0);
+    QCOMPARE(bar2.m_incUi->pattern->findText(QStringLiteral("foo")), 1);
 
     // testcase for https://bugs.kde.org/show_bug.cgi?id=248305
     bar2.m_incUi->pattern->setCurrentIndex(1);
@@ -583,27 +725,27 @@ void SearchBarTest::testSearchHistoryPower()
     KateViewConfig *const config = view.config();
     KTextEditor::EditorPrivate::self()->searchHistoryModel()->setStringList(QStringList());
 
-    doc.setText("foo bar");
+    doc.setText(QStringLiteral("foo bar"));
 
     KateSearchBar bar(true, &view, config);
 
     QCOMPARE(bar.m_powerUi->pattern->count(), 0);
 
-    bar.setSearchPattern("foo");
+    bar.setSearchPattern(QStringLiteral("foo"));
     bar.findNext();
 
-    QCOMPARE(bar.m_powerUi->pattern->findText("foo"), 0);
+    QCOMPARE(bar.m_powerUi->pattern->findText(QStringLiteral("foo")), 0);
 
     bar.findNext();
 
-    QCOMPARE(bar.m_powerUi->pattern->findText("foo"), 0);
+    QCOMPARE(bar.m_powerUi->pattern->findText(QStringLiteral("foo")), 0);
     QCOMPARE(bar.m_powerUi->pattern->count(), 1);
 
-    bar.setSearchPattern("bar");
+    bar.setSearchPattern(QStringLiteral("bar"));
     bar.findNext();
 
-    QCOMPARE(bar.m_powerUi->pattern->findText("bar"), 0);
-    QCOMPARE(bar.m_powerUi->pattern->findText("foo"), 1);
+    QCOMPARE(bar.m_powerUi->pattern->findText(QStringLiteral("bar")), 0);
+    QCOMPARE(bar.m_powerUi->pattern->findText(QStringLiteral("foo")), 1);
     QCOMPARE(bar.m_powerUi->pattern->count(), 2);
 
     KTextEditor::DocumentPrivate doc2;
@@ -611,8 +753,8 @@ void SearchBarTest::testSearchHistoryPower()
     KateViewConfig *const config2 = view2.config();
     KateSearchBar bar2(true, &view2, config2);
 
-    QCOMPARE(bar2.m_powerUi->pattern->findText("bar"), 0);
-    QCOMPARE(bar2.m_powerUi->pattern->findText("foo"), 1);
+    QCOMPARE(bar2.m_powerUi->pattern->findText(QStringLiteral("bar")), 0);
+    QCOMPARE(bar2.m_powerUi->pattern->findText(QStringLiteral("foo")), 1);
 }
 
 // Make sure Kate doesn't replace anything outside selection in block mode (see bug 253191)
@@ -623,17 +765,17 @@ void SearchBarTest::testReplaceInBlockMode()
     view.setInputMode(View::NormalInputMode);
     KateViewConfig config(&view);
 
-    doc.setText("111\n111");
+    doc.setText(QStringLiteral("111\n111"));
     view.setBlockSelection(true);
     view.setSelection(KTextEditor::Range(0, 1, 1, 2));
 
     KateSearchBar bar(true, &view, &config);
 
-    bar.setSearchPattern("1");
-    bar.setReplacementPattern("2");
+    bar.setSearchPattern(QStringLiteral("1"));
+    bar.setReplacementPattern(QStringLiteral("2"));
     bar.replaceAll();
 
-    QCOMPARE(doc.text(), QString("121\n121"));
+    QCOMPARE(doc.text(), QStringLiteral("121\n121"));
 }
 
 void SearchBarTest::testReplaceManyCapturesBug365124()
@@ -642,17 +784,17 @@ void SearchBarTest::testReplaceManyCapturesBug365124()
     KTextEditor::ViewPrivate view(&doc, nullptr);
     KateViewConfig config(&view);
 
-    doc.setText("one two three four five six seven eight nine ten eleven twelve thirteen\n");
+    doc.setText(QStringLiteral("one two three four five six seven eight nine ten eleven twelve thirteen\n"));
 
     KateSearchBar bar(true, &view, &config);
 
-    bar.setSearchPattern("^(.*) (.*) (.*) (.*) (.*) (.*) (.*) (.*) (.*) (.*) (.*) (.*) (.*)$");
+    bar.setSearchPattern(QStringLiteral("^(.*) (.*) (.*) (.*) (.*) (.*) (.*) (.*) (.*) (.*) (.*) (.*) (.*)$"));
     bar.setSearchMode(KateSearchBar::MODE_REGEX);
-    bar.setReplacementPattern("\\{1}::\\2::\\3::\\4::\\5::\\6::\\7::\\8::\\9::\\{10}::\\{11}::\\{12}::\\{13}");
+    bar.setReplacementPattern(QStringLiteral("\\{1}::\\2::\\3::\\4::\\5::\\6::\\7::\\8::\\9::\\{10}::\\{11}::\\{12}::\\{13}"));
 
     bar.replaceAll();
 
-    QCOMPARE(doc.text(), QString("one::two::three::four::five::six::seven::eight::nine::ten::eleven::twelve::thirteen\n"));
+    QCOMPARE(doc.text(), QStringLiteral("one::two::three::four::five::six::seven::eight::nine::ten::eleven::twelve::thirteen\n"));
 }
 
 void SearchBarTest::testReplaceEscapeSequence_data()
