@@ -1,21 +1,9 @@
-/* This file is part of the KDE libraries
-   Copyright (C) 2010 Bernhard Beschow <bbeschow@cs.tu-berlin.de>
-   Copyright (C) 2009-2018 Dominik Haumann <dhaumann@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2010 Bernhard Beschow <bbeschow@cs.tu-berlin.de>
+    SPDX-FileCopyrightText: 2009-2018 Dominik Haumann <dhaumann@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "undomanager_test.h"
@@ -25,7 +13,7 @@
 #include <kateundomanager.h>
 #include <kateview.h>
 
-#include <QtTestWidgets>
+#include <QTest>
 
 QTEST_MAIN(UndoManagerTest)
 
@@ -37,125 +25,115 @@ UndoManagerTest::UndoManagerTest()
     KTextEditor::EditorPrivate::enableUnitTestMode();
 }
 
-class UndoManagerTest::TestDocument : public KTextEditor::DocumentPrivate
-{
-public:
-    TestDocument()
-        : KTextEditor::DocumentPrivate()
-    {
-    }
-};
-
 void UndoManagerTest::testUndoRedoCount()
 {
-    TestDocument doc;
-    KateUndoManager *undoManager = doc.undoManager();
+    KTextEditor::DocumentPrivate doc;
 
     // no undo/redo items at the beginning
-    QCOMPARE(undoManager->undoCount(), 0u);
-    QCOMPARE(undoManager->redoCount(), 0u);
+    QCOMPARE(doc.undoCount(), 0u);
+    QCOMPARE(doc.redoCount(), 0u);
 
-    doc.insertText(Cursor(0, 0), QLatin1String("a"));
+    doc.insertText(Cursor(0, 0), QStringLiteral("a"));
 
     // create one insert-group
-    QCOMPARE(undoManager->undoCount(), 1u);
-    QCOMPARE(undoManager->redoCount(), 0u);
+    QCOMPARE(doc.undoCount(), 1u);
+    QCOMPARE(doc.redoCount(), 0u);
 
     doc.undo();
 
     // move insert-group to redo items
-    QCOMPARE(undoManager->undoCount(), 0u);
-    QCOMPARE(undoManager->redoCount(), 1u);
+    QCOMPARE(doc.undoCount(), 0u);
+    QCOMPARE(doc.redoCount(), 1u);
 
     doc.redo();
 
     // move insert-group back to undo items
-    QCOMPARE(undoManager->undoCount(), 1u);
-    QCOMPARE(undoManager->redoCount(), 0u);
+    QCOMPARE(doc.undoCount(), 1u);
+    QCOMPARE(doc.redoCount(), 0u);
 
-    doc.insertText(Cursor(0, 1), QLatin1String("b"));
+    doc.insertText(Cursor(0, 1), QStringLiteral("b"));
 
     // merge "b" into insert-group
-    QCOMPARE(undoManager->undoCount(), 1u);
-    QCOMPARE(undoManager->redoCount(), 0u);
+    QCOMPARE(doc.undoCount(), 1u);
+    QCOMPARE(doc.redoCount(), 0u);
 
     doc.removeText(Range(0, 1, 0, 2));
 
     // create an additional remove-group
-    QCOMPARE(undoManager->undoCount(), 2u);
-    QCOMPARE(undoManager->redoCount(), 0u);
+    QCOMPARE(doc.undoCount(), 2u);
+    QCOMPARE(doc.redoCount(), 0u);
 
     doc.undo();
 
     // move remove-group to redo items
-    QCOMPARE(undoManager->undoCount(), 1u);
-    QCOMPARE(undoManager->redoCount(), 1u);
+    QCOMPARE(doc.undoCount(), 1u);
+    QCOMPARE(doc.redoCount(), 1u);
 
-    doc.insertText(Cursor(0, 1), QLatin1String("b"));
+    doc.insertText(Cursor(0, 1), QStringLiteral("b"));
 
     // merge "b" into insert-group
     // and remove remove-group
-    QCOMPARE(undoManager->undoCount(), 1u);
-    QCOMPARE(undoManager->redoCount(), 0u);
+    QCOMPARE(doc.undoCount(), 1u);
+    QCOMPARE(doc.redoCount(), 0u);
 }
 
 void UndoManagerTest::testSafePoint()
 {
-    TestDocument doc;
+    KTextEditor::DocumentPrivate doc;
     KateUndoManager *undoManager = doc.undoManager();
 
-    doc.insertText(Cursor(0, 0), QLatin1String("a"));
+    doc.insertText(Cursor(0, 0), QStringLiteral("a"));
 
     // create one undo group
-    QCOMPARE(undoManager->undoCount(), 1u);
-    QCOMPARE(undoManager->redoCount(), 0u);
+    QCOMPARE(doc.undoCount(), 1u);
+    QCOMPARE(doc.redoCount(), 0u);
 
     undoManager->undoSafePoint();
-    doc.insertText(Cursor(0, 1), QLatin1String("b"));
+    doc.insertText(Cursor(0, 1), QStringLiteral("b"));
 
     // create a second undo group (don't merge)
-    QCOMPARE(undoManager->undoCount(), 2u);
+    QCOMPARE(doc.undoCount(), 2u);
 
     doc.undo();
 
     // move second undo group to redo items
-    QCOMPARE(undoManager->undoCount(), 1u);
-    QCOMPARE(undoManager->redoCount(), 1u);
+    QCOMPARE(doc.undoCount(), 1u);
+    QCOMPARE(doc.redoCount(), 1u);
 
-    doc.insertText(Cursor(0, 1), QLatin1String("b"));
+    doc.insertText(Cursor(0, 1), QStringLiteral("b"));
 
     // create a second undo group again, (don't merge)
-    QCOMPARE(undoManager->undoCount(), 2u);
-    QCOMPARE(undoManager->redoCount(), 0u);
+    QCOMPARE(doc.undoCount(), 2u);
+    QCOMPARE(doc.redoCount(), 0u);
 
     doc.editStart();
-    doc.insertText(Cursor(0, 2), QLatin1String("c"));
+    doc.insertText(Cursor(0, 2), QStringLiteral("c"));
     undoManager->undoSafePoint();
-    doc.insertText(Cursor(0, 3), QLatin1String("d"));
+    doc.insertText(Cursor(0, 3), QStringLiteral("d"));
     doc.editEnd();
 
     // merge both edits into second undo group
-    QCOMPARE(undoManager->undoCount(), 2u);
-    QCOMPARE(undoManager->redoCount(), 0u);
+    QCOMPARE(doc.undoCount(), 2u);
+    QCOMPARE(doc.redoCount(), 0u);
 
-    doc.insertText(Cursor(0, 4), QLatin1String("e"));
+    doc.insertText(Cursor(0, 4), QStringLiteral("e"));
 
     // create a third undo group (don't merge)
-    QCOMPARE(undoManager->undoCount(), 3u);
-    QCOMPARE(undoManager->redoCount(), 0u);
+    QCOMPARE(doc.undoCount(), 3u);
+    QCOMPARE(doc.redoCount(), 0u);
 }
 
 void UndoManagerTest::testCursorPosition()
 {
-    TestDocument doc;
+    KTextEditor::DocumentPrivate doc;
     KTextEditor::ViewPrivate *view = static_cast<KTextEditor::ViewPrivate *>(doc.createView(nullptr));
 
     doc.setText(
-        QLatin1String("aaaa bbbb cccc\n"
-                      "dddd  ffff"));
+        QStringLiteral("aaaa bbbb cccc\n"
+                       "dddd  ffff"));
     view->setCursorPosition(KTextEditor::Cursor(1, 5));
 
-    doc.typeChars(view, QLatin1String("eeee"));
+    doc.typeChars(view, QStringLiteral("eeee"));
 
     // cursor position: "dddd eeee| ffff"
     QCOMPARE(view->cursorPosition(), KTextEditor::Cursor(1, 9));
@@ -173,17 +151,17 @@ void UndoManagerTest::testCursorPosition()
 
 void UndoManagerTest::testSelectionUndo()
 {
-    TestDocument doc;
+    KTextEditor::DocumentPrivate doc;
     KTextEditor::ViewPrivate *view = static_cast<KTextEditor::ViewPrivate *>(doc.createView(nullptr));
 
     doc.setText(
-        QLatin1String("aaaa bbbb cccc\n"
-                      "dddd eeee ffff"));
+        QStringLiteral("aaaa bbbb cccc\n"
+                       "dddd eeee ffff"));
     view->setCursorPosition(KTextEditor::Cursor(1, 9));
     KTextEditor::Range selectionRange(KTextEditor::Cursor(0, 5), KTextEditor::Cursor(1, 9));
     view->setSelection(selectionRange);
 
-    doc.typeChars(view, QLatin1String(QLatin1String("eeee")));
+    doc.typeChars(view, QStringLiteral("eeee"));
 
     // cursor position: "aaaa eeee| ffff", no selection anymore
     QCOMPARE(view->cursorPosition(), KTextEditor::Cursor(0, 9));
@@ -206,7 +184,7 @@ void UndoManagerTest::testSelectionUndo()
 
 void UndoManagerTest::testUndoWordWrapBug301367()
 {
-    TestDocument doc;
+    KTextEditor::DocumentPrivate doc;
     doc.setWordWrap(true);
     doc.setWordWrapAt(20);
     KTextEditor::ViewPrivate *view = static_cast<KTextEditor::ViewPrivate *>(doc.createView(nullptr));
@@ -218,7 +196,7 @@ void UndoManagerTest::testUndoWordWrapBug301367()
     doc.setText(text);
     view->setCursorPosition(KTextEditor::Cursor(0, 0));
 
-    doc.typeChars(view, QLatin1String("           "));
+    doc.typeChars(view, QStringLiteral("           "));
 
     while (doc.undoCount() > 1) {
         doc.undo();
@@ -241,4 +219,34 @@ void UndoManagerTest::testUndoWordWrapBug301367()
     delete view;
 }
 
+void UndoManagerTest::testUndoIndentBug373009()
+{
+    KTextEditor::DocumentPrivate doc;
+    KTextEditor::ViewPrivate *view = static_cast<KTextEditor::ViewPrivate *>(doc.createView(nullptr));
+
+    doc.setMode(QStringLiteral("C"));
+
+    QString text = QString::fromLatin1(
+        "    while (whatever) printf (\"please fix indentation.\\n\");\n"
+        "    return 0;");
+    doc.setText(text);
+
+    // position cursor right before return
+    view->setCursorPosition(KTextEditor::Cursor(1, 4));
+
+    QCOMPARE(view->cursorPosition(), KTextEditor::Cursor(1, 4));
+    QCOMPARE(doc.characterAt(view->cursorPosition()), QLatin1Char('r'));
+
+    view->keyReturn();
+
+    QCOMPARE(doc.undoCount(), 2);
+
+    // After indent we should be able to revert with
+    // one undo operation
+    doc.undo();
+    QCOMPARE(doc.text(), text);
+
+    delete view;
+}
+
 #include "moc_undomanager_test.cpp"
