@@ -11,11 +11,15 @@
 #include "vimode/mappings.h"
 #include <inputmode/kateviinputmode.h>
 #include <kateconfig.h>
+#include <katedocument.h>
 #include <kateundomanager.h>
+#include <kateview.h>
 #include <vimode/emulatedcommandbar/emulatedcommandbar.h>
 
-#include <katedocument.h>
-#include <kateview.h>
+#include <QMainWindow>
+#include <QTest>
+#include <QVBoxLayout>
+#include <inputmodemanager.h>
 
 using namespace KateVi;
 using namespace KTextEditor;
@@ -33,25 +37,25 @@ BaseTest::BaseTest()
     mainWindow->setCentralWidget(centralWidget);
     mainWindow->resize(640, 480);
 
-    m_codesToModifiers.insert("ctrl", CONTROL_MODIFIER);
-    m_codesToModifiers.insert("alt", Qt::AltModifier);
-    m_codesToModifiers.insert("meta", META_MODIFIER);
-    m_codesToModifiers.insert("keypad", Qt::KeypadModifier);
+    m_codesToModifiers.insert(QStringLiteral("ctrl"), CONTROL_MODIFIER);
+    m_codesToModifiers.insert(QStringLiteral("alt"), Qt::AltModifier);
+    m_codesToModifiers.insert(QStringLiteral("meta"), META_MODIFIER);
+    m_codesToModifiers.insert(QStringLiteral("keypad"), Qt::KeypadModifier);
 
-    m_codesToSpecialKeys.insert("backspace", Qt::Key_Backspace);
-    m_codesToSpecialKeys.insert("esc", Qt::Key_Escape);
-    m_codesToSpecialKeys.insert("return", Qt::Key_Return);
-    m_codesToSpecialKeys.insert("enter", Qt::Key_Enter);
-    m_codesToSpecialKeys.insert("left", Qt::Key_Left);
-    m_codesToSpecialKeys.insert("right", Qt::Key_Right);
-    m_codesToSpecialKeys.insert("up", Qt::Key_Up);
-    m_codesToSpecialKeys.insert("down", Qt::Key_Down);
-    m_codesToSpecialKeys.insert("home", Qt::Key_Home);
-    m_codesToSpecialKeys.insert("end", Qt::Key_End);
-    m_codesToSpecialKeys.insert("delete", Qt::Key_Delete);
-    m_codesToSpecialKeys.insert("insert", Qt::Key_Insert);
-    m_codesToSpecialKeys.insert("pageup", Qt::Key_PageUp);
-    m_codesToSpecialKeys.insert("pagedown", Qt::Key_PageDown);
+    m_codesToSpecialKeys.insert(QStringLiteral("backspace"), Qt::Key_Backspace);
+    m_codesToSpecialKeys.insert(QStringLiteral("esc"), Qt::Key_Escape);
+    m_codesToSpecialKeys.insert(QStringLiteral("return"), Qt::Key_Return);
+    m_codesToSpecialKeys.insert(QStringLiteral("enter"), Qt::Key_Enter);
+    m_codesToSpecialKeys.insert(QStringLiteral("left"), Qt::Key_Left);
+    m_codesToSpecialKeys.insert(QStringLiteral("right"), Qt::Key_Right);
+    m_codesToSpecialKeys.insert(QStringLiteral("up"), Qt::Key_Up);
+    m_codesToSpecialKeys.insert(QStringLiteral("down"), Qt::Key_Down);
+    m_codesToSpecialKeys.insert(QStringLiteral("home"), Qt::Key_Home);
+    m_codesToSpecialKeys.insert(QStringLiteral("end"), Qt::Key_End);
+    m_codesToSpecialKeys.insert(QStringLiteral("delete"), Qt::Key_Delete);
+    m_codesToSpecialKeys.insert(QStringLiteral("insert"), Qt::Key_Insert);
+    m_codesToSpecialKeys.insert(QStringLiteral("pageup"), Qt::Key_PageUp);
+    m_codesToSpecialKeys.insert(QStringLiteral("pagedown"), Qt::Key_PageDown);
 }
 
 BaseTest::~BaseTest()
@@ -83,11 +87,15 @@ void BaseTest::init()
     kate_document->config()->setIndentationWidth(2);
     kate_document->config()->setReplaceTabsDyn(false);
 
+    // ensure the spellchecking doesn't mess with the expected results
+    static_cast<KTextEditor::DocumentPrivate *>(kate_document)->setDefaultDictionary(QStringLiteral("notexistinglanguage"));
+
     kate_view = new KTextEditor::ViewPrivate(kate_document, mainWindow);
     mainWindowLayout->addWidget(kate_view);
+    kate_view->config()->setValue(KateViewConfig::AutoBrackets, false);
     kate_view->setInputMode(View::ViInputMode);
     Q_ASSERT(kate_view->currentInputMode()->viewInputMode() == KTextEditor::View::ViInputMode);
-    vi_input_mode = dynamic_cast<KateViInputMode *>(kate_view->currentInputMode());
+    vi_input_mode = static_cast<KateViInputMode *>(kate_view->currentInputMode());
     vi_input_mode_manager = vi_input_mode->viInputModeManager();
     Q_ASSERT(vi_input_mode_manager);
     vi_global = vi_input_mode->globalState();
@@ -95,6 +103,7 @@ void BaseTest::init()
     kate_document->config()->setShowSpaces(KateDocumentConfig::Trailing); // Flush out some issues in the KateRenderer when rendering spaces.
     kate_view->config()->setValue(KateViewConfig::ShowScrollBarMiniMap, false);
     kate_view->config()->setValue(KateViewConfig::ShowScrollBarPreview, false);
+    kate_view->setStatusBarEnabled(false);
 
     connect(kate_document, &KTextEditor::DocumentPrivate::textInsertedRange, this, &BaseTest::textInserted);
     connect(kate_document, &KTextEditor::DocumentPrivate::textRemoved, this, &BaseTest::textRemoved);
@@ -114,7 +123,7 @@ void BaseTest::TestPressKey(const QString &str)
         QString key;
         int keyCode = -1;
         // Looking for keyboard modifiers
-        if (str[i] == QChar('\\')) {
+        if (str[i] == QLatin1Char('\\')) {
             int endOfModifier = -1;
             Qt::KeyboardModifier parsedModifier = parseCodedModifier(str, i, &endOfModifier);
             int endOfSpecialKey = -1;
@@ -127,19 +136,19 @@ void BaseTest::TestPressKey(const QString &str)
                 int endOfSpecialKeyAfterModifier = -1;
                 const Qt::Key parsedCodedSpecialKeyAfterModifier = parseCodedSpecialKey(str, i, &endOfSpecialKeyAfterModifier);
                 if (parsedCodedSpecialKeyAfterModifier != Qt::Key_unknown) {
-                    key = QString(QChar(parsedCodedSpecialKeyAfterModifier));
+                    key = parsedCodedSpecialKeyAfterModifier <= 0xffff ? QString(QChar(parsedCodedSpecialKeyAfterModifier)) : QString();
                     keyCode = parsedCodedSpecialKeyAfterModifier;
                     i = endOfSpecialKeyAfterModifier;
                 }
             } else if (parsedSpecialKey != Qt::Key_unknown) {
-                key = QString(QChar(parsedSpecialKey));
+                key = parsedSpecialKey <= 0xffff ? QString(QChar(parsedSpecialKey)) : QString();
                 keyCode = parsedSpecialKey;
                 i = endOfSpecialKey;
-            } else if (str.mid(i, 2) == QString("\\:")) {
+            } else if (str.mid(i, 2) == QStringLiteral("\\:")) {
                 int start_cmd = i + 2;
                 for (i += 2; true; i++) {
-                    if (str.at(i) == '\\') {
-                        if (i + 1 < str.length() && str.at(i + 1) == '\\') {
+                    if (str.at(i) == QLatin1Char('\\')) {
+                        if (i + 1 < str.length() && str.at(i + 1) == QLatin1Char('\\')) {
                             // A backslash within a command; skip.
                             i += 2;
                         } else {
@@ -148,14 +157,14 @@ void BaseTest::TestPressKey(const QString &str)
                         }
                     }
                 }
-                const QString commandToExecute = str.mid(start_cmd, i - start_cmd).replace("\\\\", "\\");
+                const QString commandToExecute = str.mid(start_cmd, i - start_cmd).replace(QLatin1String("\\\\"), QLatin1String("\\"));
                 qDebug() << "Executing command directly from ViModeTest:\n" << commandToExecute;
                 vi_input_mode->viModeEmulatedCommandBar()->executeCommand(commandToExecute);
                 // We've handled the command; go back round the loop, avoiding sending
                 // the closing \ to vi_input_mode_manager.
                 continue;
-            } else if (str.mid(i, 2) == QString("\\\\")) {
-                key = QString("\\");
+            } else if (str.mid(i, 2) == QStringLiteral("\\\\")) {
+                key = QStringLiteral("\\");
                 keyCode = Qt::Key_Backslash;
                 i++;
             } else {
@@ -178,7 +187,7 @@ void BaseTest::TestPressKey(const QString &str)
             }
         }
 
-        QKeyEvent *key_event = new QKeyEvent(QEvent::KeyPress, keyCode, keyboard_modifier, key);
+        QKeyEvent key_event(QEvent::KeyPress, keyCode, keyboard_modifier, key);
         // Attempt to simulate how Qt usually sends events - typically, we want to send them
         // to kate_view->focusProxy() (which is a KateViewInternal).
         QWidget *destWidget = nullptr;
@@ -194,8 +203,7 @@ void BaseTest::TestPressKey(const QString &str)
         } else {
             destWidget = kate_view->focusProxy();
         }
-        QApplication::postEvent(destWidget, key_event);
-        QApplication::sendPostedEvents();
+        QApplication::sendEvent(destWidget, &key_event);
     }
 }
 
@@ -211,31 +219,44 @@ void BaseTest::BeginTest(const QString &original)
     m_firstBatchOfKeypressesForTest = true;
 }
 
-void BaseTest::FinishTest_(int line, const char *file, const QString &expected, Expectation expectation, const QString &failureReason)
+void BaseTest::FinishTest_(int line, const char *file, const char *expected, Expectation expectation, const char *failureReason)
 {
     if (expectation == ShouldFail) {
-        if (!QTest::qExpectFail("", failureReason.toLocal8Bit().constData(), QTest::Continue, file, line)) {
+        if (!QTest::qExpectFail("", failureReason, QTest::Continue, file, line)) {
             return;
         }
         qDebug() << "Actual text:\n\t" << kate_document->text() << "\nShould be (for this test to pass):\n\t" << expected;
     }
-    if (!QTest::qCompare(kate_document->text(), expected, "kate_document->text()", "expected_text", file, line)) {
+    if (!QTest::qCompare(kate_document->text(), QString::fromUtf8(expected), "kate_document->text()", "expected_text", file, line)) {
         return;
     }
     Q_ASSERT(!emulatedCommandBarTextEdit()->isVisible() && "Make sure you close the command bar before the end of a test!");
 }
 
 void BaseTest::DoTest_(int line,
+                       const char *file,
+                       const char *original,
+                       const char *command,
+                       const char *expected,
+                       Expectation expectation,
+                       const char *failureReason)
+{
+    BeginTest(QLatin1String(original));
+    TestPressKey(QLatin1String(command));
+    FinishTest_(line, file, expected, expectation, failureReason);
+}
+
+void BaseTest::DoTest2(int line,
                        const char *file,
                        const QString &original,
                        const QString &command,
                        const QString &expected,
                        Expectation expectation,
-                       const QString &failureReason)
+                       const char *failureReason)
 {
     BeginTest(original);
     TestPressKey(command);
-    FinishTest_(line, file, expected, expectation, failureReason);
+    FinishTest_(line, file, expected.toUtf8().constData(), expectation, failureReason);
 }
 
 Qt::KeyboardModifier BaseTest::parseCodedModifier(const QString &string, int startPos, int *destEndOfCodedModifier)
@@ -243,11 +264,11 @@ Qt::KeyboardModifier BaseTest::parseCodedModifier(const QString &string, int sta
     for (auto it = m_codesToModifiers.constBegin(), end = m_codesToModifiers.constEnd(); it != end; ++it) {
         const QString &modifierCode = it.key();
         // The "+2" is from the leading '\' and the trailing '-'
-        if (string.mid(startPos, modifierCode.length() + 2) == QString("\\") + modifierCode + "-") {
+        if (string.mid(startPos, modifierCode.length() + 2) == QStringLiteral("\\") + modifierCode + QStringLiteral("-")) {
             if (destEndOfCodedModifier) {
                 // destEndOfCodeModifier lies on the trailing '-'.
                 *destEndOfCodedModifier = startPos + modifierCode.length() + 1;
-                Q_ASSERT(string[*destEndOfCodedModifier] == '-');
+                Q_ASSERT(string[*destEndOfCodedModifier] == QLatin1Char('-'));
             }
             return it.value();
         }
@@ -260,7 +281,7 @@ Qt::Key BaseTest::parseCodedSpecialKey(const QString &string, int startPos, int
     for (auto it = m_codesToSpecialKeys.constBegin(), end = m_codesToSpecialKeys.constEnd(); it != end; ++it) {
         const QString &specialKeyCode = it.key();
         // "+1" is for the leading '\'.
-        if (string.mid(startPos, specialKeyCode.length() + 1) == QString("\\") + specialKeyCode) {
+        if (string.mid(startPos, specialKeyCode.length() + 1) == QStringLiteral("\\") + specialKeyCode) {
             if (destEndOfCodedKey) {
                 *destEndOfCodedKey = startPos + specialKeyCode.length();
             }
@@ -279,7 +300,7 @@ KateVi::EmulatedCommandBar *BaseTest::emulatedCommandBar()
 
 QLineEdit *BaseTest::emulatedCommandBarTextEdit()
 {
-    QLineEdit *emulatedCommandBarText = emulatedCommandBar()->findChild<QLineEdit *>("commandtext");
+    QLineEdit *emulatedCommandBarText = emulatedCommandBar()->findChild<QLineEdit *>(QStringLiteral("commandtext"));
     Q_ASSERT(emulatedCommandBarText);
     return emulatedCommandBarText;
 }
@@ -322,4 +343,5 @@ void BaseTest::textRemoved(Document *document, KTextEditor::Range range)
     m_docChanges.append(DocChange(DocChange::TextRemoved, range));
 }
 
+#include "moc_base.cpp"
 // END: BaseTest
