@@ -7,10 +7,8 @@
 #include "katetextblock.h"
 #include "katetextbuffer.h"
 #include "katetextcursor.h"
-#include "katetextline.h"
 #include "katetextrange.h"
 
-
 namespace Kate
 {
 TextBlock::TextBlock(TextBuffer *buffer, int startLine)
@@ -18,12 +16,13 @@ TextBlock::TextBlock(TextBuffer *buffer, int startLine)
     , m_startLine(startLine)
 {
     // reserve the block size
-    m_lines.reserve(m_buffer->m_blockSize);
+    m_lines.reserve(BufferBlockSize);
 }
 
 TextBlock::~TextBlock()
 {
     // blocks should be empty before they are deleted!
+    Q_ASSERT(m_blockSize == 0);
     Q_ASSERT(m_lines.empty());
     Q_ASSERT(m_cursors.empty());
 
@@ -50,12 +49,14 @@ TextLine TextBlock::line(int line) const
 
 void TextBlock::appendLine(const QString &textOfLine)
 {
-    m_lines.push_back(TextLine::create(textOfLine));
+    m_lines.push_back(std::make_shared<Kate::TextLineData>(textOfLine));
+    m_blockSize += textOfLine.size();
 }
 
 void TextBlock::clearLines()
 {
     m_lines.clear();
+    m_blockSize = 0;
 }
 
 void TextBlock::text(QString &text) const
@@ -369,6 +370,8 @@ void TextBlock::insertText(const KTextEditor::Cursor position, const QString &te
     // notify the text history
     m_buffer->history().insertText(position, text.size(), oldLength);
 
+    m_blockSize += text.size();
+
     // cursor and range handling below
 
     // no cursors in this block, no work to do..
@@ -443,6 +446,8 @@ void TextBlock::removeText(KTextEditor::Range range, QString &removedText)
     // notify the text history
     m_buffer->history().removeText(range, oldLength);
 
+    m_blockSize -= removedText.size();
+
     // cursor and range handling below
 
     // no cursors in this block, no work to do..
@@ -491,7 +496,11 @@ void TextBlock::debugPrint(int blockIndex) const
 {
     // print all blocks
     for (size_t i = 0; i < m_lines.size(); ++i) {
-        printf("%4d - %4lld : %4d : '%s'\n", blockIndex, (unsigned long long)startLine() + i, m_lines.at(i)->text().size(), qPrintable(m_lines.at(i)->text()));
+        printf("%4d - %4llu : %4llu : '%s'\n",
+               blockIndex,
+               (unsigned long long)startLine() + i,
+               (unsigned long long)m_lines.at(i)->text().size(),
+               qPrintable(m_lines.at(i)->text()));
     }
 }
 
@@ -506,8 +515,12 @@ TextBlock *TextBlock::splitBlock(int fromLine)
     // move lines
     newBlock->m_lines.reserve(linesOfNewBlock);
     for (size_t i = fromLine; i < m_lines.size(); ++i) {
-        newBlock->m_lines.push_back(m_lines.at(i));
+        auto line = std::move(m_lines[i]);
+        m_blockSize -= line->length();
+        newBlock->m_blockSize += line->length();
+        newBlock->m_lines.push_back(std::move(line));
     }
+
     m_lines.resize(fromLine);
 
     // move cursors
@@ -530,8 +543,8 @@ TextBlock *TextBlock::splitBlock(int fromLine)
     // copy is necessary as update range may modify the uncached ranges
     std::vector<TextRange *> allRanges;
     allRanges.reserve(m_uncachedRanges.size() + m_cachedLineForRanges.size());
-    std::for_each(m_cachedLineForRanges.begin(), m_cachedLineForRanges.end(), [&allRanges](const std::pair<TextRange *, int> &pair) {
-        allRanges.push_back(pair.first);
+    std::for_each(m_cachedLineForRanges.keyBegin(), m_cachedLineForRanges.keyEnd(), [&allRanges](TextRange *range) {
+        allRanges.push_back(range);
     });
     allRanges.insert(allRanges.end(), m_uncachedRanges.begin(), m_uncachedRanges.end());
     for (TextRange *range : allRanges) {
@@ -559,14 +572,15 @@ void TextBlock::mergeBlock(TextBlock *targetBlock)
     for (size_t i = 0; i < m_lines.size(); ++i) {
         targetBlock->m_lines.push_back(m_lines.at(i));
     }
-    m_lines.clear();
+    targetBlock->m_blockSize += m_blockSize;
+    clearLines();
 
     // fix ALL ranges!
     // copy is necessary as update range may modify the uncached ranges
     std::vector<TextRange *> allRanges;
     allRanges.reserve(m_uncachedRanges.size() + m_cachedLineForRanges.size());
-    std::for_each(m_cachedLineForRanges.begin(), m_cachedLineForRanges.end(), [&allRanges](const std::pair<TextRange *, int> &pair) {
-        allRanges.push_back(pair.first);
+    std::for_each(m_cachedLineForRanges.keyBegin(), m_cachedLineForRanges.keyEnd(), [&allRanges](TextRange *range) {
+        allRanges.push_back(range);
     });
     allRanges.insert(allRanges.end(), m_uncachedRanges.begin(), m_uncachedRanges.end());
     for (TextRange *range : allRanges) {
@@ -597,7 +611,7 @@ void TextBlock::deleteBlockContent()
     }
 
     // kill lines
-    m_lines.clear();
+    clearLines();
 }
 
 void TextBlock::clearBlockContent(TextBlock *targetBlock)
@@ -621,14 +635,22 @@ void TextBlock::clearBlockContent(TextBlock *targetBlock)
     }
 
     // kill lines
-    m_lines.clear();
+    clearLines();
 }
 
-QVector<TextRange *> TextBlock::rangesForLine(int line, KTextEditor::View *view, bool rangesWithAttributeOnly) const
+QList<TextRange *> TextBlock::rangesForLine(int line, KTextEditor::View *view, bool rangesWithAttributeOnly) const
 {
     const auto cachedRanges = cachedRangesForLine(line);
-    QVector<TextRange *> ranges;
-    ranges.reserve(m_uncachedRanges.size() + cachedRanges.size());
+    QList<TextRange *> ranges;
+    ranges.reserve(m_uncachedRanges.size() + (cachedRanges ? cachedRanges->size() : 0));
+    rangesForLine(line, view, rangesWithAttributeOnly, ranges);
+    return ranges;
+}
+
+void TextBlock::rangesForLine(int line, KTextEditor::View *view, bool rangesWithAttributeOnly, QList<TextRange *> &outRanges) const
+{
+    const auto cachedRanges = cachedRangesForLine(line);
+    outRanges.clear();
 
     auto predicate = [line, view, rangesWithAttributeOnly](TextRange *range) {
         if (rangesWithAttributeOnly && !range->hasAttribute()) {
@@ -652,9 +674,10 @@ QVector<TextRange *> TextBlock::rangesForLine(int line, KTextEditor::View *view,
         return false;
     };
 
-    std::copy_if(cachedRanges.begin(), cachedRanges.end(), std::back_inserter(ranges), predicate);
-    std::copy_if(m_uncachedRanges.begin(), m_uncachedRanges.end(), std::back_inserter(ranges), predicate);
-    return ranges;
+    if (cachedRanges) {
+        std::copy_if(cachedRanges->begin(), cachedRanges->end(), std::back_inserter(outRanges), predicate);
+    }
+    std::copy_if(m_uncachedRanges.begin(), m_uncachedRanges.end(), std::back_inserter(outRanges), predicate);
 }
 
 void TextBlock::markModifiedLinesAsSaved()
@@ -683,7 +706,7 @@ void TextBlock::updateRange(TextRange *range)
     // The range is still a single-line range, and is still cached to the correct line.
     if (isSingleLine) {
         auto it = m_cachedLineForRanges.find(range);
-        if (it != m_cachedLineForRanges.end() && it->second == startLine - m_startLine) {
+        if (it != m_cachedLineForRanges.end() && it.value() == startLine - m_startLine) {
             return;
         }
     }
@@ -712,7 +735,9 @@ void TextBlock::updateRange(TextRange *range)
     }
 
     // insert into mapping
-    m_cachedRangesForLine[lineOffset].insert(range);
+    if (!m_cachedRangesForLine[lineOffset].contains(range)) {
+        m_cachedRangesForLine[lineOffset].push_back(range);
+    }
     m_cachedLineForRanges[range] = lineOffset;
 }
 
@@ -733,13 +758,14 @@ void TextBlock::removeRange(TextRange *range)
         // must be only cached!
         Q_ASSERT(!m_uncachedRanges.contains(range));
 
-        int line = it->second;
+        int line = it.value();
 
         // query the range from cache, must be there
-        Q_ASSERT(m_cachedRangesForLine.at(line).contains(range));
+        int idx = m_cachedRangesForLine[line].indexOf(range);
+        Q_ASSERT(idx != -1);
 
         // remove it and be done
-        m_cachedRangesForLine[line].remove(range);
+        m_cachedRangesForLine[line].remove(idx);
         m_cachedLineForRanges.erase(it);
         return;
     }
