@@ -1,35 +1,24 @@
-/*  SPDX-License-Identifier: LGPL-2.0-or-later
-
-    Copyright (C) 2010 Christoph Cullmann <cullmann@kde.org>
+/*
+    SPDX-FileCopyrightText: 2010 Christoph Cullmann <cullmann@kde.org>
 
     Based on code of the SmartCursor/Range by:
-    Copyright (C) 2003-2005 Hamish Rodda <rodda@kde.org>
+    SPDX-FileCopyrightText: 2003-2005 Hamish Rodda <rodda@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KATE_TEXTRANGE_H
 #define KATE_TEXTRANGE_H
 
 #include <ktexteditor/movingrange.h>
-#include <ktexteditor/movingrangefeedback.h>
-#include <ktexteditor/view.h>
 
 #include "katetextcursor.h"
-#include <ktexteditor_export.h>
+
+namespace KTextEditor
+{
+class MovingRangeFeedback;
+class View;
+}
 
 namespace Kate
 {
@@ -43,7 +32,7 @@ class TextBuffer;
  * a TextRange will become automatically invalid as soon as start() == end()
  * position holds.
  */
-class KTEXTEDITOR_EXPORT TextRange : public KTextEditor::MovingRange
+class TextRange final : public KTextEditor::MovingRange
 {
     // this is a friend, block changes might invalidate ranges...
     friend class TextBlock;
@@ -58,13 +47,23 @@ public:
      * @param insertBehavior Define whether the range should expand when text is inserted adjacent to the range.
      * @param emptyBehavior Define whether the range should invalidate itself on becoming empty.
      */
-    TextRange(TextBuffer &buffer, const KTextEditor::Range &range, InsertBehaviors insertBehavior, EmptyBehavior emptyBehavior = AllowEmpty);
+    TextRange(TextBuffer &buffer, KTextEditor::Range range, InsertBehaviors insertBehavior, EmptyBehavior emptyBehavior = AllowEmpty);
+
+    /**
+     * No copy constructor, don't allow this to be copied.
+     */
+    TextRange(const TextRange &) = delete;
 
     /**
      * Destruct the text block
      */
     ~TextRange() override;
 
+    /**
+     * No assignment operator, no copying around.
+     */
+    TextRange &operator=(const TextRange &) = delete;
+
     /**
      * Set insert behaviors.
      * @param insertBehaviors new insert behaviors
@@ -104,7 +103,7 @@ public:
      * automatically invalid!
      * @param range new range for this clever range
      */
-    void setRange(const KTextEditor::Range &range) override;
+    void setRange(KTextEditor::Range range) override;
 
     /**
      * \overload
@@ -114,7 +113,7 @@ public:
      * @param start new start for this clever range
      * @param end new end for this clever range
      */
-    void setRange(const KTextEditor::Cursor &start, const KTextEditor::Cursor &end)
+    void setRange(KTextEditor::Cursor start, KTextEditor::Cursor end)
     {
         KTextEditor::MovingRange::setRange(start, end);
     }
@@ -155,13 +154,23 @@ public:
         return m_end;
     }
 
+    /**
+     * Hides parent's impl of toLineRange() and uses non-virtual functions internally.
+     */
+    KTextEditor::LineRange toLineRange() const
+    {
+        return {startInternal().lineInternal(), endInternal().lineInternal()};
+    }
+
     /**
      * Convert this clever range into a dumb one.
      * @return normal range
      */
     const KTextEditor::Range toRange() const
     {
-        return KTextEditor::Range(start().toCursor(), end().toCursor());
+        auto startCursor = KTextEditor::Cursor(startInternal().lineInternal(), startInternal().columnInternal());
+        auto endCursor = KTextEditor::Cursor(endInternal().lineInternal(), endInternal().columnInternal());
+        return KTextEditor::Range(startCursor, endCursor);
     }
 
     /**
@@ -170,7 +179,7 @@ public:
      */
     operator KTextEditor::Range() const
     {
-        return KTextEditor::Range(start().toCursor(), end().toCursor());
+        return toRange();
     }
 
     /**
@@ -198,7 +207,7 @@ public:
      *
      * \return a pointer to the active attribute
      */
-    KTextEditor::Attribute::Ptr attribute() const override
+    const KTextEditor::Attribute::Ptr &attribute() const override
     {
         return m_attribute;
     }
@@ -280,16 +289,6 @@ public:
     void setZDepth(qreal zDepth) override;
 
 private:
-    /**
-     * no copy constructor, don't allow this to be copied.
-     */
-    TextRange(const TextRange &) = delete;
-
-    /**
-     * no assignment operator, no copying around.
-     */
-    TextRange &operator=(const TextRange &) = delete;
-
     /**
      * Check if range is valid, used by constructor and setRange.
      * If at least one cursor is invalid, both will set to invalid.
@@ -297,20 +296,17 @@ private:
      *
      * IMPORTANT: Notifications might need to deletion of this range!
      *
-     * @param oldStartLine old start line of this range before changing of cursors, needed to add/remove range from m_ranges in blocks
-     * @param oldEndLine old end line of this range
+     * @param oldLineRange line range of this range before changing of cursors, needed to add/remove range from m_ranges in blocks, required!
      * @param notifyAboutChange should feedback be emitted or not?
      */
-    void checkValidity(int oldStartLine = -1, int oldEndLine = -1, bool notifyAboutChange = true);
+    void checkValidity(KTextEditor::LineRange oldLineRange, bool notifyAboutChange = true);
 
     /**
      * Add/Remove range from the lookup m_ranges hash of each block
-     * @param oldStartLine old start line of this range before changing of cursors, needed to add/remove range from m_ranges in blocks
-     * @param oldEndLine old end line of this range
-     * @param startLine start line to start looking for the range to remove
-     * @param endLine end line of this range
+     * @param oldLineRange old line range before changing of cursors, needed to add/remove range from m_ranges in blocks
+     * @param lineRange line range to start looking for the range to remove
      */
-    void fixLookup(int oldStartLine, int oldEndLine, int startLine, int endLine);
+    void fixLookup(KTextEditor::LineRange oldLineRange, KTextEditor::LineRange lineRange);
 
     /**
      * Mark this range for later validity checking.
