@@ -17,6 +17,8 @@
 #include "expandingtree/expandingwidgetmodel.h"
 #include <ktexteditor_export.h>
 
+#include <set>
+
 class KateCompletionWidget;
 class KateArgumentHintModel;
 namespace KTextEditor
@@ -34,7 +36,7 @@ class HierarchicalModelHandler;
  *
  * @author Hamish Rodda <rodda@kde.org>
  */
-class KTEXTEDITOR_EXPORT KateCompletionModel : public ExpandingWidgetModel
+class KateCompletionModel : public ExpandingWidgetModel
 {
     Q_OBJECT
 
@@ -48,16 +50,16 @@ public:
 
     QList<KTextEditor::CodeCompletionModel *> completionModels() const;
     void clearCompletionModels();
-    void addCompletionModel(KTextEditor::CodeCompletionModel *model);
-    void setCompletionModel(KTextEditor::CodeCompletionModel *model);
+    KTEXTEDITOR_EXPORT void addCompletionModel(KTextEditor::CodeCompletionModel *model);
+    KTEXTEDITOR_EXPORT void setCompletionModel(KTextEditor::CodeCompletionModel *model);
     void setCompletionModels(const QList<KTextEditor::CodeCompletionModel *> &models);
-    void removeCompletionModel(KTextEditor::CodeCompletionModel *model);
+    KTEXTEDITOR_EXPORT void removeCompletionModel(KTextEditor::CodeCompletionModel *model);
 
     KTextEditor::ViewPrivate *view() const;
     KateCompletionWidget *widget() const;
 
-    QString currentCompletion(KTextEditor::CodeCompletionModel *model) const;
-    void setCurrentCompletion(KTextEditor::CodeCompletionModel *model, const QString &completion);
+    KTEXTEDITOR_EXPORT QString currentCompletion(KTextEditor::CodeCompletionModel *model) const;
+    void setCurrentCompletion(QMap<KTextEditor::CodeCompletionModel *, QString> currentMatch);
 
     int translateColumn(int sourceColumn) const;
 
@@ -65,7 +67,7 @@ public:
     /// If there is no common prefix, extracts the next useful prefix for the selected index
     QString commonPrefix(QModelIndex selectedIndex) const;
 
-    void rowSelected(const QModelIndex &row);
+    void rowSelected(const QModelIndex &row) const;
 
     bool indexIsItem(const QModelIndex &index) const override;
 
@@ -81,19 +83,12 @@ public:
     QModelIndex parent(const QModelIndex &index) const override;
     int rowCount(const QModelIndex &parent = QModelIndex()) const override;
 
-    // Disabled in case of bugs, reenable once fully debugged.
-    // virtual QModelIndex sibling ( int row, int column, const QModelIndex & index ) const;
-    void sort(int column, Qt::SortOrder order = Qt::AscendingOrder) override;
-
     /// Maps from this display-model into the appropriate source code-completion model
     virtual QModelIndex mapToSource(const QModelIndex &proxyIndex) const;
 
     /// Maps from an index in a source-model to the index of the item in this display-model
     virtual QModelIndex mapFromSource(const QModelIndex &sourceIndex) const;
 
-    // Grouping
-    bool isGroupingEnabled() const;
-
     enum gm { ScopeType = 0x1, Scope = 0x2, AccessType = 0x4, ItemType = 0x8 };
 
     enum { // An own property that will be used to mark the best-matches group internally
@@ -106,28 +101,12 @@ public:
     static const int AccessTypeMask = 0x7;
     static const int ItemTypeMask = 0xfe0;
 
-    GroupingMethods groupingMethod() const;
-    void setGroupingMethod(GroupingMethods m);
-
-    bool accessIncludeConst() const;
-    void setAccessIncludeConst(bool include);
-    bool accessIncludeStatic() const;
-    void setAccessIncludeStatic(bool include);
-    bool accessIncludeSignalSlot() const;
-    void setAccessIncludeSignalSlot(bool include);
-
-    // Column merging
-    bool isColumnMergingEnabled() const;
-
-    const QList<QList<int>> &columnMerges() const;
-    void setColumnMerges(const QList<QList<int>> &columnMerges);
-
     void debugStats();
 
     /// Returns whether one of the filtered items exactly matches its completion string
     bool shouldMatchHideCompletionList() const;
 
-    uint filteredItemCount() const;
+    KTEXTEDITOR_EXPORT uint filteredItemCount() const;
 
 protected:
     int contextMatchQuality(const QModelIndex &index) const override;
@@ -137,10 +116,6 @@ Q_SIGNALS:
     // Emitted whenever something has changed about the group of argument-hints
     void argumentHintsChanged();
 
-public Q_SLOTS:
-    void setGroupingEnabled(bool enable);
-    void setColumnMergingEnabled(bool enable);
-
 private Q_SLOTS:
     void slotRowsInserted(const QModelIndex &parent, int start, int end);
     void slotRowsRemoved(const QModelIndex &parent, int start, int end);
@@ -159,7 +134,7 @@ private:
     QTreeView *treeView() const override;
 
     friend class KateArgumentHintModel;
-    ModelRow modelRowPair(const QModelIndex &index) const;
+    static ModelRow modelRowPair(const QModelIndex &index);
 
     // Represents a source row; provides sorting method
     class Item
@@ -167,42 +142,31 @@ private:
     public:
         Item(bool doInitialMatch, KateCompletionModel *model, const HierarchicalModelHandler &handler, ModelRow sourceRow);
 
-        bool isValid() const;
         // Returns true if the item is not filtered and matches the current completion string
         bool isVisible() const;
-        // Returns whether the item is filtered or not
-        bool isFiltered() const;
-        // Returns whether the item matches the current completion string
-        bool isMatching() const;
 
         enum MatchType { NoMatch = 0, PerfectMatch, StartsWithMatch, AbbreviationMatch, ContainsMatch };
-        MatchType match();
+        MatchType match(KateCompletionModel *model);
 
         const ModelRow &sourceRow() const;
 
         // Sorting operator
-        bool operator<(const Item &rhs) const;
+        bool lessThan(KateCompletionModel *model, const Item &rhs) const;
 
         bool haveExactMatch() const
         {
             return m_haveExactMatch;
         }
 
-        void clearExactMatch()
-        {
-            m_haveExactMatch = false;
-        }
-
         QString name() const
         {
             return m_nameColumn;
         }
 
     private:
-        KateCompletionModel *model;
         ModelRow m_sourceRow;
 
-        mutable QString m_nameColumn;
+        QString m_nameColumn;
 
         int inheritanceDepth;
 
@@ -252,7 +216,14 @@ public:
         int customSortingKey;
     };
 
-    bool hasGroups() const;
+    typedef std::set<Group *> GroupSet;
+
+    bool hasGroups() const
+    {
+        // qCDebug(LOG_KTE) << "m_groupHash.size()"<<m_groupHash.size();
+        // qCDebug(LOG_KTE) << "m_rowTable.count()"<<m_rowTable.count();
+        return m_hasGroups;
+    }
 
 private:
     QString commonPrefixInternal(const QString &forcePrefix) const;
@@ -260,16 +231,16 @@ private:
     void createGroups();
     /// Creates all sub-items of index i, or the item corresponding to index i. Returns the affected groups.
     /// i must be an index in the source model
-    QSet<Group *> createItems(const HierarchicalModelHandler &, const QModelIndex &i, bool notifyModel = false);
+    GroupSet createItems(const HierarchicalModelHandler &, const QModelIndex &i, bool notifyModel = false);
     /// Deletes all sub-items of index i, or the item corresponding to index i. Returns the affected groups.
     /// i must be an index in the source model
-    QSet<Group *> deleteItems(const QModelIndex &i);
+    GroupSet deleteItems(const QModelIndex &i);
     Group *createItem(const HierarchicalModelHandler &, const QModelIndex &i, bool notifyModel = false);
     /// @note Make sure you're in a {begin,end}ResetModel block when calling this!
     void clearGroups();
     void hideOrShowGroup(Group *g, bool notifyModel = false);
     /// When forceGrouping is enabled, all given attributes will be used for grouping, regardless of the completion settings.
-    Group *fetchGroup(int attribute, const QString &scope = QString(), bool forceGrouping = false);
+    Group *fetchGroup(int attribute, bool forceGrouping = false);
     // If this returns nonzero on an index, the index is the header of the returned group
     Group *groupForIndex(const QModelIndex &index) const;
     inline Group *groupOfParent(const QModelIndex &child) const
@@ -282,17 +253,18 @@ private:
     enum changeTypes { Broaden, Narrow, Change };
 
     // Returns whether the model needs to be reset
-    void changeCompletions(Group *g, changeTypes changeType, bool notifyModel);
+    void changeCompletions(Group *g);
 
     bool hasCompletionModel() const;
 
     /// Removes attributes not used in grouping from the input \a attribute
     int groupingAttributes(int attribute) const;
-    int countBits(int value) const;
+    static int countBits(int value);
 
     void resort();
 
-    static bool matchesAbbreviation(const QString &word, const QString &typed, int &score);
+    KTEXTEDITOR_EXPORT static bool matchesAbbreviation(const QString &word, const QString &typed, int &score);
+    // exported for completion_test
 
     bool m_hasGroups = false;
 
@@ -302,7 +274,11 @@ private:
     QMap<KTextEditor::CodeCompletionModel *, QString> m_currentMatch;
 
     // Column merging
-    QList<QList<int>> m_columnMerges;
+    const std::array<std::vector<int>, 3> m_columnMerges = {{
+        {0},
+        {1, 2, 3, 4},
+        {5},
+    }};
 
     QTimer *m_updateBestMatchesTimer;
 
@@ -311,21 +287,13 @@ private:
     Group *m_bestMatches; // A temporary group used for holding the best matches of all visible items
 
     // Storing the sorted order
-    QList<Group *> m_rowTable;
-    QList<Group *> m_emptyGroups;
+    std::vector<Group *> m_rowTable;
+    std::vector<Group *> m_emptyGroups;
     // Quick access to each specific group (if it exists)
     QMultiHash<int, Group *> m_groupHash;
     // Maps custom group-names to their specific groups
     QHash<QString, Group *> m_customGroupHash;
 
-    // Grouping
-    bool m_groupingEnabled = false;
-    GroupingMethods m_groupingMethod;
-    bool m_accessConst = false, m_accessStatic = false, m_accesSignalSlot = false;
-
-    // Column merging
-    bool m_columnMergingEnabled = false /*, m_haveExactMatch*/;
-
     friend class CompletionTest;
 };
 
