@@ -9,7 +9,6 @@
 
 #include <ktexteditor/codecompletionmodelcontrollerinterface.h>
 
-#include "katebuffer.h"
 #include "kateconfig.h"
 #include "katedocument.h"
 #include "katerenderer.h"
@@ -41,14 +40,14 @@ const bool hideAutomaticCompletionOnExactMatch = true;
 #define CALLCI(WHAT, WHATELSE, WHAT2, model, FUNC)                                                                                                             \
     {                                                                                                                                                          \
         static KTextEditor::CodeCompletionModelControllerInterface defaultIf;                                                                                  \
-        KTextEditor::CodeCompletionModelControllerInterface *ret = dynamic_cast<KTextEditor::CodeCompletionModelControllerInterface *>(model);                 \
+        KTextEditor::CodeCompletionModelControllerInterface *ret = qobject_cast<KTextEditor::CodeCompletionModelControllerInterface *>(model);                 \
         if (!ret) {                                                                                                                                            \
             WHAT2 defaultIf.FUNC;                                                                                                                              \
         } else                                                                                                                                                 \
             WHAT2 ret->FUNC;                                                                                                                                   \
     }
 
-static KTextEditor::Range _completionRange(KTextEditor::CodeCompletionModel *model, KTextEditor::View *view, const KTextEditor::Cursor &cursor)
+static KTextEditor::Range _completionRange(KTextEditor::CodeCompletionModel *model, KTextEditor::View *view, KTextEditor::Cursor cursor)
 {
     CALLCI(return, , return, model, completionRange(view, cursor));
 }
@@ -58,8 +57,7 @@ static KTextEditor::Range _updateRange(KTextEditor::CodeCompletionModel *model,
     CALLCI(, return range, return, model, updateCompletionRange(view, range));
 }
 
-static QString
-_filterString(KTextEditor::CodeCompletionModel *model, KTextEditor::View *view, const KTextEditor::Range &range, const KTextEditor::Cursor &cursor)
+static QString _filterString(KTextEditor::CodeCompletionModel *model, KTextEditor::View *view, const KTextEditor::Range &range, KTextEditor::Cursor cursor)
 {
     CALLCI(return, , return, model, filterString(view, range, cursor));
 }
@@ -79,7 +77,7 @@ static bool _shouldStartCompletion(KTextEditor::CodeCompletionModel *model,
                                    KTextEditor::View *view,
                                    const QString &automaticInvocationLine,
                                    bool m_lastInsertionByUser,
-                                   const KTextEditor::Cursor &cursor)
+                                   KTextEditor::Cursor cursor)
 {
     CALLCI(return, , return, model, shouldStartCompletion(view, automaticInvocationLine, m_lastInsertionByUser, cursor));
 }
@@ -92,7 +90,6 @@ KateCompletionWidget::KateCompletionWidget(KTextEditor::ViewPrivate *parent)
     , m_argumentHintTree(new KateArgumentHintTree(this))
     , m_docTip(new DocTip(this))
     , m_automaticInvocationDelay(100)
-    , m_filterInstalled(false)
     , m_lastInsertionByUser(false)
     , m_inCompletionList(false)
     , m_isSuspended(false)
@@ -149,10 +146,10 @@ KateCompletionWidget::KateCompletionWidget(KTextEditor::ViewPrivate *parent)
     });
 
     // connect to all possible editing primitives
-    connect(&view()->doc()->buffer(), &KateBuffer::lineWrapped, this, &KateCompletionWidget::wrapLine);
-    connect(&view()->doc()->buffer(), &KateBuffer::lineUnwrapped, this, &KateCompletionWidget::unwrapLine);
-    connect(&view()->doc()->buffer(), &KateBuffer::textInserted, this, &KateCompletionWidget::insertText);
-    connect(&view()->doc()->buffer(), &KateBuffer::textRemoved, this, &KateCompletionWidget::removeText);
+    connect(view()->doc(), &KTextEditor::Document::lineWrapped, this, &KateCompletionWidget::wrapLine);
+    connect(view()->doc(), &KTextEditor::Document::lineUnwrapped, this, &KateCompletionWidget::unwrapLine);
+    connect(view()->doc(), &KTextEditor::Document::textInserted, this, &KateCompletionWidget::insertText);
+    connect(view()->doc(), &KTextEditor::Document::textRemoved, this, &KateCompletionWidget::removeText);
 
     // This is a non-focus widget, it is passed keyboard input from the view
 
@@ -363,14 +360,15 @@ void KateCompletionWidget::startCompletion(KTextEditor::Range word,
         }
     }
 
-    if (!m_filterInstalled) {
-        if (!QApplication::activeWindow()) {
-            qCWarning(LOG_KTE) << "No active window to install event filter on!!";
-            return;
+    // Enable the cc box to move when the editor window is moved
+    if (auto viewWindow = view()->window(); m_windowToMoveWith != viewWindow) {
+        if (m_windowToMoveWith) {
+            m_windowToMoveWith->removeEventFilter(this);
+        }
+        m_windowToMoveWith = viewWindow;
+        if (m_windowToMoveWith) {
+            m_windowToMoveWith->installEventFilter(this);
         }
-        // Enable the cc box to move when the editor window is moved
-        QApplication::activeWindow()->installEventFilter(this);
-        m_filterInstalled = true;
     }
 
     m_presentationModel->clearCompletionModels();
@@ -924,7 +922,7 @@ bool KateCompletionWidget::execute()
     afterTailMCursor->move(tailStr.size());
 
     // Handle completion for multi cursors
-    QSharedPointer<QMetaObject::Connection> connection(new QMetaObject::Connection());
+    std::shared_ptr<QMetaObject::Connection> connection(new QMetaObject::Connection());
     auto autoCompleteMulticursors = [connection, this](KTextEditor::Document *document, const KTextEditor::Range &range) {
         disconnect(*connection);
         const QString text = document->text(range);
@@ -1323,7 +1321,7 @@ void KateCompletionWidget::completionModelReset()
 
 void KateCompletionWidget::modelDestroyed(QObject *model)
 {
-    m_sourceModels.removeAll(static_cast<KTextEditor::CodeCompletionModel *>(model));
+    m_sourceModels.removeAll(model);
     abortCompletion();
 }
 
@@ -1371,15 +1369,15 @@ int KateCompletionWidget::automaticInvocationDelay() const
 void KateCompletionWidget::setIgnoreBufferSignals(bool ignore) const
 {
     if (ignore) {
-        disconnect(&view()->doc()->buffer(), &KateBuffer::lineWrapped, this, &KateCompletionWidget::wrapLine);
-        disconnect(&view()->doc()->buffer(), &KateBuffer::lineUnwrapped, this, &KateCompletionWidget::unwrapLine);
-        disconnect(&view()->doc()->buffer(), &KateBuffer::textInserted, this, &KateCompletionWidget::insertText);
-        disconnect(&view()->doc()->buffer(), &KateBuffer::textRemoved, this, &KateCompletionWidget::removeText);
+        disconnect(view()->doc(), &KTextEditor::Document::lineWrapped, this, &KateCompletionWidget::wrapLine);
+        disconnect(view()->doc(), &KTextEditor::Document::lineUnwrapped, this, &KateCompletionWidget::unwrapLine);
+        disconnect(view()->doc(), &KTextEditor::Document::textInserted, this, &KateCompletionWidget::insertText);
+        disconnect(view()->doc(), &KTextEditor::Document::textRemoved, this, &KateCompletionWidget::removeText);
     } else {
-        connect(&view()->doc()->buffer(), &KateBuffer::lineWrapped, this, &KateCompletionWidget::wrapLine);
-        connect(&view()->doc()->buffer(), &KateBuffer::lineUnwrapped, this, &KateCompletionWidget::unwrapLine);
-        connect(&view()->doc()->buffer(), &KateBuffer::textInserted, this, &KateCompletionWidget::insertText);
-        connect(&view()->doc()->buffer(), &KateBuffer::textRemoved, this, &KateCompletionWidget::removeText);
+        connect(view()->doc(), &KTextEditor::Document::lineWrapped, this, &KateCompletionWidget::wrapLine);
+        connect(view()->doc(), &KTextEditor::Document::lineUnwrapped, this, &KateCompletionWidget::unwrapLine);
+        connect(view()->doc(), &KTextEditor::Document::textInserted, this, &KateCompletionWidget::insertText);
+        connect(view()->doc(), &KTextEditor::Document::textRemoved, this, &KateCompletionWidget::removeText);
     }
 }
 
@@ -1388,7 +1386,7 @@ void KateCompletionWidget::setAutomaticInvocationDelay(int delay)
     m_automaticInvocationDelay = delay;
 }
 
-void KateCompletionWidget::wrapLine(const KTextEditor::Cursor &)
+void KateCompletionWidget::wrapLine(KTextEditor::Document *, KTextEditor::Cursor)
 {
     m_lastInsertionByUser = !m_completionEditRunning;
 
@@ -1397,7 +1395,7 @@ void KateCompletionWidget::wrapLine(const KTextEditor::Cursor &)
     m_automaticInvocationTimer->stop();
 }
 
-void KateCompletionWidget::unwrapLine(int)
+void KateCompletionWidget::unwrapLine(KTextEditor::Document *, int)
 {
     m_lastInsertionByUser = !m_completionEditRunning;
 
@@ -1406,7 +1404,7 @@ void KateCompletionWidget::unwrapLine(int)
     m_automaticInvocationTimer->stop();
 }
 
-void KateCompletionWidget::insertText(const KTextEditor::Cursor &position, const QString &text)
+void KateCompletionWidget::insertText(KTextEditor::Document *, KTextEditor::Cursor position, const QString &text)
 {
     m_lastInsertionByUser = !m_completionEditRunning;
 
@@ -1434,7 +1432,7 @@ void KateCompletionWidget::insertText(const KTextEditor::Cursor &position, const
     m_automaticInvocationTimer->start(m_automaticInvocationDelay);
 }
 
-void KateCompletionWidget::removeText(const KTextEditor::Range &)
+void KateCompletionWidget::removeText(KTextEditor::Document *, KTextEditor::Range, const QString &)
 {
     m_lastInsertionByUser = !m_completionEditRunning;
 
@@ -1500,3 +1498,5 @@ void KateCompletionWidget::tabCompletion(Direction direction)
         }
     }
 }
+
+#include "moc_katecompletionwidget.cpp"
