@@ -10,10 +10,10 @@
 
 #include <QFrame>
 #include <QObject>
+#include <QPointer>
 
 #include <ktexteditor_export.h>
 
-#include <ktexteditor/codecompletioninterface.h>
 #include <ktexteditor/codecompletionmodel.h>
 #include <ktexteditor/movingrange.h>
 
@@ -67,6 +67,13 @@ public Q_SLOTS:
     void cursorUp();
 
 public:
+    enum Direction {
+        Down,
+        Up,
+    };
+
+    void tabCompletion(Direction direction = Down);
+
     void toggleDocumentation();
 
     const KateCompletionModel *model() const;
@@ -80,7 +87,7 @@ public:
     int automaticInvocationDelay() const;
     void setAutomaticInvocationDelay(int delay);
 
-    void setIgnoreBufferSignals(bool ignore);
+    void setIgnoreBufferSignals(bool ignore) const;
 
     bool m_ignoreBufferSignals = false;
 
@@ -163,25 +170,34 @@ private Q_SLOTS:
     void rowsInserted(const QModelIndex &parent, int row, int rowEnd);
     void viewFocusOut();
 
-    void wrapLine(const KTextEditor::Cursor &position);
-    void unwrapLine(int line);
-    void insertText(const KTextEditor::Cursor &position, const QString &text);
-    void removeText(const KTextEditor::Range &range);
+    void wrapLine(KTextEditor::Document *document, KTextEditor::Cursor position);
+    void unwrapLine(KTextEditor::Document *, int line);
+    void insertText(KTextEditor::Document *, KTextEditor::Cursor position, const QString &text);
+    void removeText(KTextEditor::Document *, KTextEditor::Range range, const QString &);
 
 private:
+    KTEXTEDITOR_NO_EXPORT
     void updateAndShow();
+    KTEXTEDITOR_NO_EXPORT
     void updateArgumentHintGeometry();
+    KTEXTEDITOR_NO_EXPORT
     QModelIndex selectedIndex() const;
 
+    KTEXTEDITOR_NO_EXPORT
     void clear();
     // Switch cursor between argument-hint list / completion-list
+    KTEXTEDITOR_NO_EXPORT
     void switchList();
+    KTEXTEDITOR_NO_EXPORT
     void completionRangeChanged(KTextEditor::CodeCompletionModel *, const KTextEditor::Range &word);
 
+    KTEXTEDITOR_NO_EXPORT
     QString tailString() const;
 
+    KTEXTEDITOR_NO_EXPORT
     void deleteCompletionRanges();
 
+private:
     QList<KTextEditor::CodeCompletionModel *> m_sourceModels;
     KateCompletionModel *m_presentationModel;
 
@@ -200,7 +216,8 @@ private:
     KTextEditor::Cursor m_automaticInvocationAt;
     QString m_automaticInvocationLine;
     int m_automaticInvocationDelay;
-    bool m_filterInstalled;
+
+    QPointer<QWidget> m_windowToMoveWith;
 
     bool m_lastInsertionByUser;
     bool m_inCompletionList; // Are we in the completion-list? If not, we're in the argument-hint list
