@@ -25,6 +25,8 @@
 #include <KParts/Part>
 #include <KToggleAction>
 
+#include <Sonnet/Speller>
+
 #include <QAction>
 #include <QCheckBox>
 #include <QLabel>
@@ -36,8 +38,8 @@
 
 // END
 
-/// Amount of characters the document may have to enable automatic invocation (1MB)
-static const int autoInvocationMaxFilesize = 1000000;
+/// amount of lines to scan backwards and forwards
+static const int maxLinesToScan = 10000;
 
 // BEGIN KateWordCompletionModel
 KateWordCompletionModel::KateWordCompletionModel(QObject *parent)
@@ -142,11 +144,6 @@ bool KateWordCompletionModel::shouldStartCompletion(KTextEditor::View *view,
 
     KTextEditor::ViewPrivate *v = qobject_cast<KTextEditor::ViewPrivate *>(view);
 
-    if (view->document()->totalCharacters() > autoInvocationMaxFilesize) {
-        // Disable automatic invocation for files larger than 1MB (see benchmarks)
-        return false;
-    }
-
     const QString &text = view->document()->line(position.line()).left(position.column());
     const uint check = v->config()->wordCompletionMinimalWordLength();
     // Start completion immediately if min. word size is zero
@@ -192,25 +189,34 @@ void KateWordCompletionModel::completionInvoked(KTextEditor::View *view, const K
  * ignoring any dublets and words shorter than configured and/or
  * reasonable minimum length.
  */
-QStringList KateWordCompletionModel::allMatches(KTextEditor::View *view, const KTextEditor::Range &range) const
+QStringList KateWordCompletionModel::allMatches(KTextEditor::View *view, const KTextEditor::Range &range)
 {
-    QSet<QString> result;
+    QSet<QStringView> result;
     const int minWordSize = qMax(2, qobject_cast<KTextEditor::ViewPrivate *>(view)->config()->wordCompletionMinimalWordLength());
-    const int lines = view->document()->lines();
-    for (int line = 0; line < lines; line++) {
-        const QString &text = view->document()->line(line);
+    const auto cursorPosition = view->cursorPosition();
+    const auto document = view->document();
+    const int startLine = std::max(0, cursorPosition.line() - maxLinesToScan);
+    const int endLine = std::min(cursorPosition.line() + maxLinesToScan, view->document()->lines());
+    for (int line = startLine; line < endLine; line++) {
+        const QString text = document->line(line);
+        if (text.isEmpty() || text.isNull()) {
+            continue;
+        }
+        QStringView textView = text;
         int wordBegin = 0;
         int offset = 0;
         const int end = text.size();
-        const bool cursorLine = view->cursorPosition().line() == line;
+        const bool cursorLine = cursorPosition.line() == line;
+        const bool isNotLastLine = line != range.end().line();
+        const QChar *d = text.data();
         while (offset < end) {
-            const QChar c = text.at(offset);
+            const QChar c = d[offset];
             // increment offset when at line end, so we take the last character too
-            if ((!c.isLetterOrNumber() && c != QLatin1Char('_')) || (offset == end - 1 && offset++)) {
-                if (offset - wordBegin > minWordSize && (line != range.end().line() || offset != range.end().column())) {
+            if ((!c.isLetterOrNumber() && c != QChar(u'_')) || (offset == end - 1 && offset++)) {
+                if (offset - wordBegin >= minWordSize && (isNotLastLine || offset != range.end().column())) {
                     // don't add the word we are inside with cursor!
-                    if (!cursorLine || (view->cursorPosition().column() < wordBegin || view->cursorPosition().column() > offset)) {
-                        result.insert(text.mid(wordBegin, offset - wordBegin));
+                    if (!cursorLine || (cursorPosition.column() < wordBegin || cursorPosition.column() > offset)) {
+                        result.insert(textView.mid(wordBegin, offset - wordBegin));
                     }
                 }
                 wordBegin = offset + 1;
@@ -221,38 +227,36 @@ QStringList KateWordCompletionModel::allMatches(KTextEditor::View *view, const K
             offset += 1;
         }
     }
-    return result.values();
+
+    // ensure words that are ok spell check wise always end up in the completion, see bug 468705
+    const auto language = static_cast<KTextEditor::DocumentPrivate *>(document)->defaultDictionary();
+    const auto word = view->document()->text(range);
+    Sonnet::Speller speller;
+    QStringList spellerSuggestions;
+    speller.setLanguage(language);
+    if (speller.isValid()) {
+        if (speller.isCorrect(word)) {
+            result.insert(word);
+        } else {
+            spellerSuggestions = speller.suggest(word);
+            for (const auto &alternative : std::as_const(spellerSuggestions)) {
+                result.insert(alternative);
+            }
+        }
+    }
+
+    m_matches.clear();
+    m_matches.reserve(result.size());
+    for (auto v : std::as_const(result)) {
+        m_matches << v.toString();
+    }
+
+    return m_matches;
 }
 
 void KateWordCompletionModel::executeCompletionItem(KTextEditor::View *view, const KTextEditor::Range &word, const QModelIndex &index) const
 {
-    KTextEditor::ViewPrivate *v = qobject_cast<KTextEditor::ViewPrivate *>(view);
-    if (v->config()->wordCompletionRemoveTail()) {
-        int tailStart = word.end().column();
-        const QString &line = view->document()->line(word.end().line());
-        int tailEnd = line.length();
-        for (int i = word.end().column(); i < tailEnd; ++i) {
-            // Letters, numbers and underscore are part of a word!
-            /// \todo Introduce configurable \e word-separators??
-            if (!line[i].isLetterOrNumber() && line[i] != QLatin1Char('_')) {
-                tailEnd = i;
-            }
-        }
-
-        int sizeDiff = m_matches.at(index.row()).size() - (word.end().column() - word.start().column());
-
-        tailStart += sizeDiff;
-        tailEnd += sizeDiff;
-
-        KTextEditor::Range tail(KTextEditor::Cursor(word.start().line(), tailStart), KTextEditor::Cursor(word.end().line(), tailEnd));
-
-        view->document()->replaceText(word, m_matches.at(index.row()));
-        v->doc()->editEnd();
-        v->doc()->editStart();
-        view->document()->replaceText(tail, QString());
-    } else {
-        view->document()->replaceText(word, m_matches.at(index.row()));
-    }
+    view->document()->replaceText(word, m_matches.at(index.row()));
 }
 
 KTextEditor::CodeCompletionModelControllerInterface::MatchReaction KateWordCompletionModel::matchingItem(const QModelIndex & /*matched*/)
@@ -267,25 +271,6 @@ bool KateWordCompletionModel::shouldHideItemsWithEqualNames() const
     return true;
 }
 
-// Return the range containing the word left of the cursor
-KTextEditor::Range KateWordCompletionModel::completionRange(KTextEditor::View *view, const KTextEditor::Cursor &position)
-{
-    int line = position.line();
-    int col = position.column();
-
-    KTextEditor::Document *doc = view->document();
-    while (col > 0) {
-        const QChar c = (doc->characterAt(KTextEditor::Cursor(line, col - 1)));
-        if (c.isLetterOrNumber() || c.isMark() || c == QLatin1Char('_')) {
-            col--;
-            continue;
-        }
-
-        break;
-    }
-
-    return KTextEditor::Range(KTextEditor::Cursor(line, col), position);
-}
 // END KateWordCompletionModel
 
 // BEGIN KateWordCompletionView
@@ -315,20 +300,21 @@ KateWordCompletionView::KateWordCompletionView(KTextEditor::View *view, KActionC
 
     QAction *action;
 
-    if (qobject_cast<KTextEditor::CodeCompletionInterface *>(view)) {
-        action = new QAction(i18n("Shell Completion"), this);
-        ac->addAction(QStringLiteral("doccomplete_sh"), action);
-        connect(action, &QAction::triggered, this, &KateWordCompletionView::shellComplete);
-    }
+    action = new QAction(i18n("Shell Completion"), this);
+    ac->addAction(QStringLiteral("doccomplete_sh"), action);
+    action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+    connect(action, &QAction::triggered, this, &KateWordCompletionView::shellComplete);
 
     action = new QAction(i18n("Reuse Word Above"), this);
     ac->addAction(QStringLiteral("doccomplete_bw"), action);
-    ac->setDefaultShortcut(action, Qt::CTRL + Qt::Key_8);
+    ac->setDefaultShortcut(action, Qt::CTRL | Qt::Key_8);
+    action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
     connect(action, &QAction::triggered, this, &KateWordCompletionView::completeBackwards);
 
     action = new QAction(i18n("Reuse Word Below"), this);
     ac->addAction(QStringLiteral("doccomplete_fw"), action);
-    ac->setDefaultShortcut(action, Qt::CTRL + Qt::Key_9);
+    ac->setDefaultShortcut(action, Qt::CTRL | Qt::Key_9);
+    action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
     connect(action, &QAction::triggered, this, &KateWordCompletionView::completeForwards);
 }
 
@@ -353,8 +339,7 @@ void KateWordCompletionView::popupCompletionList()
     qCDebug(LOG_KTE) << "entered ...";
     KTextEditor::Range r = range();
 
-    KTextEditor::CodeCompletionInterface *cci = qobject_cast<KTextEditor::CodeCompletionInterface *>(m_view);
-    if (!cci || cci->isCompletionActive()) {
+    if (m_view->isCompletionActive()) {
         return;
     }
 
@@ -366,7 +351,7 @@ void KateWordCompletionView::popupCompletionList()
         return;
     }
 
-    cci->startCompletion(r, m_dWCompletionModel);
+    m_view->startCompletion(r, m_dWCompletionModel);
 }
 
 // Contributed by <brain@hdsnet.hu>
@@ -374,7 +359,7 @@ void KateWordCompletionView::shellComplete()
 {
     KTextEditor::Range r = range();
 
-    QStringList matches = m_dWCompletionModel->allMatches(m_view, r);
+    const QStringList matches = m_dWCompletionModel->allMatches(m_view, r);
 
     if (matches.size() == 0) {
         return;
@@ -450,7 +435,7 @@ void KateWordCompletionView::complete(bool fw)
 
         if (match.hasMatch()) { // we matched a word
             // qCDebug(LOG_KTE)<<"USABLE MATCH";
-            const QStringRef m = match.capturedRef(1);
+            const QStringView m = match.capturedView(1);
             if (m != doc->text(*d->liRange) && (d->dcCursor.line() != d->dcRange.start().line() || pos != d->dcRange.start().column())) {
                 // we got good a match! replace text and return.
                 d->isCompleting = true;
@@ -522,7 +507,7 @@ void KateWordCompletionView::slotCursorMoved()
 }
 
 // Contributed by <brain@hdsnet.hu> FIXME
-QString KateWordCompletionView::findLongestUnique(const QStringList &matches, int lead) const
+QString KateWordCompletionView::findLongestUnique(const QStringList &matches, int lead)
 {
     QString partial = matches.first();
 
@@ -556,3 +541,5 @@ KTextEditor::Range KateWordCompletionView::range() const
     return m_dWCompletionModel->completionRange(m_view, m_view->cursorPosition());
 }
 // END
+
+#include "moc_katewordcompletion.cpp"
