@@ -38,10 +38,11 @@
 #include "ui_opensaveconfigadvwidget.h"
 #include "ui_opensaveconfigwidget.h"
 #include "ui_spellcheckconfigwidget.h"
+#include "ui_statusbarconfigwidget.h"
 #include "ui_textareaappearanceconfigwidget.h"
 
 #include <KIO/Job>
-#include <KIO/JobUiDelegate>
+#include <KIO/JobUiDelegateFactory>
 #include <KIO/OpenUrlJob>
 
 #include "kateabstractinputmodefactory.h"
@@ -60,11 +61,11 @@
 #include <QDomDocument>
 #include <QFile>
 #include <QFileDialog>
+#include <QFontDatabase>
 #include <QGroupBox>
 #include <QKeyEvent>
 #include <QLabel>
 #include <QLayout>
-#include <QMap>
 #include <QPainter>
 #include <QRadioButton>
 #include <QSettings>
@@ -73,7 +74,6 @@
 #include <QStringList>
 #include <QTabBar>
 #include <QTemporaryFile>
-#include <QTextCodec>
 #include <QTextStream>
 #include <QToolButton>
 #include <QWhatsThis>
@@ -228,6 +228,7 @@ KateCompletionConfigTab::KateCompletionConfigTab(QWidget *parent)
 
     observeChanges(ui->chkAutoCompletionEnabled);
     observeChanges(ui->chkAutoSelectFirstEntry);
+    observeChanges(ui->chkTabCompletion);
     observeChanges(ui->gbKeywordCompletion);
     observeChanges(ui->gbShowDoc);
     observeChanges(ui->gbWordCompletion);
@@ -259,6 +260,7 @@ void KateCompletionConfigTab::apply()
 
     KateViewConfig::global()->setValue(KateViewConfig::AutomaticCompletionInvocation, ui->chkAutoCompletionEnabled->isChecked());
     KateViewConfig::global()->setValue(KateViewConfig::AutomaticCompletionPreselectFirst, ui->chkAutoSelectFirstEntry->isChecked());
+    KateViewConfig::global()->setValue(KateViewConfig::TabCompletion, ui->chkTabCompletion->isChecked());
     KateViewConfig::global()->setValue(KateViewConfig::KeywordCompletion, ui->gbKeywordCompletion->isChecked());
     KateViewConfig::global()->setValue(KateViewConfig::WordCompletion, ui->gbWordCompletion->isChecked());
     KateViewConfig::global()->setValue(KateViewConfig::WordCompletionMinimalWordLength, ui->minimalWordLength->value());
@@ -272,6 +274,7 @@ void KateCompletionConfigTab::reload()
 {
     ui->chkAutoCompletionEnabled->setChecked(KateViewConfig::global()->automaticCompletionInvocation());
     ui->chkAutoSelectFirstEntry->setChecked(KateViewConfig::global()->automaticCompletionPreselectFirst());
+    ui->chkTabCompletion->setChecked(KateViewConfig::global()->tabCompletion());
 
     ui->gbKeywordCompletion->setChecked(KateViewConfig::global()->keywordCompletion());
     ui->gbWordCompletion->setChecked(KateViewConfig::global()->wordCompletion());
@@ -337,9 +340,9 @@ void KateSpellCheckConfigTab::apply()
     KateDocumentConfig::global()->setOnTheFlySpellCheck(settings.value(QStringLiteral("checkerEnabledByDefault"), false).toBool());
     KateDocumentConfig::global()->configEnd();
 
-    const auto docs = KTextEditor::EditorPrivate::self()->kateDocuments();
-    for (KTextEditor::DocumentPrivate *doc : docs) {
-        doc->refreshOnTheFlyCheck();
+    const auto docs = KTextEditor::EditorPrivate::self()->documents();
+    for (KTextEditor::Document *doc : docs) {
+        static_cast<KTextEditor::DocumentPrivate *>(doc)->refreshOnTheFlyCheck();
     }
 }
 
@@ -492,6 +495,7 @@ KateEditGeneralConfigTab::KateEditGeneralConfigTab(QWidget *parent)
     observeChanges(ui->chkShowStaticWordWrapMarker);
     observeChanges(ui->chkTextDragAndDrop);
     observeChanges(ui->chkSmartCopyCut);
+    observeChanges(ui->sbClipboardHistoryEntries);
     observeChanges(ui->chkStaticWordWrap);
     observeChanges(ui->cmbEncloseSelection);
     ui->lblBracketHelp->setFont(QFontDatabase::systemFont(QFontDatabase::SmallestReadableFont));
@@ -512,6 +516,7 @@ KateEditGeneralConfigTab::KateEditGeneralConfigTab(QWidget *parent)
     });
     observeChanges(ui->cmbInputMode);
     observeChanges(ui->sbWordWrap);
+    observeChanges(ui->chkAccessibility);
 
     layout->addWidget(newWidget);
 }
@@ -548,6 +553,8 @@ void KateEditGeneralConfigTab::apply()
     KateViewConfig::global()->setValue(KateViewConfig::MousePasteAtCursorPosition, ui->chkMousePasteAtCursorPosition->isChecked());
     KateViewConfig::global()->setValue(KateViewConfig::TextDragAndDrop, ui->chkTextDragAndDrop->isChecked());
     KateViewConfig::global()->setValue(KateViewConfig::SmartCopyCut, ui->chkSmartCopyCut->isChecked());
+    KateViewConfig::global()->setValue(KateViewConfig::ClipboardHistoryEntries, ui->sbClipboardHistoryEntries->value());
+    KateViewConfig::global()->setValue(KateViewConfig::EnableAccessibility, ui->chkAccessibility->isChecked());
 
     KateDocumentConfig::global()->configEnd();
     KateViewConfig::global()->configEnd();
@@ -561,6 +568,7 @@ void KateEditGeneralConfigTab::reload()
     ui->chkTextDragAndDrop->setChecked(KateViewConfig::global()->textDragAndDrop());
     ui->chkSmartCopyCut->setChecked(KateViewConfig::global()->smartCopyCut());
     ui->chkStaticWordWrap->setChecked(KateDocumentConfig::global()->wordWrap());
+    ui->sbClipboardHistoryEntries->setValue(KateViewConfig::global()->clipboardHistoryEntries());
 
     ui->sbWordWrap->setSuffix(ki18ncp("Wrap words at (value is at 20 or larger)", " character", " characters"));
     ui->sbWordWrap->setValue(KateDocumentConfig::global()->wordWrapAt());
@@ -584,6 +592,7 @@ void KateEditGeneralConfigTab::reload()
 
     const int id = static_cast<int>(KateViewConfig::global()->inputMode());
     ui->cmbInputMode->setCurrentIndex(ui->cmbInputMode->findData(id));
+    ui->chkAccessibility->setChecked(KateViewConfig::global()->value(KateViewConfig::EnableAccessibility).toBool());
 }
 
 QString KateEditGeneralConfigTab::name() const
@@ -623,7 +632,7 @@ KateEditConfigTab::KateEditConfigTab(QWidget *parent)
     for (const auto &factory : KTextEditor::EditorPrivate::self()->inputModeFactories()) {
         KateConfigPage *tab = factory->createConfigPage(this);
         if (tab) {
-            m_inputModeConfigTabs << tab;
+            m_inputModeConfigTabs.push_back(tab);
             tabWidget->insertTab(i, tab, tab->name());
             observeChanges(tab);
             i++;
@@ -646,7 +655,7 @@ void KateEditConfigTab::apply()
     indentConfigTab->apply();
     completionConfigTab->apply();
     spellCheckConfigTab->apply();
-    for (KateConfigPage *tab : std::as_const(m_inputModeConfigTabs)) {
+    for (KateConfigPage *tab : m_inputModeConfigTabs) {
         tab->apply();
     }
 }
@@ -658,7 +667,7 @@ void KateEditConfigTab::reload()
     indentConfigTab->reload();
     completionConfigTab->reload();
     spellCheckConfigTab->reload();
-    for (KateConfigPage *tab : std::as_const(m_inputModeConfigTabs)) {
+    for (KateConfigPage *tab : m_inputModeConfigTabs) {
         tab->reload();
     }
 }
@@ -670,7 +679,7 @@ void KateEditConfigTab::reset()
     indentConfigTab->reset();
     completionConfigTab->reset();
     spellCheckConfigTab->reset();
-    for (KateConfigPage *tab : std::as_const(m_inputModeConfigTabs)) {
+    for (KateConfigPage *tab : m_inputModeConfigTabs) {
         tab->reset();
     }
 }
@@ -682,7 +691,7 @@ void KateEditConfigTab::defaults()
     indentConfigTab->defaults();
     completionConfigTab->defaults();
     spellCheckConfigTab->defaults();
-    for (KateConfigPage *tab : std::as_const(m_inputModeConfigTabs)) {
+    for (KateConfigPage *tab : m_inputModeConfigTabs) {
         tab->defaults();
     }
 }
@@ -709,6 +718,7 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
     : KateConfigPage(parent)
     , textareaUi(new Ui::TextareaAppearanceConfigWidget())
     , bordersUi(new Ui::BordersAppearanceConfigWidget())
+    , statusBarUi(new Ui::StatusbarConfigWidget())
 {
     QLayout *layout = new QVBoxLayout(this);
     QTabWidget *tabWidget = new QTabWidget(this);
@@ -723,6 +733,10 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
     bordersUi->setupUi(bordersTab);
     tabWidget->addTab(bordersTab, i18n("Borders"));
 
+    QWidget *statusbarTab = new QWidget(tabWidget);
+    statusBarUi->setupUi(statusbarTab);
+    tabWidget->addTab(statusbarTab, i18n("Statusbar"));
+
     textareaUi->cmbDynamicWordWrapIndicator->addItem(i18n("Off"));
     textareaUi->cmbDynamicWordWrapIndicator->addItem(i18n("Follow Line Numbers"));
     textareaUi->cmbDynamicWordWrapIndicator->addItem(i18n("Always On"));
@@ -780,12 +794,22 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
     observeChanges(bordersUi->rbSortBookmarksByPosition);
     observeChanges(bordersUi->spBoxMiniMapWidth);
     observeChanges(bordersUi->cmbFoldingArrowVisiblity);
+
+    // statusBarUi
+    observeChanges(statusBarUi->cbShowActiveDictionary);
+    observeChanges(statusBarUi->cbShowHighlightingMode);
+    observeChanges(statusBarUi->cbShowInputMode);
+    observeChanges(statusBarUi->cbShowLineColumn);
+    observeChanges(statusBarUi->cbShowTabSetting);
+    observeChanges(statusBarUi->cbShowEncoding);
+    observeChanges(statusBarUi->cbShowEOL);
 }
 
 KateViewDefaultsConfig::~KateViewDefaultsConfig()
 {
     delete bordersUi;
     delete textareaUi;
+    delete statusBarUi;
 }
 
 void KateViewDefaultsConfig::apply()
@@ -838,6 +862,15 @@ void KateViewDefaultsConfig::apply()
     bool showOnHoverOnly = bordersUi->cmbFoldingArrowVisiblity->currentIndex() == 0;
     KateViewConfig::global()->setValue(KateViewConfig::ShowFoldingOnHoverOnly, showOnHoverOnly);
 
+    // Statusbar stuff
+    KateViewConfig::global()->setValue(KateViewConfig::ShowStatusbarDictionary, statusBarUi->cbShowActiveDictionary->isChecked());
+    KateViewConfig::global()->setValue(KateViewConfig::ShowStatusbarHighlightingMode, statusBarUi->cbShowHighlightingMode->isChecked());
+    KateViewConfig::global()->setValue(KateViewConfig::ShowStatusbarInputMode, statusBarUi->cbShowInputMode->isChecked());
+    KateViewConfig::global()->setValue(KateViewConfig::ShowStatusbarLineColumn, statusBarUi->cbShowLineColumn->isChecked());
+    KateViewConfig::global()->setValue(KateViewConfig::ShowStatusbarTabSettings, statusBarUi->cbShowTabSetting->isChecked());
+    KateViewConfig::global()->setValue(KateViewConfig::ShowStatusbarFileEncoding, statusBarUi->cbShowEncoding->isChecked());
+    KateViewConfig::global()->setValue(KateViewConfig::ShowStatusbarEOL, statusBarUi->cbShowEOL->isChecked());
+
     KateRendererConfig::global()->configEnd();
     KateViewConfig::global()->configEnd();
 }
@@ -879,6 +912,14 @@ void KateViewDefaultsConfig::reload()
     textareaUi->spacesComboBox->setCurrentIndex(KateDocumentConfig::global()->showSpaces());
     textareaUi->chkFocusFrame->setChecked(KateViewConfig::global()->showFocusFrame());
     textareaUi->spbLineHeightMultiplier->setValue(KateRendererConfig::global()->lineHeightMultiplier());
+
+    statusBarUi->cbShowLineColumn->setChecked(KateViewConfig::global()->value(KateViewConfig::ShowStatusbarLineColumn).toBool());
+    statusBarUi->cbShowActiveDictionary->setChecked(KateViewConfig::global()->value(KateViewConfig::ShowStatusbarDictionary).toBool());
+    statusBarUi->cbShowTabSetting->setChecked(KateViewConfig::global()->value(KateViewConfig::ShowStatusbarTabSettings).toBool());
+    statusBarUi->cbShowHighlightingMode->setChecked(KateViewConfig::global()->value(KateViewConfig::ShowStatusbarHighlightingMode).toBool());
+    statusBarUi->cbShowInputMode->setChecked(KateViewConfig::global()->value(KateViewConfig::ShowStatusbarInputMode).toBool());
+    statusBarUi->cbShowEncoding->setChecked(KateViewConfig::global()->value(KateViewConfig::ShowStatusbarFileEncoding).toBool());
+    statusBarUi->cbShowEOL->setChecked(KateViewConfig::global()->value(KateViewConfig::ShowStatusbarEOL).toBool());
 }
 
 void KateViewDefaultsConfig::reset()
@@ -929,7 +970,10 @@ KateSaveConfigTab::KateSaveConfigTab(QWidget *parent)
     QWidget *newWidget2 = new QWidget(tabWidget);
     uiadv = new Ui::OpenSaveConfigAdvWidget();
     uiadv->setupUi(newWidget2);
-    uiadv->lblExplanatory->setText(i18n("%1 backs up unsaved files to \"swap files.\" Swap files allow %1 to recover your work in the case of a system crash. Disabling swap files may cause data loss in case of a system crash.", QCoreApplication::applicationName()));
+    uiadv->lblExplanatory->setText(
+        i18n("%1 backs up unsaved files to \"swap files.\" Swap files allow %1 to recover your work in the case of a system crash. Disabling swap files may "
+             "cause data loss in case of a system crash.",
+             QCoreApplication::applicationName()));
     uiadv->lblExplanatory->setFont(QFontDatabase::systemFont(QFontDatabase::SmallestReadableFont));
 
     // "What's This?" help can be found in the ui file
@@ -954,7 +998,7 @@ KateSaveConfigTab::KateSaveConfigTab(QWidget *parent)
     observeChanges(uiadv->chkBackupLocalFiles);
     observeChanges(uiadv->chkBackupRemoteFiles);
     observeChanges(uiadv->cmbSwapFileMode);
-    connect(uiadv->cmbSwapFileMode, qOverload<int>(&QComboBox::currentIndexChanged), this, &KateSaveConfigTab::swapFileModeChanged);
+    connect(uiadv->cmbSwapFileMode, &QComboBox::currentIndexChanged, this, &KateSaveConfigTab::swapFileModeChanged);
 
     observeChanges(uiadv->edtBackupPrefix);
     observeChanges(uiadv->edtBackupSuffix);
@@ -1078,25 +1122,19 @@ void KateSaveConfigTab::reload()
     ui->cmbEncoding->clear();
     ui->cmbEncodingFallback->clear();
     QStringList encodings(KCharsets::charsets()->descriptiveEncodingNames());
-    int insert = 0;
     for (int i = 0; i < encodings.count(); i++) {
-        bool found = false;
-        QTextCodec *codecForEnc = KCharsets::charsets()->codecForName(KCharsets::charsets()->encodingForName(encodings[i]), found);
+        const auto encodingName = KCharsets::charsets()->encodingForName(encodings[i]);
 
-        if (found) {
-            ui->cmbEncoding->addItem(encodings[i]);
-            ui->cmbEncodingFallback->addItem(encodings[i]);
+        ui->cmbEncoding->addItem(encodings[i]);
+        ui->cmbEncodingFallback->addItem(encodings[i]);
 
-            if (codecForEnc == KateDocumentConfig::global()->codec()) {
-                ui->cmbEncoding->setCurrentIndex(insert);
-            }
+        if (encodingName == KateDocumentConfig::global()->encoding()) {
+            ui->cmbEncoding->setCurrentIndex(i);
+        }
 
-            if (codecForEnc == KateGlobalConfig::global()->fallbackCodec()) {
-                // adjust index for fallback config, has no default!
-                ui->cmbEncodingFallback->setCurrentIndex(insert);
-            }
-
-            insert++;
+        if (encodingName == KateGlobalConfig::global()->fallbackEncoding()) {
+            // adjust index for fallback config, has no default!
+            ui->cmbEncodingFallback->setCurrentIndex(i);
         }
     }
 
@@ -1214,7 +1252,7 @@ KateGotoBar::KateGotoBar(KTextEditor::View *view, QWidget *parent)
     btn = m_modifiedUp = new QToolButton(this);
     btn->setAutoRaise(true);
     btn->setMinimumSize(QSize(1, btn->minimumSizeHint().height()));
-    btn->setDefaultAction(m_view->action("modified_line_up"));
+    btn->setDefaultAction(m_view->action(QStringLiteral("modified_line_up")));
     btn->setIcon(QIcon::fromTheme(QStringLiteral("go-up-search")));
     btn->setText(QString());
     btn->installEventFilter(this);
@@ -1223,7 +1261,7 @@ KateGotoBar::KateGotoBar(KTextEditor::View *view, QWidget *parent)
     btn = m_modifiedDown = new QToolButton(this);
     btn->setAutoRaise(true);
     btn->setMinimumSize(QSize(1, btn->minimumSizeHint().height()));
-    btn->setDefaultAction(m_view->action("modified_line_down"));
+    btn->setDefaultAction(m_view->action(QStringLiteral("modified_line_down")));
     btn->setIcon(QIcon::fromTheme(QStringLiteral("go-down-search")));
     btn->setText(QString());
     btn->installEventFilter(this);
@@ -1276,7 +1314,11 @@ bool KateGotoBar::eventFilter(QObject *object, QEvent *event)
 void KateGotoBar::gotoClipboard()
 {
     static const QRegularExpression rx(QStringLiteral("-?\\d+"));
-    const int lineNo = rx.match(QApplication::clipboard()->text(QClipboard::Selection)).captured().toInt();
+    bool ok = false;
+    const int lineNo = rx.match(QApplication::clipboard()->text(QClipboard::Selection)).captured().toInt(&ok);
+    if (!ok) {
+        return;
+    }
     if (lineNo >= m_gotoRange->minimum() && lineNo <= m_gotoRange->maximum()) {
         m_gotoRange->setValue(lineNo);
         gotoLine();
@@ -1382,45 +1424,45 @@ void KateDictionaryBar::dictionaryChanged(const QString &dictionary)
 // END KateGotoBar
 
 // BEGIN KateModOnHdPrompt
-KateModOnHdPrompt::KateModOnHdPrompt(KTextEditor::DocumentPrivate *doc, KTextEditor::ModificationInterface::ModifiedOnDiskReason modtype, const QString &reason)
+KateModOnHdPrompt::KateModOnHdPrompt(KTextEditor::DocumentPrivate *doc, KTextEditor::Document::ModifiedOnDiskReason modtype, const QString &reason)
     : QObject(doc)
     , m_doc(doc)
-    , m_modtype(modtype)
+    , m_message(new KTextEditor::Message(reason, KTextEditor::Message::Information))
     , m_fullDiffPath(QStandardPaths::findExecutable(QStringLiteral("diff")))
     , m_proc(nullptr)
     , m_diffFile(nullptr)
     , m_diffAction(nullptr)
 {
-    m_message = new KTextEditor::Message(reason, KTextEditor::Message::Information);
     m_message->setPosition(KTextEditor::Message::AboveView);
     m_message->setWordWrap(true);
 
     // If the file isn't deleted, present a diff button
-    const bool onDiskDeleted = modtype == KTextEditor::ModificationInterface::OnDiskDeleted;
+    const bool onDiskDeleted = modtype == KTextEditor::Document::OnDiskDeleted;
     if (!onDiskDeleted) {
         QAction *aAutoReload = new QAction(i18n("Enable Auto Reload"), this);
         aAutoReload->setIcon(QIcon::fromTheme(QStringLiteral("view-refresh")));
-        aAutoReload->setToolTip(i18n("Will never again warn about on disk changes but always reload."));
+        aAutoReload->setToolTip(
+            i18n("Reloads and will automatically reload without warning about disk changes from now until you close either the tab or window."));
         m_message->addAction(aAutoReload, false);
         connect(aAutoReload, &QAction::triggered, this, &KateModOnHdPrompt::autoReloadTriggered);
 
         if (!m_fullDiffPath.isEmpty()) {
             m_diffAction = new QAction(i18n("View &Difference"), this);
             m_diffAction->setIcon(QIcon::fromTheme(QStringLiteral("document-multiple")));
-            m_diffAction->setToolTip(i18n("Shows a diff of the changes"));
+            m_diffAction->setToolTip(i18n("Shows a diff of the changes."));
             m_message->addAction(m_diffAction, false);
             connect(m_diffAction, &QAction::triggered, this, &KateModOnHdPrompt::slotDiff);
         }
 
         QAction *aReload = new QAction(i18n("&Reload"), this);
         aReload->setIcon(QIcon::fromTheme(QStringLiteral("view-refresh")));
-        aReload->setToolTip(i18n("Reload the file from disk. Unsaved changes will be lost."));
+        aReload->setToolTip(i18n("Reloads the file from disk. Unsaved changes will be lost."));
         m_message->addAction(aReload);
         connect(aReload, &QAction::triggered, this, &KateModOnHdPrompt::reloadTriggered);
     } else {
         QAction *closeFile = new QAction(i18nc("@action:button closes the opened file", "&Close File"), this);
         closeFile->setIcon(QIcon::fromTheme(QStringLiteral("document-close")));
-        closeFile->setToolTip(i18n("Close the file, discarding its content."));
+        closeFile->setToolTip(i18n("Closes the file, discarding its content."));
         m_message->addAction(closeFile, false);
         connect(closeFile, &QAction::triggered, this, &KateModOnHdPrompt::closeTriggered);
 
@@ -1519,9 +1561,11 @@ void KateModOnHdPrompt::slotPDone()
     m_diffFile = nullptr;
 
     KIO::OpenUrlJob *job = new KIO::OpenUrlJob(url, QStringLiteral("text/x-patch"));
-    job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, nullptr /*TODO window*/));
+    job->setUiDelegate(KIO::createDefaultJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, m_doc->activeView()));
     job->setDeleteTemporaryFile(true); // delete the file, once the client exits
     job->start();
 }
 
 // END KateModOnHdPrompt
+
+#include "moc_katedialogs.cpp"
