@@ -14,20 +14,15 @@
 #include "katehighlight.h"
 #include "katepartdebug.h"
 #include "katesyntaxmanager.h"
+#include "ktexteditor/message.h"
 
 #include <KLocalizedString>
 
 #include <QDate>
 #include <QFile>
 #include <QFileInfo>
-#include <QTextCodec>
+#include <QStringEncoder>
 #include <QTextStream>
-#include <QTimer>
-
-/**
- * Initial value for m_maxDynamicContexts
- */
-static const int KATE_MAX_DYNAMIC_CONTEXTS = 512;
 
 /**
  * Create an empty buffer. (with one block with one empty line)
@@ -41,7 +36,6 @@ KateBuffer::KateBuffer(KTextEditor::DocumentPrivate *doc)
     , m_highlight(nullptr)
     , m_tabWidth(8)
     , m_lineHighlighted(0)
-    , m_maxDynamicContexts(KATE_MAX_DYNAMIC_CONTEXTS)
 {
 }
 
@@ -112,8 +106,8 @@ bool KateBuffer::openFile(const QString &m_file, bool enforceTextCodec)
 {
     // first: setup fallback and normal encoding
     setEncodingProberType(KateGlobalConfig::global()->proberType());
-    setFallbackTextCodec(KateGlobalConfig::global()->fallbackCodec());
-    setTextCodec(m_doc->config()->codec());
+    setFallbackTextCodec(KateGlobalConfig::global()->fallbackEncoding());
+    setTextCodec(m_doc->config()->encoding());
 
     // setup eol
     setEndOfLineMode((EndOfLineMode)m_doc->config()->eol());
@@ -144,7 +138,6 @@ bool KateBuffer::openFile(const QString &m_file, bool enforceTextCodec)
 
         // remember error
         m_doc->m_openingError = true;
-        m_doc->m_openingErrorMessage = i18n("The file %1 does not exist.", m_doc->url().toString());
         return true;
     }
 
@@ -161,7 +154,7 @@ bool KateBuffer::openFile(const QString &m_file, bool enforceTextCodec)
     }
 
     // save back encoding
-    m_doc->config()->setEncoding(QString::fromLatin1(textCodec()->name()));
+    m_doc->config()->setEncoding(textCodec());
 
     // set eol mode, if a eol char was found
     if (m_doc->config()->allowEolDetection()) {
@@ -179,16 +172,31 @@ bool KateBuffer::openFile(const QString &m_file, bool enforceTextCodec)
 
 bool KateBuffer::canEncode()
 {
-    QTextCodec *codec = m_doc->config()->codec();
-
     // hardcode some Unicode encodings which can encode all chars
-    if ((QString::fromLatin1(codec->name()) == QLatin1String("UTF-8")) || (QString::fromLatin1(codec->name()) == QLatin1String("ISO-10646-UCS-2"))) {
-        return true;
+    if (const auto setEncoding = QStringConverter::encodingForName(m_doc->config()->encoding().toUtf8().constData())) {
+        for (const auto encoding : {QStringConverter::Utf8,
+                                    QStringConverter::Utf16,
+                                    QStringConverter::Utf16BE,
+                                    QStringConverter::Utf16LE,
+                                    QStringConverter::Utf32,
+                                    QStringConverter::Utf32BE,
+                                    QStringConverter::Utf32LE}) {
+            if (setEncoding == encoding) {
+                return true;
+            }
+        }
     }
 
+    QStringEncoder encoder(m_doc->config()->encoding().toUtf8().constData());
     for (int i = 0; i < lines(); i++) {
-        if (!codec->canEncode(line(i)->text())) {
-            qCDebug(LOG_KTE) << QLatin1String("ENC NAME: ") << codec->name();
+        {
+            // actual encoding happens not during the call to encode() but
+            // during the conversion to QByteArray, so we need to force it
+            QByteArray result = encoder.encode(line(i)->text());
+            Q_UNUSED(result);
+        }
+        if (encoder.hasError()) {
+            qCDebug(LOG_KTE) << QLatin1String("ENC NAME: ") << m_doc->config()->encoding();
             qCDebug(LOG_KTE) << QLatin1String("STRING LINE: ") << line(i)->text();
             qCDebug(LOG_KTE) << QLatin1String("ENC WORKING: FALSE");
 
@@ -203,8 +211,8 @@ bool KateBuffer::saveFile(const QString &m_file)
 {
     // first: setup fallback and normal encoding
     setEncodingProberType(KateGlobalConfig::global()->proberType());
-    setFallbackTextCodec(KateGlobalConfig::global()->fallbackCodec());
-    setTextCodec(m_doc->config()->codec());
+    setFallbackTextCodec(KateGlobalConfig::global()->fallbackEncoding());
+    setTextCodec(m_doc->config()->encoding());
 
     // setup eol
     setEndOfLineMode((EndOfLineMode)m_doc->config()->eol());
@@ -346,7 +354,7 @@ void KateBuffer::doHighlight(int startLine, int endLine, bool invalidate)
         qCDebug(LOG_KTE) << "current line to hl: " << current_line;
         qCDebug(LOG_KTE) << "text length: " << textLine->length() << " attribute list size: " << textLine->attributesList().size();
 
-        const QVector<int> &ml(textLine->attributesList());
+        const QList<int> &ml(textLine->attributesList());
         for (int i = 2; i < ml.size(); i += 3) {
             qCDebug(LOG_KTE) << "start: " << ml.at(i - 2) << " len: " << ml.at(i - 1) << " at: " << ml.at(i) << " ";
         }
@@ -493,19 +501,19 @@ KTextEditor::Range KateBuffer::computeFoldingRangeForStartLine(int startLine)
     // 'normal' attribute based folding, aka token based like '{' BLUB '}'
 
     // first step: search the first region type, that stays open for the start line
-    short openedRegionType = 0;
+    int openedRegionType = -1;
     int openedRegionOffset = -1;
     {
         // mapping of type to "first" offset of it and current number of not matched openings
-        QHash<short, QPair<int, int>> foldingStartToOffsetAndCount;
+        QHash<int, QPair<int, int>> foldingStartToOffsetAndCount;
 
         // walk over all attributes of the line and compute the matchings
         const auto &startLineAttributes = startTextLine->foldings();
         for (size_t i = 0; i < startLineAttributes.size(); ++i) {
             // folding close?
-            if (startLineAttributes[i].foldingValue < 0) {
+            if (startLineAttributes[i].foldingRegion.type() == KSyntaxHighlighting::FoldingRegion::End) {
                 // search for this type, try to decrement counter, perhaps erase element!
-                QHash<short, QPair<int, int>>::iterator end = foldingStartToOffsetAndCount.find(-startLineAttributes[i].foldingValue);
+                auto end = foldingStartToOffsetAndCount.find(startLineAttributes[i].foldingRegion.id());
                 if (end != foldingStartToOffsetAndCount.end()) {
                     if (end.value().second > 1) {
                         --(end.value().second);
@@ -516,19 +524,19 @@ KTextEditor::Range KateBuffer::computeFoldingRangeForStartLine(int startLine)
             }
 
             // folding open?
-            if (startLineAttributes[i].foldingValue > 0) {
+            if (startLineAttributes[i].foldingRegion.type() == KSyntaxHighlighting::FoldingRegion::Begin) {
                 // search for this type, either insert it, with current offset or increment counter!
-                QHash<short, QPair<int, int>>::iterator start = foldingStartToOffsetAndCount.find(startLineAttributes[i].foldingValue);
+                auto start = foldingStartToOffsetAndCount.find(startLineAttributes[i].foldingRegion.id());
                 if (start != foldingStartToOffsetAndCount.end()) {
                     ++(start.value().second);
                 } else {
-                    foldingStartToOffsetAndCount.insert(startLineAttributes[i].foldingValue, qMakePair(startLineAttributes[i].offset, 1));
+                    foldingStartToOffsetAndCount.insert(startLineAttributes[i].foldingRegion.id(), qMakePair(startLineAttributes[i].offset, 1));
                 }
             }
         }
 
         // compute first type with offset
-        QHashIterator<short, QPair<int, int>> hashIt(foldingStartToOffsetAndCount);
+        QHashIterator<int, QPair<int, int>> hashIt(foldingStartToOffsetAndCount);
         while (hashIt.hasNext()) {
             hashIt.next();
             if (openedRegionOffset == -1 || hashIt.value().first < openedRegionOffset) {
@@ -539,7 +547,7 @@ KTextEditor::Range KateBuffer::computeFoldingRangeForStartLine(int startLine)
     }
 
     // no opening region found, bad, nothing to do
-    if (openedRegionType == 0) {
+    if (openedRegionType == -1) {
         return KTextEditor::Range::invalid();
     }
 
@@ -554,7 +562,7 @@ KTextEditor::Range KateBuffer::computeFoldingRangeForStartLine(int startLine)
         const auto &lineAttributes = textLine->foldings();
         for (size_t i = 0; i < lineAttributes.size(); ++i) {
             // matching folding close?
-            if (lineAttributes[i].foldingValue == -openedRegionType) {
+            if (lineAttributes[i].foldingRegion.type() == KSyntaxHighlighting::FoldingRegion::End && lineAttributes[i].foldingRegion.id() == openedRegionType) {
                 --countOfOpenRegions;
 
                 // end reached?
@@ -571,7 +579,8 @@ KTextEditor::Range KateBuffer::computeFoldingRangeForStartLine(int startLine)
             }
 
             // matching folding open?
-            if (lineAttributes[i].foldingValue == openedRegionType) {
+            if (lineAttributes[i].foldingRegion.type() == KSyntaxHighlighting::FoldingRegion::Begin
+                && lineAttributes[i].foldingRegion.id() == openedRegionType) {
                 ++countOfOpenRegions;
             }
         }
