@@ -14,21 +14,14 @@
 #include <QStack>
 #include <QTimer>
 
-#include <KJob>
-
-#include <ktexteditor/annotationinterface.h>
-#include <ktexteditor/configinterface.h>
 #include <ktexteditor/document.h>
-#include <ktexteditor/inlinenoteinterface.h>
 #include <ktexteditor/mainwindow.h>
-#include <ktexteditor/markinterface.h>
-#include <ktexteditor/message.h>
-#include <ktexteditor/modificationinterface.h>
-#include <ktexteditor/movinginterface.h>
+#include <ktexteditor/movingrangefeedback.h>
 
 #include "katetextline.h"
 #include <ktexteditor_export.h>
 
+class KJob;
 class KateTemplateHandler;
 namespace KTextEditor
 {
@@ -50,6 +43,7 @@ class SwapFile;
 class KateBuffer;
 namespace KTextEditor
 {
+class Message;
 class ViewPrivate;
 }
 class KateDocumentConfig;
@@ -68,28 +62,24 @@ class KToggleAction;
  * @warning This file is @e private API and not part of the public
  *          KTextEditor interfaces.
  */
-class KTEXTEDITOR_EXPORT KTextEditor::DocumentPrivate final : public KTextEditor::Document,
-                                                              public KTextEditor::MarkInterfaceV2,
-                                                              public KTextEditor::ModificationInterface,
-                                                              public KTextEditor::ConfigInterface,
-                                                              public KTextEditor::AnnotationInterface,
-                                                              public KTextEditor::MovingInterface,
-                                                              private KTextEditor::MovingRangeFeedback
+class KTEXTEDITOR_EXPORT KTextEditor::DocumentPrivate final : public KTextEditor::Document, private KTextEditor::MovingRangeFeedback
 {
     Q_OBJECT
-    Q_INTERFACES(KTextEditor::MarkInterface)
-    Q_INTERFACES(KTextEditor::MarkInterfaceV2)
-    Q_INTERFACES(KTextEditor::ModificationInterface)
-    Q_INTERFACES(KTextEditor::AnnotationInterface)
-    Q_INTERFACES(KTextEditor::ConfigInterface)
-    Q_INTERFACES(KTextEditor::MovingInterface)
 
     friend class KTextEditor::Document;
     friend class ::KateDocumentTest;
     friend class ::KateBuffer;
 
 public:
-    explicit DocumentPrivate(bool bSingleViewMode = false, bool bReadOnly = false, QWidget *parentWidget = nullptr, QObject * = nullptr);
+    explicit DocumentPrivate(const KPluginMetaData &data,
+                             bool bSingleViewMode = false,
+                             bool bReadOnly = false,
+                             QWidget *parentWidget = nullptr,
+                             QObject * = nullptr);
+    explicit DocumentPrivate(bool bSingleViewMode = false, bool bReadOnly = false, QWidget *parentWidget = nullptr, QObject * = nullptr)
+        : DocumentPrivate(KPluginMetaData(), bSingleViewMode, bReadOnly, parentWidget)
+    {
+    }
     ~DocumentPrivate() override;
 
     using ReadWritePart::closeUrl;
@@ -99,8 +89,6 @@ public:
 
     KTextEditor::Range rangeOnLine(KTextEditor::Range range, int line) const;
 
-    void setMetaData(const KPluginMetaData &metaData);
-
 private:
     KTEXTEDITOR_NO_EXPORT
     void showAndSetOpeningErrorAccess();
@@ -136,16 +124,15 @@ public:
 
     QList<KTextEditor::View *> views() const override
     {
-        return m_viewsCache;
+        return m_views;
     }
 
-    virtual KTextEditor::View *activeView() const
+    KTextEditor::View *activeView() const
     {
         return m_activeView;
     }
 
 private:
-    QHash<KTextEditor::View *, KTextEditor::ViewPrivate *> m_views;
     KTextEditor::View *m_activeView = nullptr;
 
     //
@@ -156,33 +143,33 @@ public Q_SLOTS:
     bool setText(const QStringList &text) override;
     bool clear() override;
 
-    bool insertText(const KTextEditor::Cursor &position, const QString &s, bool block = false) override;
-    bool insertText(const KTextEditor::Cursor &position, const QStringList &text, bool block = false) override;
+    bool insertText(KTextEditor::Cursor position, const QString &s, bool block = false) override;
+    bool insertText(KTextEditor::Cursor position, const QStringList &text, bool block = false) override;
 
     bool insertLine(int line, const QString &s) override;
     bool insertLines(int line, const QStringList &s) override;
 
-    bool removeText(const KTextEditor::Range &range, bool block = false) override;
+    bool removeText(KTextEditor::Range range, bool block = false) override;
     bool removeLine(int line) override;
 
-    bool replaceText(const KTextEditor::Range &range, const QString &s, bool block = false) override;
+    bool replaceText(KTextEditor::Range range, const QString &s, bool block = false) override;
 
     // unhide method...
-    bool replaceText(const KTextEditor::Range &r, const QStringList &l, bool b) override
+    bool replaceText(KTextEditor::Range r, const QStringList &l, bool b) override
     {
         return KTextEditor::Document::replaceText(r, l, b);
     }
 
 public:
     bool isEditingTransactionRunning() const override;
-    QString text(const KTextEditor::Range &range, bool blockwise = false) const override;
-    QStringList textLines(const KTextEditor::Range &range, bool block = false) const override;
+    QString text(KTextEditor::Range range, bool blockwise = false) const override;
+    QStringList textLines(KTextEditor::Range range, bool block = false) const override;
     QString text() const override;
     QString line(int line) const override;
-    QChar characterAt(const KTextEditor::Cursor &position) const override;
-    QString wordAt(const KTextEditor::Cursor &cursor) const override;
-    KTextEditor::Range wordRangeAt(const KTextEditor::Cursor &cursor) const override;
-    bool isValidTextPosition(const KTextEditor::Cursor &cursor) const override;
+    QChar characterAt(KTextEditor::Cursor position) const override;
+    QString wordAt(KTextEditor::Cursor cursor) const override;
+    KTextEditor::Range wordRangeAt(KTextEditor::Cursor cursor) const override;
+    bool isValidTextPosition(KTextEditor::Cursor cursor) const override;
     int lines() const override;
     bool isLineModified(int line) const override;
     bool isLineSaved(int line) const override;
@@ -190,9 +177,11 @@ public:
     KTextEditor::Cursor documentEnd() const override;
     int totalCharacters() const override;
     int lineLength(int line) const override;
+    qsizetype cursorToOffset(KTextEditor::Cursor c) const override;
+    KTextEditor::Cursor offsetToCursor(qsizetype offset) const override;
 
 Q_SIGNALS:
-    void charactersSemiInteractivelyInserted(const KTextEditor::Cursor &position, const QString &text);
+    void charactersSemiInteractivelyInserted(KTextEditor::Cursor position, const QString &text);
 
     /**
      * The \p document emits this signal whenever text was inserted.  The
@@ -202,7 +191,7 @@ Q_SIGNALS:
      * \param range range that the newly inserted text occupies
      * \see insertText(), insertLine()
      */
-    void textInsertedRange(KTextEditor::Document *document, const KTextEditor::Range &range);
+    void textInsertedRange(KTextEditor::Document *document, KTextEditor::Range range);
 
     /**
      * The \p document emits this signal whenever \p range was removed, i.e.
@@ -212,7 +201,7 @@ Q_SIGNALS:
      * \param oldText the text that has been removed
      * \see removeText(), removeLine(), clear()
      */
-    void textRemoved(KTextEditor::Document *document, const KTextEditor::Range &range, const QString &oldText);
+    void textRemoved(KTextEditor::Document *document, KTextEditor::Range range, const QString &oldText);
 
 public:
     // BEGIN editStart/editEnd (start, end, undo, cursor update, view update)
@@ -222,14 +211,6 @@ public:
      */
     bool editStart();
 
-    /**
-     * Alias for @p editStart()
-     */
-    void editBegin()
-    {
-        editStart();
-    }
-
     /**
      * End a editor operation.
      * @see editStart()
@@ -239,15 +220,6 @@ public:
     void pushEditState();
     void popEditState();
 
-    virtual bool startEditing()
-    {
-        return editStart();
-    }
-    virtual bool finishEditing()
-    {
-        return editEnd();
-    }
-
     // END editStart/editEnd
 
     void inputMethodStart();
@@ -372,7 +344,7 @@ private:
     bool editIsRunning = false;
     bool m_undoMergeAllEdits = false;
     KTextEditor::Cursor m_editLastChangeStartCursor = KTextEditor::Cursor::invalid();
-    QStack<QSharedPointer<KTextEditor::MovingCursor>> m_editingStack;
+    QStack<std::shared_ptr<KTextEditor::MovingCursor>> m_editingStack;
     int m_editingStackPosition = -1;
 
     //
@@ -410,7 +382,7 @@ Q_SIGNALS:
     void undoChanged();
 
 public:
-    QVector<KTextEditor::Range> searchText(KTextEditor::Range range, const QString &pattern, const KTextEditor::SearchOptions options) const;
+    QList<KTextEditor::Range> searchText(KTextEditor::Range range, const QString &pattern, const KTextEditor::SearchOptions options) const;
 
 private:
     /**
@@ -435,7 +407,7 @@ public:
     /**
      * @copydoc KTextEditor::Document::defaultStyleAt()
      */
-    KTextEditor::DefaultStyle defaultStyleAt(const KTextEditor::Cursor &position) const override;
+    KSyntaxHighlighting::Theme::TextStyle defaultStyleAt(KTextEditor::Cursor position) const override;
 
     /**
      * Return the name of the currently used mode
@@ -553,45 +525,26 @@ public Q_SLOTS:
     /// Returns true if the context-menu event should not further be processed
     bool handleMarkContextMenu(int line, QPoint position);
 
-    void setMarkPixmap(MarkInterface::MarkTypes, const QPixmap &) override;
-
-    void setMarkDescription(MarkInterface::MarkTypes, const QString &) override;
+    void setMarkDescription(Document::MarkTypes, const QString &) override;
 
     void setEditableMarks(uint markMask) override;
+    void setMarkIcon(Document::MarkTypes markType, const QIcon &icon) override;
 
 public:
     uint mark(int line) override;
     const QHash<int, KTextEditor::Mark *> &marks() override;
-    QPixmap markPixmap(MarkInterface::MarkTypes) const override;
-    QString markDescription(MarkInterface::MarkTypes) const override;
-    virtual QColor markColor(MarkInterface::MarkTypes) const;
+    QString markDescription(Document::MarkTypes) const override;
+    virtual QColor markColor(Document::MarkTypes) const;
     uint editableMarks() const override;
-
-Q_SIGNALS:
-    void markToolTipRequested(KTextEditor::Document *document, KTextEditor::Mark mark, QPoint position, bool &handled);
-
-    void markContextMenuRequested(KTextEditor::Document *document, KTextEditor::Mark mark, QPoint pos, bool &handled);
-
-    void markClicked(KTextEditor::Document *document, KTextEditor::Mark mark, bool &handled);
-
-    void marksChanged(KTextEditor::Document *) override;
-    void markChanged(KTextEditor::Document *, KTextEditor::Mark, KTextEditor::MarkInterface::MarkChangeAction) override;
+    QIcon markIcon(Document::MarkTypes markType) const override;
 
 private:
     QHash<int, KTextEditor::Mark *> m_marks;
-    QHash<int, QVariant> m_markIcons; // QPixmap or QIcon, KF6: remove QPixmap support
+    QHash<int, QIcon> m_markIcons; // QPixmap or QIcon, KF6: remove QPixmap support
     QHash<int, QString> m_markDescriptions;
     uint m_editableMarks = markType01;
 
     //
-    // KTextEditor::MarkInterfaceV2
-    //
-public Q_SLOTS:
-    void setMarkIcon(MarkInterface::MarkTypes markType, const QIcon &icon) override;
-
-public:
-    QIcon markIcon(MarkInterface::MarkTypes markType) const override;
-
     // KTextEditor::PrintInterface
     //
 public Q_SLOTS:
@@ -653,7 +606,7 @@ public:
      * @param insertBehavior insertion behavior
      * @return new moving cursor for the document
      */
-    KTextEditor::MovingCursor *newMovingCursor(const KTextEditor::Cursor &position,
+    KTextEditor::MovingCursor *newMovingCursor(KTextEditor::Cursor position,
                                                KTextEditor::MovingCursor::InsertBehavior insertBehavior = KTextEditor::MovingCursor::MoveOnInsert) override;
 
     /**
@@ -663,7 +616,7 @@ public:
      * @param emptyBehavior behavior on becoming empty
      * @return new moving range for the document
      */
-    KTextEditor::MovingRange *newMovingRange(const KTextEditor::Range &range,
+    KTextEditor::MovingRange *newMovingRange(KTextEditor::Range range,
                                              KTextEditor::MovingRange::InsertBehaviors insertBehaviors = KTextEditor::MovingRange::DoNotExpand,
                                              KTextEditor::MovingRange::EmptyBehavior emptyBehavior = KTextEditor::MovingRange::AllowEmpty) override;
 
@@ -729,23 +682,6 @@ public:
                         qint64 fromRevision,
                         qint64 toRevision = -1) override;
 
-    //
-    // MovingInterface Signals
-    //
-Q_SIGNALS:
-    /**
-     * This signal is emitted before the cursors/ranges/revisions of a document are destroyed as the document is deleted.
-     * @param document the document which the interface belongs too which is in the process of being deleted
-     */
-    void aboutToDeleteMovingInterfaceContent(KTextEditor::Document *document);
-
-    /**
-     * This signal is emitted before the ranges of a document are invalidated and the revisions are deleted as the document is cleared (for example on
-     * load/reload). While this signal is emitted, still the old document content is around before the clear.
-     * @param document the document which the interface belongs too which will invalidate its data
-     */
-    void aboutToInvalidateMovingInterfaceContent(KTextEditor::Document *document);
-
     //
     // Annotation Interface
     //
@@ -1047,15 +983,6 @@ public Q_SLOTS:
 public:
     bool saveAs(const QUrl &url) override;
 
-Q_SIGNALS:
-    /**
-     * Indicate this file is modified on disk
-     * @param doc the KTextEditor::Document object that represents the file on disk
-     * @param isModified indicates the file was modified rather than created or deleted
-     * @param reason the reason we are emitting the signal.
-     */
-    void modifiedOnDisk(KTextEditor::Document *doc, bool isModified, KTextEditor::ModificationInterface::ModifiedOnDiskReason reason) override;
-
 private:
     // helper to handle the embedded notification for externally modified files
     QPointer<KateModOnHdPrompt> m_modOnHdHandler;
@@ -1129,6 +1056,12 @@ private:
     void removeTrailingSpacesAndAddNewLineAtEof();
 
 public:
+    /**
+     * This function doesn't check for config and is
+     * available for use all the time via an action
+     */
+    void removeAllTrailingSpaces();
+
     /**
      * Returns a git compatible sha1 checksum of this document on disk.
      * @return checksum for this document on disk
@@ -1246,9 +1179,10 @@ private:
     /**
       Reads all the variables in the document.
       Called when opening/saving a document
+      Returns true if variables were read
     */
-    KTEXTEDITOR_NO_EXPORT
-    void readVariables(bool onlyViewAndRenderer = false);
+    bool readVariables(bool onlyViewAndRenderer = false);
+    // exported for katedocument_test
 
     /**
       Reads and applies the variables in a single line
@@ -1276,7 +1210,7 @@ private:
     KTEXTEDITOR_NO_EXPORT
     static bool checkIntValue(const QString &value, int *result);
     /**
-      Feeds value into @p col using QColor::setNamedColor() and returns
+      Feeds value into @p col using QColor::fromString() and returns
       whether the color is valid
     */
     KTEXTEDITOR_NO_EXPORT
@@ -1305,7 +1239,7 @@ public:
     //
 public:
     QStringList embeddedHighlightingModes() const override;
-    QString highlightingModeAt(const KTextEditor::Cursor &position) override;
+    QString highlightingModeAt(KTextEditor::Cursor position) override;
 
     //
     // BEGIN: KTextEditor::MessageInterface
@@ -1317,7 +1251,7 @@ public Q_SLOTS:
     void messageDestroyed(KTextEditor::Message *message);
 
 private:
-    QHash<KTextEditor::Message *, QList<QSharedPointer<QAction>>> m_messageHash;
+    QHash<KTextEditor::Message *, QList<std::shared_ptr<QAction>>> m_messageHash;
     // END KTextEditor::MessageInterface
 
 public:
@@ -1378,7 +1312,7 @@ public:
     Kate::SwapFile *swapFile();
 
     // helpers for scripting and codefolding
-    int defStyleNum(int line, int column);
+    KSyntaxHighlighting::Theme::TextStyle defStyleNum(int line, int column);
     bool isComment(int line, int column);
 
 public:
@@ -1470,11 +1404,6 @@ private:
      */
     bool m_openingError = false;
 
-    /**
-     * Last open file error message
-     */
-    QString m_openingErrorMessage;
-
 public:
     /**
      * reads the line length limit from config, if it is not overridden
@@ -1516,10 +1445,7 @@ Q_SIGNALS:
     void loaded(KTextEditor::DocumentPrivate *document);
 
 private:
-    // To calculate a QHash.keys() is quite expensive,
-    // better keep a copy of that list updated when a view is added or removed.
-    QList<KTextEditor::View *> m_viewsCache;
-
+    QList<KTextEditor::View *> m_views;
     QTimer m_autoSaveTimer;
 };
 
