@@ -10,16 +10,17 @@
 
 #include <ktexteditor_export.h>
 
-#include <ktexteditor/attribute.h>
 #include <ktexteditor/cursor.h>
+#include <ktexteditor/movingcursor.h>
+#include <ktexteditor/movingrange.h>
 #include <ktexteditor/range.h>
 
 // our main baseclass of the KTextEditor::Document
 #include <KParts/ReadWritePart>
+#include <KSyntaxHighlighting/Theme>
 
 // the list of views
 #include <QList>
-#include <QMetaType>
 
 class KConfigGroup;
 
@@ -30,6 +31,7 @@ class EditingTransactionPrivate;
 class MainWindow;
 class Message;
 class View;
+class AnnotationModel;
 
 /**
  * \brief Search flags for use with searchText.
@@ -63,6 +65,20 @@ enum SearchOption {
 Q_DECLARE_FLAGS(SearchOptions, SearchOption)
 Q_DECLARE_OPERATORS_FOR_FLAGS(SearchOptions)
 
+/**
+ * \class Mark
+ * \see doc_marktext
+ */
+class Mark
+{
+public:
+    /** The line that contains the mark. */
+    int line;
+
+    /** The mark types in the line, combined with logical OR. */
+    uint type;
+};
+
 /**
  * \class Document document.h <KTextEditor/Document>
  *
@@ -75,7 +91,11 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(SearchOptions)
  *  - \ref doc_readwrite
  *  - \ref doc_notifications
  *  - \ref doc_recovery
- *  - \ref doc_extensions
+ *  - \ref doc_movinginterface
+ *  - \ref doc_config
+ *  - \ref doc_modiface
+ *  - \ref doc_marktext
+ *  - \ref doc_annoiface
  *
  * \section doc_intro Introduction
  *
@@ -166,20 +186,98 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(SearchOptions)
  * either trigger the data recovery by calling recoverData() or discard it
  * through discardDataRecovery().
  *
- * \section doc_extensions Document Extension Interfaces
+ * \section doc_movinginterface Document Moving Interface
  *
- * A simple document represents text and provides text manipulation methods.
- * However, a real text editor should support advanced concepts like session
- * support, textsearch support, bookmark/general mark support etc. That is why
- * the KTextEditor library provides several additional interfaces to extend
- * a document's capabilities via multiple inheritance.
+ * Document Moving Interface allows you to create MovingRange and MovingCursor. A
+ * Moving Range or Cursor is a special type of range/cursor because it automatically
+ * moves on text insertion or removal. Additionally, one can use the moving ranges to
+ * change the color of a particular word or give it a different attribute such as bold.
+ * Use newMovingCursor() to create a new moving cursor and newMovingRange() to create
+ * a new moving range.
+ *
+ * Upon destruction or reload, a document will remove all its moving ranges. You can
+ * connect to aboutToDeleteMovingInterfaceContent() and aboutToInvalidateMovingInterfaceContent()
+ * signals to know when that is going to happen and update the cached ranges accordingly.
+ *
+ * \section doc_config Document Config
+ * Config provides methods to access and modify the low level config information for a given
+ * Document.
+ * KTextEditor::Document has support for the following config keys:
+ *  - backup-on-save-local [bool], enable/disable backup when saving local files
+ *  - backup-on-save-remote [bool], enable/disable backup when saving remote files
+ *  - backup-on-save-suffix [string], set the suffix for file backups, e.g. "~"
+ *  - backup-on-save-prefix [string], set the prefix for file backups, e.g. "."
+ *  - replace-tabs [bool], whether to replace tabs
+ *  - indent-pasted-text [bool], whether to indent pasted text
+ *  - tab-width [int], read/set the width for tabs
+ *  - indent-width [int], read/set the indentation width
+ *  - on-the-fly-spellcheck [bool], enable/disable on the fly spellcheck
+ *
+ * \section doc_modiface  External modification extension interface for the Document.
+ *
+ * The class ModificationInterface provides methods to handle modifications
+ * of all opened files caused by external programs. Whenever the
+ * modified-on-disk state changes the signal modifiedOnDisk() is emitted
+ * along with a ModifiedOnDiskReason. Set the state by calling
+ * setModifiedOnDisk(). Whether the Editor should show warning dialogs to
+ * inform the user about external modified files can be controlled with
+ * setModifiedOnDiskWarning(). The slot modifiedOnDisk() is called to ask
+ * the user what to do whenever a file was modified.
+ *
+ * \section doc_marktext
+ *
+ * The Mark Interface provides methods to enable and disable marks in a
+ * Document, a marked line can be visualized for example with a shaded
+ * background color and/or a pixmap in the iconborder of the Document's View.
+ * There are a number of predefined mark types, specified in
+ * reservedMarkersCount(). Additionally it is possible to add custom marks
+ * and set custom pixmaps.
+ *
+ * Get all marks in the document by calling marks(). Use clearMarks() to
+ * remove all marks in the entire document. A single mark can be retrieved
+ * with mark(). To remove all marks from a line call clearMark(). To add
+ * and remove marks from a given line use addMark() and removeMark(). It is
+ * also possible to replace all marks with setMark(), i.e. setMark() is the
+ * same as a call of clearMark() followed by addMark(). The signals
+ * marksChanged() and markChanged() are emitted whenever a line's marks
+ * changed.
+ *
+ * \attention A mark type is represented as an \e uint. An \e uint can have
+ *     several mark types combined (see above: logical OR). That means for
+ *     all functions/signals with an \e uint parameter, e.g. setMark(),
+ *     removeMark(), etc, the \e uint may contain \e multiple marks, i.e.
+ *     you can add and remove multiple marks \e simultaneously.
+ *
+ * All marks that should be editable by the user can be specified with a mark
+ * mask via setEditableMarks(). To set a description and pixmap of a mark type
+ * call setMarkDescription() and setMarkPixmap().
+ *
+ * \section doc_annoiface Annotation Interface
+ *
+ * The Annotation Interface is designed to provide line annotation information
+ * for a document. This interface provides means to associate a document with a
+ * annotation model, which provides some annotation information for each line
+ * in the document.
+ *
+ * Setting a model for a Document makes the model data available for all views.
+ * If you only want to provide annotations in exactly one view, you can use
+ * the AnnotationViewInterface directly. See the AnnotationViewInterface for
+ * further details. To summarize, the two use cases are
+ * - (1) show annotations in all views. This means you set an AnnotationModel
+ *       with this interface, and then call setAnnotationBorderVisible() for
+ *       each view.
+ * - (2) show annotations only in one view. This means to \e not use this
+ *       interface. Instead, use the AnnotationViewInterface, which inherits
+ *       this interface. This means you set a model for the specific View.
+ *
+ * If you set a model to the Document \e and the View, the View's model has
+ * higher priority.
  *
  * More information about interfaces for the document can be found in
  * \ref kte_group_doc_extensions.
  *
  * \see KParts::ReadWritePart, KTextEditor::Editor, KTextEditor::View,
- *      KTextEditor::MarkInterface, KTextEditor::ModificationInterface,
- *      KTextEditor::MovingInterface
+ *      KTextEditor::MarkInterface
  * \author Christoph Cullmann \<cullmann@kde.org\>
  */
 class KTEXTEDITOR_EXPORT Document : public KParts::ReadWritePart
@@ -198,13 +296,13 @@ protected:
      * \param parent parent object
      * \see Editor::createDocument()
      */
-    Document(DocumentPrivate *impl, QObject *parent);
+    Document(DocumentPrivate *impl, const KPluginMetaData &data, QObject *parent);
 
 public:
     /**
      * Virtual destructor.
      */
-    virtual ~Document();
+    ~Document() override;
 
     /**
      * \name Manage View%s of this Document
@@ -324,7 +422,7 @@ public:
      * opened without the correct encoding, this will \e not fix it, you
      * would for example need to trigger a reload for this.
      * \param encoding new encoding for the document, the name must be
-     *        accepted by QTextCodec, if an empty encoding name is given, the
+     *        accepted by QStringDecoder/QStringEncoder, if an empty encoding name is given, the
      *        part should fallback to its own default encoding, e.g. the
      *        system encoding or the global user settings
      * \return \e true on success, or \e false, if the encoding could not be set.
@@ -382,14 +480,6 @@ public:
      */
     bool openingError() const;
 
-    /**
-     * Error message if any problem occurred on last load.
-     * @return error message what went wrong on loading
-     */
-    // TODO KF6: Not needed anymore since we show load trouble as KTextEditor::Message.
-    //      Remove all code which set m_openingErrorMessage
-    QString openingErrorMessage() const;
-
     /*
      * SIGNALS
      * Following signals should be emitted by the document if the text content
@@ -435,6 +525,17 @@ Q_SIGNALS:
      */
     void reloaded(KTextEditor::Document *document);
 
+    /**
+     * Emitted just before the document will be saved
+     * Any modifications made to the document at this point
+     * will get stored on disk.
+     *
+     * \param document the document that was reloaded.
+     *
+     * @since 5.91
+     */
+    void aboutToSave(KTextEditor::Document *document);
+
     //!\}
 
     /**
@@ -481,6 +582,16 @@ public:
          */
         explicit EditingTransaction(Document *document);
 
+        /**
+         * No copy constructor, don't allow this to be copied.
+         */
+        EditingTransaction(const EditingTransaction &) = delete;
+
+        /**
+         * No assignment operator, no copying around editing transations.
+         */
+        EditingTransaction &operator=(const EditingTransaction &) = delete;
+
         /**
          * Destructs the object and, if needed, finishes a running editing
          * transaction by calling finish().
@@ -506,11 +617,6 @@ public:
         void finish();
 
     private:
-        /**
-         * no copying allowed
-         */
-        Q_DISABLE_COPY(EditingTransaction)
-
         /**
          * private d-pointer
          */
@@ -543,7 +649,7 @@ public:
      * \return the requested text part, or QString() for invalid ranges.
      * \see setText()
      */
-    virtual QString text(const Range &range, bool block = false) const = 0;
+    virtual QString text(Range range, bool block = false) const = 0;
 
     /**
      * Get the character at text position \p cursor.
@@ -551,7 +657,7 @@ public:
      * \return the requested character, or QChar() for invalid cursors.
      * \see setText()
      */
-    virtual QChar characterAt(const Cursor &position) const = 0;
+    virtual QChar characterAt(KTextEditor::Cursor position) const = 0;
 
     /**
      * Get the word at the text position \p cursor.
@@ -568,7 +674,7 @@ public:
      *
      * \see wordRangeAt(), characterAt()
      */
-    virtual QString wordAt(const KTextEditor::Cursor &cursor) const = 0;
+    virtual QString wordAt(KTextEditor::Cursor cursor) const = 0;
 
     /**
      * Get the text range for the word located under the text position \p cursor.
@@ -586,7 +692,7 @@ public:
      *
      * \see wordAt(), characterAt(), KTextEditor::Range::isValid()
      */
-    virtual KTextEditor::Range wordRangeAt(const KTextEditor::Cursor &cursor) const = 0;
+    virtual KTextEditor::Range wordRangeAt(KTextEditor::Cursor cursor) const = 0;
 
     /**
      * Get whether \p cursor is a valid text position.
@@ -602,7 +708,7 @@ public:
      *
      * \since 5.0
      */
-    virtual bool isValidTextPosition(const KTextEditor::Cursor &cursor) const = 0;
+    virtual bool isValidTextPosition(KTextEditor::Cursor cursor) const = 0;
 
     /**
      * Get the document content within the given \p range.
@@ -613,7 +719,7 @@ public:
      *         no end of line termination is included.
      * \see setText()
      */
-    virtual QStringList textLines(const Range &range, bool block = false) const = 0;
+    virtual QStringList textLines(Range range, bool block = false) const = 0;
 
     /**
      * Get a single text line.
@@ -744,7 +850,7 @@ public:
      * \return \e true on success, otherwise \e false
      * \see setText(), removeText()
      */
-    virtual bool insertText(const Cursor &position, const QString &text, bool block = false) = 0;
+    virtual bool insertText(KTextEditor::Cursor position, const QString &text, bool block = false) = 0;
 
     /**
      * Insert \p text at \p position.
@@ -754,7 +860,7 @@ public:
      * \return \e true on success, otherwise \e false
      * \see setText(), removeText()
      */
-    virtual bool insertText(const Cursor &position, const QStringList &text, bool block = false) = 0;
+    virtual bool insertText(KTextEditor::Cursor position, const QStringList &text, bool block = false) = 0;
 
     /**
      * Replace text from \p range with specified \p text.
@@ -764,7 +870,7 @@ public:
      * \return \e true on success, otherwise \e false
      * \see setText(), removeText(), insertText()
      */
-    virtual bool replaceText(const Range &range, const QString &text, bool block = false);
+    virtual bool replaceText(Range range, const QString &text, bool block = false);
 
     /**
      * Replace text from \p range with specified \p text.
@@ -774,7 +880,7 @@ public:
      * \return \e true on success, otherwise \e false
      * \see setText(), removeText(), insertText()
      */
-    virtual bool replaceText(const Range &range, const QStringList &text, bool block = false);
+    virtual bool replaceText(Range range, const QStringList &text, bool block = false);
 
     /**
      * Remove the text specified in \p range.
@@ -783,7 +889,7 @@ public:
      * \return \e true on success, otherwise \e false
      * \see setText(), insertText()
      */
-    virtual bool removeText(const Range &range, bool block = false) = 0;
+    virtual bool removeText(Range range, bool block = false) = 0;
 
     /**
      * Insert line(s) at the given line number. The newline character '\\n'
@@ -842,8 +948,21 @@ public:
      *
      * \since 5.11
      */
-    QVector<KTextEditor::Range> searchText(const KTextEditor::Range &range, const QString &pattern, const SearchOptions options = Default) const;
+    QList<KTextEditor::Range> searchText(KTextEditor::Range range, const QString &pattern, const SearchOptions options = Default) const;
 
+    /**
+     * \brief Retrives the offset for the given cursor position
+     * NOTE: It will return -1 if the cursor was invalid or out of bounds.
+     * \since 6.0
+     */
+    virtual qsizetype cursorToOffset(KTextEditor::Cursor c) const = 0;
+
+    /**
+     * \brief Retrives the cursor position for given offset
+     * NOTE: It will return an invalid cursor(-1, -1) if offset is invalid.
+     * \since 6.0
+     */
+    virtual KTextEditor::Cursor offsetToCursor(qsizetype offset) const = 0;
     /*
      * SIGNALS
      * Following signals should be emitted by the document if the text content
@@ -874,7 +993,7 @@ Q_SIGNALS:
      * \param document document which emitted this signal
      * @param position position where the wrap occurred
      */
-    void lineWrapped(KTextEditor::Document *document, const KTextEditor::Cursor &position);
+    void lineWrapped(KTextEditor::Document *document, KTextEditor::Cursor position);
 
     /**
      * A line got unwrapped.
@@ -889,7 +1008,7 @@ Q_SIGNALS:
      * @param position position where the insertion occurred
      * @param text inserted text
      */
-    void textInserted(KTextEditor::Document *document, const KTextEditor::Cursor &position, const QString &text);
+    void textInserted(KTextEditor::Document *document, KTextEditor::Cursor position, const QString &text);
 
     /**
      * Text got removed.
@@ -897,7 +1016,7 @@ Q_SIGNALS:
      * @param range range where the removal occurred
      * @param text removed text
      */
-    void textRemoved(KTextEditor::Document *document, const KTextEditor::Range &range, const QString &text);
+    void textRemoved(KTextEditor::Document *document, KTextEditor::Range range, const QString &text);
 
     /**
      * The \p document emits this signal whenever its text changes.
@@ -917,7 +1036,7 @@ public:
     /**
      * Get the default style of the character located at @p position.
      * If @p position is not a valid text position, the default style
-     * DefaultStyle::dsNormal is returned.
+     * KSyntaxHighlighting::Theme::TextStyle::Normal is returned.
      *
      * @note Further information about the colors of default styles depend on
      *       the currently chosen schema. Since each View may have a different
@@ -925,10 +1044,10 @@ public:
      *       View::defaultStyleAttribute() and View::lineAttributes().
      *
      * @param position text position
-     * @return default style, see enum KTextEditor::DefaultStyle
+     * @return default style, see enum KSyntaxHighlighting::Theme::TextStyle
      * @see View::defaultStyleAttribute(), View::lineAttributes()
      */
-    virtual DefaultStyle defaultStyleAt(const KTextEditor::Cursor &position) const = 0;
+    virtual KSyntaxHighlighting::Theme::TextStyle defaultStyleAt(KTextEditor::Cursor position) const = 0;
 
     /**
      * Return the name of the currently used mode
@@ -969,7 +1088,7 @@ public:
      *
      * \see highlightingModes()
      */
-    virtual QString highlightingModeAt(const Cursor &position) = 0;
+    virtual QString highlightingModeAt(KTextEditor::Cursor position) = 0;
 
     /**
      * Return a list of the names of all possible modes
@@ -1158,6 +1277,473 @@ public:
 
     //!\}
 
+Q_SIGNALS:
+    /**
+     * This signal is emitted whenever the current document configuration is changed.
+     *
+     * \param document the document which's config has changed
+     *
+     * \since 5.79
+     */
+    void configChanged(KTextEditor::Document *document);
+
+    /**
+     * \name Moving Interface
+     *
+     * \{
+     */
+public:
+    /**
+     * Create a new moving cursor for this document.
+     * @param position position of the moving cursor to create
+     * @param insertBehavior insertion behavior
+     * @return new moving cursor for the document
+     */
+    virtual MovingCursor *newMovingCursor(KTextEditor::Cursor position, MovingCursor::InsertBehavior insertBehavior = MovingCursor::MoveOnInsert) = 0;
+
+    /**
+     * Create a new moving range for this document.
+     * @param range range of the moving range to create
+     * @param insertBehaviors insertion behaviors
+     * @param emptyBehavior behavior on becoming empty
+     * @return new moving range for the document
+     */
+    virtual MovingRange *newMovingRange(Range range,
+                                        MovingRange::InsertBehaviors insertBehaviors = MovingRange::DoNotExpand,
+                                        MovingRange::EmptyBehavior emptyBehavior = MovingRange::AllowEmpty) = 0;
+
+    /**
+     * Current revision
+     * @return current revision
+     */
+    virtual qint64 revision() const = 0;
+
+    /**
+     * Last revision the buffer got successful saved
+     * @return last revision buffer got saved, -1 if none
+     */
+    virtual qint64 lastSavedRevision() const = 0;
+
+    /**
+     * Lock a revision, this will keep it around until released again.
+     * But all revisions will always be cleared on buffer clear() (and therefor load())
+     * @param revision revision to lock
+     */
+    virtual void lockRevision(qint64 revision) = 0;
+
+    /**
+     * Release a revision.
+     * @param revision revision to release
+     */
+    virtual void unlockRevision(qint64 revision) = 0;
+
+    /**
+     * Transform a cursor from one revision to an other.
+     * @param cursor cursor to transform
+     * @param insertBehavior behavior of this cursor on insert of text at its position
+     * @param fromRevision from this revision we want to transform
+     * @param toRevision to this revision we want to transform, default of -1 is current revision
+     */
+    virtual void
+    transformCursor(KTextEditor::Cursor &cursor, KTextEditor::MovingCursor::InsertBehavior insertBehavior, qint64 fromRevision, qint64 toRevision = -1) = 0;
+
+    /**
+     * Transform a cursor from one revision to an other.
+     * @param line line number of the cursor to transform
+     * @param column column number of the cursor to transform
+     * @param insertBehavior behavior of this cursor on insert of text at its position
+     * @param fromRevision from this revision we want to transform
+     * @param toRevision to this revision we want to transform, default of -1 is current revision
+     */
+    virtual void
+    transformCursor(int &line, int &column, KTextEditor::MovingCursor::InsertBehavior insertBehavior, qint64 fromRevision, qint64 toRevision = -1) = 0;
+
+    /**
+     * Transform a range from one revision to an other.
+     * @param range range to transform
+     * @param insertBehaviors behavior of this range on insert of text at its position
+     * @param emptyBehavior behavior on becoming empty
+     * @param fromRevision from this revision we want to transform
+     * @param toRevision to this revision we want to transform, default of -1 is current revision
+     */
+    virtual void transformRange(KTextEditor::Range &range,
+                                KTextEditor::MovingRange::InsertBehaviors insertBehaviors,
+                                MovingRange::EmptyBehavior emptyBehavior,
+                                qint64 fromRevision,
+                                qint64 toRevision = -1) = 0;
+
+Q_SIGNALS:
+    /**
+     * This signal is emitted before the cursors/ranges/revisions of a document
+     * are destroyed as the document is deleted.
+     * @param document the document which the interface belongs to which is in the process of being deleted
+     */
+    void aboutToDeleteMovingInterfaceContent(KTextEditor::Document *document);
+
+    /**
+     * This signal is emitted before the ranges of a document are invalidated
+     * and the revisions are deleted as the document is cleared (for example on
+     * load/reload). While this signal is emitted, the old document content is
+     * still valid and accessible before the clear.
+     * @param document the document which the interface belongs to which will invalidate its data
+     */
+    void aboutToInvalidateMovingInterfaceContent(KTextEditor::Document *document);
+
+    //!\}
+
+    /**
+     * \name Config
+     *
+     * \{
+     */
+public:
+    /**
+     * \brief Get a list of all available keys.
+     */
+    virtual QStringList configKeys() const = 0;
+    /**
+     * \brief Get a value for the \p key.
+     */
+    virtual QVariant configValue(const QString &key) = 0;
+    /**
+     * \brief Set a the \p key's value to \p value.
+     */
+    virtual void setConfigValue(const QString &key, const QVariant &value) = 0;
+    //!\}
+
+    /**
+     * \name Modification Interface
+     *
+     * \{
+     */
+public:
+    /**
+     * Reasons why a document is modified on disk.
+     */
+    enum ModifiedOnDiskReason {
+        OnDiskUnmodified = 0, ///< Not modified
+        OnDiskModified = 1, ///< The file was modified on disk
+        OnDiskCreated = 2, ///< The file was created on disk
+        OnDiskDeleted = 3 ///< The file was deleted on disk
+    };
+
+    /**
+     * Set the document's modified-on-disk state to \p reason.
+     * KTextEditor implementations should emit the signal modifiedOnDisk()
+     * along with the reason. When the document is in a clean state again the
+     * reason should be ModifiedOnDiskReason::OnDiskUnmodified.
+     *
+     * \param reason the modified-on-disk reason.
+     * \see ModifiedOnDiskReason, modifiedOnDisk()
+     */
+    virtual void setModifiedOnDisk(ModifiedOnDiskReason reason) = 0;
+
+    /**
+     * Control, whether the editor should show a warning dialog whenever a file
+     * was modified on disk. If \p on is \e true the editor will show warning
+     * dialogs.
+     * \param on controls, whether the editor should show a warning dialog for
+     *        files modified on disk
+     */
+    virtual void setModifiedOnDiskWarning(bool on) = 0;
+
+Q_SIGNALS:
+    /**
+     * This signal is emitted whenever the \p document changed its
+     * modified-on-disk state.
+     * \param document the Document object that represents the file on disk
+     * \param isModified if \e true, the file was modified rather than created
+     *        or deleted
+     * \param reason the reason why the signal was emitted
+     * \see setModifiedOnDisk()
+     */
+    void modifiedOnDisk(KTextEditor::Document *document, bool isModified, KTextEditor::Document::ModifiedOnDiskReason reason);
+    //!\}
+
+    /**
+     * \name Mark Interface
+     *
+     * \{
+     */
+public:
+    /**
+     * Get all marks set on the \p line.
+     * \param line requested line
+     * \return a \e uint representing of the marks set in \p line concatenated
+     *         by logical OR
+     * \see addMark(), removeMark()
+     */
+    virtual uint mark(int line) = 0;
+
+    /**
+     * Set the \p line's mark types to \p markType.
+     * If \p line already contains a mark of the given type it has no effect.
+     * All other marks are deleted before the mark is set. You can achieve
+     * the same by calling
+     * \code
+     * clearMark(line);
+     * addMark(line, markType);
+     * \endcode
+     * \param line line to set the mark
+     * \param markType mark type
+     * \see clearMark(), addMark(), mark()
+     */
+    virtual void setMark(int line, uint markType) = 0;
+
+    /**
+     * Clear all marks set in the \p line.
+     * \param line line to clear marks
+     * \see clearMarks(), removeMark(), addMark()
+     */
+    virtual void clearMark(int line) = 0;
+
+    /**
+     * Add marks of type \p markType to \p line. Existing marks on this line
+     * are preserved. If the mark \p markType already is set, nothing
+     * happens.
+     * \param line line to set the mark
+     * \param markType mark type
+     * \see removeMark(), setMark()
+     */
+    virtual void addMark(int line, uint markType) = 0;
+
+    /**
+     * Remove the mark mask of type \p markType from \p line.
+     * \param line line to remove the mark
+     * \param markType mark type to be removed
+     * \see clearMark()
+     */
+    virtual void removeMark(int line, uint markType) = 0;
+
+    /**
+     * Get a hash holding all marks in the document.
+     * The hash key for a mark is its line.
+     * \return a hash holding all marks in the document
+     *
+     * KF6 TODO: Change Mark* to Mark. No need for pointer here.
+     */
+    virtual const QHash<int, KTextEditor::Mark *> &marks() = 0;
+
+    /**
+     * Clear all marks in the entire document.
+     * \see clearMark(), removeMark()
+     */
+    /// TODO: dominik: add argument unit mask = 0
+    virtual void clearMarks() = 0;
+
+    /**
+     * Get the number of predefined mark types we have so far.
+     * \note FIXME: If you change this you have to make sure katepart
+     *              supports the new size!
+     * \return number of reserved marker types
+     */
+    static int reservedMarkersCount()
+    {
+        return 7;
+    }
+
+    /**
+     * Predefined mark types.
+     *
+     * To add a new standard mark type, edit this interface and document
+     * the type.
+     */
+    enum MarkTypes {
+        /** Bookmark */
+        markType01 = 0x1,
+        /** Breakpoint active */
+        markType02 = 0x2,
+        /** Breakpoint reached */
+        markType03 = 0x4,
+        /** Breakpoint disabled */
+        markType04 = 0x8,
+        /** Execution mark */
+        markType05 = 0x10,
+        /** Warning */
+        markType06 = 0x20,
+        /** Error */
+        markType07 = 0x40,
+
+        markType08 = 0x80,
+        markType09 = 0x100,
+        markType10 = 0x200,
+        markType11 = 0x400,
+        markType12 = 0x800,
+        markType13 = 0x1000,
+        markType14 = 0x2000,
+        markType15 = 0x4000,
+        markType16 = 0x8000,
+        markType17 = 0x10000,
+        markType18 = 0x20000,
+        markType19 = 0x40000,
+        markType20 = 0x80000,
+        markType21 = 0x100000,
+        markType22 = 0x200000,
+        markType23 = 0x400000,
+        markType24 = 0x800000,
+        markType25 = 0x1000000,
+        markType26 = 0x2000000,
+        markType27 = 0x4000000,
+        markType28 = 0x8000000,
+        markType29 = 0x10000000,
+        markType30 = 0x20000000,
+        markType31 = 0x40000000,
+        markType32 = 0x80000000,
+        /* reserved marks */
+        Bookmark = markType01,
+        BreakpointActive = markType02,
+        BreakpointReached = markType03,
+        BreakpointDisabled = markType04,
+        Execution = markType05,
+        Warning = markType06,
+        Error = markType07,
+        SearchMatch = markType32,
+    };
+
+    /*
+     * Methods to modify mark properties.
+     */
+public:
+    /**
+     * Set the \p mark's description to \p text.
+     * \param mark mark to set the description
+     * \param text new descriptive text
+     * \see markDescription(), setMarkPixmap()
+     */
+    virtual void setMarkDescription(MarkTypes mark, const QString &text) = 0;
+
+    /**
+     * Get the \p mark's description to text.
+     * \param mark mark to set the description
+     * \return text of the given \p mark or QString(), if the entry does not
+     *         exist
+     * \see setMarkDescription(), setMarkPixmap()
+     */
+    virtual QString markDescription(MarkTypes mark) const = 0;
+
+    /**
+     * Set the mark mask the user is allowed to toggle to \p markMask.
+     * I.e. concatenate all editable marks with a logical OR. If the user should
+     * be able to add a bookmark and set a breakpoint with the context menu in
+     * the icon pane, you have to call
+     * \code
+     * // iface is of Type KTextEditor::MarkInterface*
+     * // only make bookmark and breakpoint editable
+     * iface->setEditableMarks( MarkInterface::Bookmark |
+     *                          MarkInterface::BreakpointActive );
+     *
+     * // or preserve last settings, and add bookmark and breakpoint
+     * iface->setEditableMarks( iface->editableMarks() |
+     *                          MarkInterface::Bookmark |
+     *                          MarkInterface::BreakpointActive );
+     * \endcode
+     * \param markMask bitmap pattern
+     * \see editableMarks(), setMarkPixmap(), setMarkDescription()
+     */
+    virtual void setEditableMarks(uint markMask) = 0;
+
+    /**
+     * Get, which marks can be toggled by the user.
+     * The returned value is a mark mask containing all editable marks combined
+     * with a logical OR.
+     * \return mark mask containing all editable marks
+     * \see setEditableMarks()
+     */
+    virtual uint editableMarks() const = 0;
+
+    /**
+     * Possible actions on a mark.
+     * \see markChanged()
+     */
+    enum MarkChangeAction {
+        MarkAdded = 0, /**< action: a mark was added.  */
+        MarkRemoved = 1 /**< action: a mark was removed. */
+    };
+
+    /**
+     * Set the \p mark's icon to \p icon.
+     * \param markType mark type to which the icon will be attached
+     * \param icon new icon
+     * \see setMarkDescription()
+     */
+    virtual void setMarkIcon(MarkTypes markType, const QIcon &icon) = 0;
+
+    /**
+     * Get the \p mark's icon.
+     * \param markType mark type. If the icon does not exist the resulting is null
+     *        (check with QIcon::isNull()).
+     * \see setMarkDescription()
+     */
+    virtual QIcon markIcon(MarkTypes markType) const = 0;
+
+Q_SIGNALS:
+    /**
+     * The \p document emits this signal whenever a mark mask changed.
+     * \param document document which emitted this signal
+     * \see markChanged()
+     */
+    void marksChanged(KTextEditor::Document *document);
+
+    /**
+     * The \p document emits this signal whenever the \p mark changes.
+     * \param document the document which emitted the signal
+     * \param mark changed mark
+     * \param action action, either removed or added
+     * \see marksChanged()
+     */
+    void markChanged(KTextEditor::Document *document, KTextEditor::Mark mark, KTextEditor::Document::MarkChangeAction action);
+
+    /**
+     * The \p document emits this signal whenever the \p mark is hovered using the mouse,
+     * and the receiver may show a tooltip.
+     * \param document the document which emitted the signal
+     * \param mark mark that was hovered
+     * \param position mouse position during the hovering
+     * \param handled set this to 'true' if this event was handled externally
+     */
+    void markToolTipRequested(KTextEditor::Document *document, KTextEditor::Mark mark, QPoint position, bool &handled);
+
+    /**
+     * The \p document emits this signal whenever the \p mark is right-clicked to show a context menu.
+     * The receiver may show an own context menu instead of the kate internal one.
+     * \param document the document which emitted the signal
+     * \param mark mark that was right-clicked
+     * \param pos position where the menu should be started
+     * \param handled set this to 'true' if this event was handled externally, and kate should not create an own context menu.
+     */
+    void markContextMenuRequested(KTextEditor::Document *document, KTextEditor::Mark mark, QPoint pos, bool &handled);
+
+    /**
+     * The \p document emits this signal whenever the \p mark is left-clicked.
+     * \param document the document which emitted the signal
+     * \param mark mark that was right-clicked
+     * \param handled set this to 'true' if this event was handled externally, and kate should not do own handling of the left click.
+     */
+    void markClicked(KTextEditor::Document *document, KTextEditor::Mark mark, bool &handled);
+    //!\}
+
+    /**
+     * \name Annotation Interface
+     *
+     * \{
+     */
+public:
+    /**
+     * Sets a new \ref AnnotationModel for this document to provide
+     * annotation information for each line.
+     *
+     * \param model the new AnnotationModel
+     */
+    virtual void setAnnotationModel(AnnotationModel *model) = 0;
+
+    /**
+     * returns the currently set \ref AnnotationModel or 0 if there's none
+     * set
+     * @returns the current \ref AnnotationModel
+     */
+    virtual AnnotationModel *annotationModel() const = 0;
+    //!\}
+
 private:
     /**
      * private d-pointer, pointing to the internal implementation
@@ -1167,6 +1753,4 @@ private:
 
 }
 
-Q_DECLARE_METATYPE(KTextEditor::Document *)
-
 #endif
