@@ -1,41 +1,28 @@
-/* This file is part of the KDE project
- *
- *  Copyright (C) 2010 Christoph Cullmann <cullmann@kde.org>
- *
- *  Based on code of the SmartCursor/Range by:
- *  Copyright (C) 2003-2005 Hamish Rodda <rodda@kde.org>
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
- */
+/*
+    SPDX-FileCopyrightText: 2010 Christoph Cullmann <cullmann@kde.org>
+
+    Based on code of the SmartCursor/Range by:
+    SPDX-FileCopyrightText: 2003-2005 Hamish Rodda <rodda@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #ifndef KTEXTEDITOR_MOVINGCURSOR_H
 #define KTEXTEDITOR_MOVINGCURSOR_H
 
-#include <ktexteditor_export.h>
 #include <ktexteditor/cursor.h>
-#include <ktexteditor/document.h>
+#include <ktexteditor_export.h>
 
 #include <QDebug>
 
 namespace KTextEditor
 {
-
 class MovingRange;
+class Document;
 
 /**
+ * \class MovingCursor movingcursor.h <KTextEditor/MovingCursor>
+ *
  * \short A Cursor which is bound to a specific Document, and maintains its position.
  *
  * \ingroup kte_group_moving_classes
@@ -76,14 +63,14 @@ public:
      */
     enum InsertBehavior {
         StayOnInsert = 0x0, ///< stay on insert
-        MoveOnInsert = 0x1  ///< move on insert
+        MoveOnInsert = 0x1 ///< move on insert
     };
 
     /**
      * Wrap behavior for end of line treatement used in move().
      */
     enum WrapBehavior {
-        Wrap = 0x0,  ///< wrap at end of line
+        Wrap = 0x0, ///< wrap at end of line
         NoWrap = 0x1 ///< do not wrap at end of line
     };
 
@@ -120,7 +107,7 @@ public:
      *
      * \param position new cursor position
      */
-    virtual void setPosition(const KTextEditor::Cursor &position) = 0;
+    virtual void setPosition(KTextEditor::Cursor position) = 0;
 
     /**
      * Retrieve the line on which this cursor is situated.
@@ -148,29 +135,29 @@ protected:
      */
     MovingCursor();
 
-private:
+public:
     /**
      * no copy constructor, don't allow this to be copied.
      */
-    MovingCursor(const MovingCursor &);
+    MovingCursor(const MovingCursor &) = delete;
 
     /**
      * no assignment operator, no copying around clever cursors.
      */
-    MovingCursor &operator= (const MovingCursor &);
+    MovingCursor &operator=(const MovingCursor &) = delete;
 
     //
     // convenience API
     //
 public:
-
     /**
      * Returns whether the current position of this cursor is a valid position,
      * i.e. whether line() >= 0 and column() >= 0.
      *
      * \return \e true , if the cursor position is valid, otherwise \e false
      */
-    inline bool isValid() const {
+    inline bool isValid() const
+    {
         return line() >= 0 && column() >= 0;
     }
 
@@ -187,9 +174,7 @@ public:
      *
      * \see Document::isValidTextPosition()
      */
-    inline bool isValidTextPosition() const {
-        return document()->isValidTextPosition(toCursor());
-    }
+    bool isValidTextPosition() const;
 
     /**
      * \overload
@@ -278,7 +263,8 @@ public:
      * Even if this cursor belongs to a range, the created one not.
      * @return normal cursor
      */
-    const Cursor toCursor() const {
+    const Cursor toCursor() const
+    {
         return Cursor(line(), column());
     }
 
@@ -287,13 +273,14 @@ public:
      * Even if this cursor belongs to a range, the created one not.
      * @return normal cursor
      */
-    operator Cursor() const {
+    operator Cursor() const
+    {
         return Cursor(line(), column());
     }
 
-//
-// operators for: MovingCursor <-> MovingCursor
-//
+    //
+    // operators for: MovingCursor <-> MovingCursor
+    //
     /**
      * Equality operator.
      *
@@ -304,7 +291,8 @@ public:
      * \param c2 second cursor to compare
      * \return \e true, if c1's and c2's line and column are \e equal.
      */
-    inline friend bool operator==(const MovingCursor &c1, const MovingCursor &c2) {
+    inline friend bool operator==(const MovingCursor &c1, const MovingCursor &c2)
+    {
         return c1.line() == c2.line() && c1.column() == c2.column();
     }
 
@@ -314,7 +302,8 @@ public:
      * \param c2 second cursor to compare
      * \return \e true, if c1's and c2's line and column are \e not equal.
      */
-    inline friend bool operator!=(const MovingCursor &c1, const MovingCursor &c2) {
+    inline friend bool operator!=(const MovingCursor &c1, const MovingCursor &c2)
+    {
         return !(c1 == c2);
     }
 
@@ -325,7 +314,8 @@ public:
      * \return \e true, if c1's position is greater than c2's position,
      *         otherwise \e false.
      */
-    inline friend bool operator>(const MovingCursor &c1, const MovingCursor &c2) {
+    inline friend bool operator>(const MovingCursor &c1, const MovingCursor &c2)
+    {
         return c1.line() > c2.line() || (c1.line() == c2.line() && c1.column() > c2.column());
     }
 
@@ -336,7 +326,8 @@ public:
      * \return \e true, if c1's position is greater than or equal to c2's
      *         position, otherwise \e false.
      */
-    inline friend bool operator>=(const MovingCursor &c1, const MovingCursor &c2) {
+    inline friend bool operator>=(const MovingCursor &c1, const MovingCursor &c2)
+    {
         return c1.line() > c2.line() || (c1.line() == c2.line() && c1.column() >= c2.column());
     }
 
@@ -347,7 +338,8 @@ public:
      * \return \e true, if c1's position is greater than or equal to c2's
      *         position, otherwise \e false.
      */
-    inline friend bool operator<(const MovingCursor &c1, const MovingCursor &c2) {
+    inline friend bool operator<(const MovingCursor &c1, const MovingCursor &c2)
+    {
         return !(c1 >= c2);
     }
 
@@ -358,7 +350,8 @@ public:
      * \return \e true, if c1's position is lesser than or equal to c2's
      *         position, otherwise \e false.
      */
-    inline friend bool operator<=(const MovingCursor &c1, const MovingCursor &c2) {
+    inline friend bool operator<=(const MovingCursor &c1, const MovingCursor &c2)
+    {
         return !(c1 > c2);
     }
 
@@ -368,7 +361,8 @@ public:
      * @param cursor cursor to print
      * @return debug stream
      */
-    inline friend QDebug operator<< (QDebug s, const MovingCursor *cursor) {
+    inline friend QDebug operator<<(QDebug s, const MovingCursor *cursor)
+    {
         if (cursor) {
             s.nospace() << "(" << cursor->line() << ", " << cursor->column() << ")";
         } else {
@@ -383,7 +377,8 @@ public:
      * @param cursor cursor to print
      * @return debug stream
      */
-    inline friend QDebug operator<< (QDebug s, const MovingCursor &cursor) {
+    inline friend QDebug operator<<(QDebug s, const MovingCursor &cursor)
+    {
         return s << &cursor;
     }
 };
@@ -391,4 +386,3 @@ public:
 }
 
 #endif
-
