@@ -1,30 +1,17 @@
-/*  SPDX-License-Identifier: LGPL-2.0-or-later
-
-    Copyright (C) 2010 Christoph Cullmann <cullmann@kde.org>
+/*
+    SPDX-FileCopyrightText: 2010 Christoph Cullmann <cullmann@kde.org>
 
     Based on code of the SmartCursor/Range by:
-    Copyright (C) 2003-2005 Hamish Rodda <rodda@kde.org>
+    SPDX-FileCopyrightText: 2003-2005 Hamish Rodda <rodda@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KTEXTEDITOR_MOVINGRANGE_H
 #define KTEXTEDITOR_MOVINGRANGE_H
 
 #include <ktexteditor/attribute.h>
+#include <ktexteditor/linerange.h>
 #include <ktexteditor/movingcursor.h>
 #include <ktexteditor/range.h>
 #include <ktexteditor_export.h>
@@ -38,6 +25,8 @@ class View;
 class MovingRangeFeedback;
 
 /**
+ * \class MovingRange movingrange.h <KTextEditor/MovingRange>
+ *
  * \short A range that is bound to a specific Document, and maintains its
  *  position.
  *
@@ -159,6 +148,7 @@ class KTEXTEDITOR_EXPORT MovingRange
     //
 public:
     /// Determine how the range reacts to characters inserted immediately outside the range.
+    /// @see InsertBehaviors
     enum InsertBehavior {
         /// Don't expand to encapsulate new characters in either direction. This is the default.
         DoNotExpand = 0x0,
@@ -167,13 +157,14 @@ public:
         /// Expand to encapsulate new characters to the right of the range.
         ExpandRight = 0x2
     };
+    /// Stores a combination of #InsertBehavior values.
     Q_DECLARE_FLAGS(InsertBehaviors, InsertBehavior)
 
     /**
      * Behavior of range if it becomes empty.
      */
     enum EmptyBehavior {
-        AllowEmpty = 0x0,       ///< allow range to be empty
+        AllowEmpty = 0x0, ///< allow range to be empty
         InvalidateIfEmpty = 0x1 ///< invalidate range, if it becomes empty
     };
 
@@ -217,7 +208,7 @@ public:
      * automatically invalid!
      * @param range new range for this clever range
      */
-    virtual void setRange(const KTextEditor::Range &range) = 0;
+    virtual void setRange(KTextEditor::Range range) = 0;
 
     /**
      * Retrieve start cursor of this range, read-only.
@@ -253,7 +244,7 @@ public:
      *
      * \return a pointer to the active attribute
      */
-    virtual Attribute::Ptr attribute() const = 0;
+    virtual const Attribute::Ptr &attribute() const = 0;
 
     /**
      * Sets the currently active attribute for this range.
@@ -334,16 +325,16 @@ protected:
      */
     MovingRange();
 
-private:
+public:
     /**
      * no copy constructor, don't allow this to be copied.
      */
-    MovingRange(const MovingRange &);
+    MovingRange(const MovingRange &) = delete;
 
     /**
      * no assignment operator, no copying around clever ranges.
      */
-    MovingRange &operator=(const MovingRange &);
+    MovingRange &operator=(const MovingRange &) = delete;
 
     //
     // convenience API
@@ -357,7 +348,7 @@ public:
      * @param start new start for this clever range
      * @param end new end for this clever range
      */
-    void setRange(const Cursor &start, const Cursor &end);
+    void setRange(Cursor start, Cursor end);
 
     /**
      * Convert this clever range into a dumb one.
@@ -377,6 +368,15 @@ public:
         return Range(start().toCursor(), end().toCursor());
     }
 
+    /**
+     * Convert this MovingRange to a simple LineRange.
+     * @return LineRange from the start line to the end line of this range.
+     */
+    inline LineRange toLineRange() const Q_DECL_NOEXCEPT
+    {
+        return {start().line(), end().line()};
+    }
+
     /**
      * qDebug() stream operator. Writes this range to the debug output in a nicely formatted way.
      * @param s debug stream
@@ -441,7 +441,7 @@ public:
      *
      * \return \e true if the cursor is contained within this range, otherwise \e false.
      */
-    inline bool contains(const Cursor &cursor) const
+    inline bool contains(Cursor cursor) const
     {
         return cursor >= start() && cursor < end();
     }
