@@ -11,12 +11,17 @@
 #ifndef KTEXTEDITOR_RANGE_H
 #define KTEXTEDITOR_RANGE_H
 
-#include <ktexteditor/cursor.h>
 #include <ktexteditor_export.h>
 
-#include <QDebug>
+#include <ktexteditor/cursor.h>
+#include <ktexteditor/linerange.h>
+
 #include <QtGlobal>
 
+class QDebug;
+class QString;
+class QStringView;
+
 namespace KTextEditor
 {
 /**
@@ -47,9 +52,7 @@ public:
      * Default constructor. Creates a valid range from position (0, 0) to
      * position (0, 0).
      */
-    Q_DECL_CONSTEXPR Range() Q_DECL_NOEXCEPT
-    {
-    }
+    constexpr Range() noexcept = default;
 
     /**
      * Constructor which creates a range from \e start to \e end.
@@ -58,7 +61,9 @@ public:
      * \param start start position
      * \param end end position
      */
-    Q_DECL_CONSTEXPR Range(const Cursor &start, const Cursor &end) Q_DECL_NOEXCEPT : m_start(qMin(start, end)), m_end(qMax(start, end))
+    constexpr Range(Cursor start, Cursor end) noexcept
+        : m_start(qMin(start, end))
+        , m_end(qMax(start, end))
     {
     }
 
@@ -69,7 +74,9 @@ public:
      * \param start start position
      * \param width width of this range in columns along the same line
      */
-    Q_DECL_CONSTEXPR Range(const Cursor &start, int width) Q_DECL_NOEXCEPT : m_start(qMin(start, Cursor(start.line(), start.column() + width))), m_end(qMax(start, Cursor(start.line(), start.column() + width)))
+    constexpr Range(Cursor start, int width) noexcept
+        : m_start(qMin(start, Cursor(start.line(), start.column() + width)))
+        , m_end(qMax(start, Cursor(start.line(), start.column() + width)))
     {
     }
 
@@ -80,7 +87,9 @@ public:
      * \param endLine end line
      * \param endColumn end column
      */
-    Q_DECL_CONSTEXPR Range(const Cursor &start, int endLine, int endColumn) Q_DECL_NOEXCEPT : m_start(qMin(start, Cursor(endLine, endColumn))), m_end(qMax(start, Cursor(endLine, endColumn)))
+    constexpr Range(Cursor start, int endLine, int endColumn) noexcept
+        : m_start(qMin(start, Cursor(endLine, endColumn)))
+        , m_end(qMax(start, Cursor(endLine, endColumn)))
     {
     }
 
@@ -92,15 +101,16 @@ public:
      * \param endLine end line
      * \param endColumn end column
      */
-    Q_DECL_CONSTEXPR Range(int startLine, int startColumn, int endLine, int endColumn) Q_DECL_NOEXCEPT : m_start(qMin(Cursor(startLine, startColumn), Cursor(endLine, endColumn))),
-                                                                                                         m_end(qMax(Cursor(startLine, startColumn), Cursor(endLine, endColumn)))
+    constexpr Range(int startLine, int startColumn, int endLine, int endColumn) noexcept
+        : m_start(qMin(Cursor(startLine, startColumn), Cursor(endLine, endColumn)))
+        , m_end(qMax(Cursor(startLine, startColumn), Cursor(endLine, endColumn)))
     {
     }
 
     /**
      * Validity check.  In the base class, returns true unless the range starts before (0,0).
      */
-    Q_DECL_CONSTEXPR inline bool isValid() const Q_DECL_NOEXCEPT
+    constexpr bool isValid() const noexcept
     {
         return start().isValid() && end().isValid();
     }
@@ -108,7 +118,7 @@ public:
     /**
      * Returns an invalid range.
      */
-    Q_DECL_CONSTEXPR static Range invalid() Q_DECL_NOEXCEPT
+    constexpr static Range invalid() noexcept
     {
         return Range(Cursor::invalid(), Cursor::invalid());
     }
@@ -118,10 +128,7 @@ public:
      * "start-line:start-column,endl-line:end-column".
      * \see fromString()
      */
-    QString toString() const
-    {
-        return QLatin1Char('[') + m_start.toString() + QLatin1String(", ") + m_end.toString() + QLatin1Char(']');
-    }
+    QString toString() const;
 
     /**
      * Returns a Range created from the string \p str containing the format
@@ -129,18 +136,7 @@ public:
      * In case the string cannot be parsed, an Range::invalid() is returned.
      * \see toString()
      */
-    static Range fromString(const QString &str) Q_DECL_NOEXCEPT
-    {
-        return fromString(str.leftRef(-1));
-    }
-
-    /**
-     * Returns a Range created from the string \p str containing the format
-     * "[(start-line, start-column), (endl-line:end-column)]".
-     * In case the string cannot be parsed, an Range::invalid() is returned.
-     * \see toString()
-     */
-    static Range fromString(const QStringRef &str) Q_DECL_NOEXCEPT;
+    static Range fromString(QStringView str) noexcept;
 
     /**
      * \name Position
@@ -154,7 +150,7 @@ public:
      *
      * \returns const reference to the start position of this range.
      */
-    Q_DECL_CONSTEXPR inline Cursor start() const Q_DECL_NOEXCEPT
+    constexpr Cursor start() const noexcept
     {
         return m_start;
     }
@@ -164,31 +160,41 @@ public:
      *
      * \returns const reference to the end position of this range.
      */
-    Q_DECL_CONSTEXPR inline Cursor end() const Q_DECL_NOEXCEPT
+    constexpr Cursor end() const noexcept
     {
         return m_end;
     }
 
+    /**
+     * Convert this Range to a LineRange
+     *
+     * @return LineRange from the start line to the end line of this range.
+     */
+    constexpr LineRange toLineRange() const noexcept
+    {
+        return {start().line(), end().line()};
+    }
+
     /**
      * Convenience function.  Set the start and end lines to \p line.
      *
      * \param line the line number to assign to start() and end()
      */
-    void setBothLines(int line) Q_DECL_NOEXCEPT;
+    void setBothLines(int line) noexcept;
 
     /**
      * Convenience function.  Set the start and end columns to \p column.
      *
      * \param column the column number to assign to start() and end()
      */
-    void setBothColumns(int column) Q_DECL_NOEXCEPT;
+    void setBothColumns(int column) noexcept;
 
     /**
      * Set the start and end cursors to \e range.start() and \e range.end() respectively.
      *
      * \param range range to assign to this range
      */
-    void setRange(const Range &range) Q_DECL_NOEXCEPT;
+    void setRange(Range range) noexcept;
 
     /**
      * \overload
@@ -200,7 +206,7 @@ public:
      * \param start start cursor
      * \param end end cursor
      */
-    void setRange(const Cursor &start, const Cursor &end) Q_DECL_NOEXCEPT;
+    void setRange(Cursor start, Cursor end) noexcept;
 
     /**
      * Set the start cursor to \e start.
@@ -209,7 +215,7 @@ public:
      *
      * \param start new start cursor
      */
-    inline void setStart(const Cursor &start) Q_DECL_NOEXCEPT
+    void setStart(Cursor start) noexcept
     {
         if (start > end()) {
             setRange(start, start);
@@ -225,7 +231,7 @@ public:
      *
      * \param end new end cursor
      */
-    inline void setEnd(const Cursor &end) Q_DECL_NOEXCEPT
+    void setEnd(Cursor end) noexcept
     {
         if (end < start()) {
             setRange(end, end);
@@ -241,7 +247,7 @@ public:
      *
      * \return \e true if expansion occurred, \e false otherwise
      */
-    bool expandToRange(const Range &range) Q_DECL_NOEXCEPT;
+    bool expandToRange(Range range) noexcept;
 
     /**
      * Confine this range if necessary to fit within \p range.
@@ -250,7 +256,7 @@ public:
      *
      * \return \e true if confinement occurred, \e false otherwise
      */
-    bool confineToRange(const Range &range) Q_DECL_NOEXCEPT;
+    bool confineToRange(Range range) noexcept;
 
     /**
      * Check whether this range is wholly contained within one line, ie. if
@@ -259,7 +265,7 @@ public:
      * \return \e true if both the start and end positions are on the same
      *         line, otherwise \e false
      */
-    Q_DECL_CONSTEXPR inline bool onSingleLine() const Q_DECL_NOEXCEPT
+    constexpr bool onSingleLine() const noexcept
     {
         return start().line() == end().line();
     }
@@ -270,7 +276,7 @@ public:
      * \return the number of lines separating the start() and end() positions;
      *         0 if the start and end lines are the same.
      */
-    Q_DECL_CONSTEXPR inline int numberOfLines() const Q_DECL_NOEXCEPT
+    constexpr int numberOfLines() const noexcept
     {
         return end().line() - start().line();
     }
@@ -281,7 +287,7 @@ public:
      * \return the number of columns separating the start() and end() positions;
      *         0 if the start and end columns are the same.
      */
-    Q_DECL_CONSTEXPR inline int columnWidth() const Q_DECL_NOEXCEPT
+    constexpr int columnWidth() const noexcept
     {
         return end().column() - start().column();
     }
@@ -292,7 +298,7 @@ public:
      *
      * \returns \e true if the range contains no characters, otherwise \e false
      */
-    Q_DECL_CONSTEXPR inline bool isEmpty() const Q_DECL_NOEXCEPT
+    constexpr bool isEmpty() const noexcept
     {
         return start() == end();
     }
@@ -314,7 +320,7 @@ public:
      *
      * \return \e true, if this range contains \e range, otherwise \e false
      */
-    Q_DECL_CONSTEXPR inline bool contains(const Range &range) const Q_DECL_NOEXCEPT
+    constexpr bool contains(Range range) const noexcept
     {
         return range.start() >= start() && range.end() <= end();
     }
@@ -326,7 +332,7 @@ public:
      *
      * \return \e true if the cursor is contained within this range, otherwise \e false.
      */
-    Q_DECL_CONSTEXPR inline bool contains(const Cursor &cursor) const Q_DECL_NOEXCEPT
+    constexpr bool contains(Cursor cursor) const noexcept
     {
         return cursor >= start() && cursor < end();
     }
@@ -338,7 +344,7 @@ public:
      *
      * \return \e true if the line is wholly encompassed by this range, otherwise \e false.
      */
-    Q_DECL_CONSTEXPR inline bool containsLine(int line) const Q_DECL_NOEXCEPT
+    constexpr bool containsLine(int line) const noexcept
     {
         return (line > start().line() || (line == start().line() && !start().column())) && line < end().line();
     }
@@ -350,7 +356,7 @@ public:
      *
      * \return \e true if the range contains \e column, otherwise \e false
      */
-    Q_DECL_CONSTEXPR inline bool containsColumn(int column) const Q_DECL_NOEXCEPT
+    constexpr bool containsColumn(int column) const noexcept
     {
         return column >= start().column() && column < end().column();
     }
@@ -362,7 +368,7 @@ public:
      *
      * \return \e true, if this range overlaps with \e range, otherwise \e false
      */
-    Q_DECL_CONSTEXPR inline bool overlaps(const Range &range) const Q_DECL_NOEXCEPT
+    constexpr bool overlaps(Range range) const noexcept
     {
         return (range.start() <= start()) ? (range.end() > start()) : (range.end() >= end()) ? (range.start() < end()) : contains(range);
     }
@@ -374,7 +380,7 @@ public:
      *
      * \return \e true, if the range overlaps at least part of \e line, otherwise \e false
      */
-    Q_DECL_CONSTEXPR inline bool overlapsLine(int line) const Q_DECL_NOEXCEPT
+    constexpr bool overlapsLine(int line) const noexcept
     {
         return line >= start().line() && line <= end().line();
     }
@@ -389,7 +395,7 @@ public:
      * \return \e true if the column is between the range's starting and ending
      *         columns, otherwise \e false.
      */
-    Q_DECL_CONSTEXPR inline bool overlapsColumn(int column) const Q_DECL_NOEXCEPT
+    constexpr bool overlapsColumn(int column) const noexcept
     {
         return start().column() <= column && end().column() > column;
     }
@@ -403,7 +409,7 @@ public:
      * \return \e true if the cursor is equal to \p start() or \p end(),
      *         otherwise \e false.
      */
-    Q_DECL_CONSTEXPR inline bool boundaryAtCursor(const Cursor &cursor) const Q_DECL_NOEXCEPT
+    constexpr bool boundaryAtCursor(Cursor cursor) const noexcept
     {
         return cursor == start() || cursor == end();
     }
@@ -418,7 +424,7 @@ public:
      *
      * \return the intersection of this range and the supplied \a range.
      */
-    Q_DECL_CONSTEXPR inline Range intersect(const Range &range) const Q_DECL_NOEXCEPT
+    constexpr Range intersect(Range range) const noexcept
     {
         return ((!isValid() || !range.isValid() || *this > range || *this < range)) ? invalid() : Range(qMax(start(), range.start()), qMin(end(), range.end()));
     }
@@ -431,9 +437,11 @@ public:
      *
      * \return the smallest range which contains this range and the supplied \a range.
      */
-    Q_DECL_CONSTEXPR inline Range encompass(const Range &range) const Q_DECL_NOEXCEPT
+    constexpr Range encompass(Range range) const noexcept
     {
-        return (!isValid()) ? (range.isValid() ? range : invalid()) : (!range.isValid()) ? (*this) : Range(qMin(start(), range.start()), qMax(end(), range.end()));
+        return (!isValid())      ? (range.isValid() ? range : invalid())
+            : (!range.isValid()) ? (*this)
+                                 : Range(qMin(start(), range.start()), qMax(end(), range.end()));
     }
 
     /**
@@ -444,7 +452,7 @@ public:
      *
      * \return a the summation of the two input ranges
      */
-    Q_DECL_CONSTEXPR inline friend Range operator+(const Range &r1, const Range &r2) Q_DECL_NOEXCEPT
+    constexpr friend Range operator+(Range r1, Range r2) noexcept
     {
         return Range(r1.start() + r2.start(), r1.end() + r2.end());
     }
@@ -457,7 +465,7 @@ public:
      *
      * \return a reference to the cursor which has just been added to
      */
-    inline friend Range &operator+=(Range &r1, const Range &r2) Q_DECL_NOEXCEPT
+    friend Range &operator+=(Range &r1, Range r2) noexcept
     {
         r1.setRange(r1.start() + r2.start(), r1.end() + r2.end());
         return r1;
@@ -472,7 +480,7 @@ public:
      *
      * \return a range representing the subtraction of \p r2 from \p r1
      */
-    Q_DECL_CONSTEXPR inline friend Range operator-(const Range &r1, const Range &r2) Q_DECL_NOEXCEPT
+    constexpr friend Range operator-(Range r1, Range r2) noexcept
     {
         return Range(r1.start() - r2.start(), r1.end() - r2.end());
     }
@@ -485,7 +493,7 @@ public:
      *
      * \return a reference to the range which has just been subtracted from
      */
-    inline friend Range &operator-=(Range &r1, const Range &r2) Q_DECL_NOEXCEPT
+    friend Range &operator-=(Range &r1, Range r2) noexcept
     {
         r1.setRange(r1.start() - r2.start(), r1.end() - r2.end());
         return r1;
@@ -499,7 +507,7 @@ public:
      *
      * \return the intersected range, invalid() if there is no overlap
      */
-    Q_DECL_CONSTEXPR inline friend Range operator&(const Range &r1, const Range &r2) Q_DECL_NOEXCEPT
+    constexpr friend Range operator&(Range r1, Range r2) noexcept
     {
         return r1.intersect(r2);
     }
@@ -512,7 +520,7 @@ public:
      *
      * \return a reference to this range, after the intersection has taken place
      */
-    inline friend Range &operator&=(Range &r1, const Range &r2) Q_DECL_NOEXCEPT
+    friend Range &operator&=(Range &r1, Range r2) noexcept
     {
         r1.setRange(r1.intersect(r2));
         return r1;
@@ -526,7 +534,7 @@ public:
      *
      * \return \e true if \e r1 and \e r2 equal, otherwise \e false
      */
-    Q_DECL_CONSTEXPR inline friend bool operator==(const Range &r1, const Range &r2) Q_DECL_NOEXCEPT
+    constexpr friend bool operator==(Range r1, Range r2) noexcept
     {
         return r1.start() == r2.start() && r1.end() == r2.end();
     }
@@ -539,7 +547,7 @@ public:
      *
      * \return \e true if \e r1 and \e r2 do \e not equal, otherwise \e false
      */
-    Q_DECL_CONSTEXPR inline friend bool operator!=(const Range &r1, const Range &r2) Q_DECL_NOEXCEPT
+    constexpr friend bool operator!=(Range r1, Range r2) noexcept
     {
         return r1.start() != r2.start() || r1.end() != r2.end();
     }
@@ -553,7 +561,7 @@ public:
      *
      * \return \e true if \e r1 starts after where \e r2 ends, otherwise \e false
      */
-    Q_DECL_CONSTEXPR inline friend bool operator>(const Range &r1, const Range &r2) Q_DECL_NOEXCEPT
+    constexpr friend bool operator>(Range r1, Range r2) noexcept
     {
         return r1.start() > r2.end();
     }
@@ -567,20 +575,11 @@ public:
      *
      * \return \e true if \e r1 ends before \e r2 begins, otherwise \e false
      */
-    Q_DECL_CONSTEXPR inline friend bool operator<(const Range &r1, const Range &r2) Q_DECL_NOEXCEPT
+    constexpr friend bool operator<(Range r1, Range r2) noexcept
     {
         return r1.end() < r2.start();
     }
 
-    /**
-     * qDebug() stream operator.  Writes this range to the debug output in a nicely formatted way.
-     */
-    inline friend QDebug operator<<(QDebug s, const Range &range)
-    {
-        s << "[" << range.start() << " -> " << range.end() << "]";
-        return s;
-    }
-
 private:
     /**
      * This range's start cursor pointer.
@@ -599,27 +598,30 @@ private:
 
 }
 
-Q_DECLARE_TYPEINFO(KTextEditor::Range, Q_MOVABLE_TYPE);
-Q_DECLARE_METATYPE(KTextEditor::Range)
+Q_DECLARE_TYPEINFO(KTextEditor::Range, Q_RELOCATABLE_TYPE);
+
+/**
+ * qDebug() stream operator.  Writes this range to the debug output in a nicely formatted way.
+ */
+KTEXTEDITOR_EXPORT QDebug operator<<(QDebug s, KTextEditor::Range range);
 
 /**
  * QHash function for KTextEditor::Range.
  * Returns the hash value for @p range.
  */
-inline uint qHash(const KTextEditor::Range &range, uint seed = 0) Q_DECL_NOTHROW
-{
-    return qHash(qMakePair(qHash(range.start()), qHash(range.end())), seed);
-}
+KTEXTEDITOR_EXPORT size_t qHash(const KTextEditor::Range &range, size_t seed = 0) noexcept;
 
 namespace QTest
 {
 // forward declaration of template in qtestcase.h
-template<typename T> char *toString(const T &);
+template<typename T>
+char *toString(const T &);
 
 /**
  * QTestLib integration to have nice output in e.g. QCOMPARE failures.
  */
-template<> KTEXTEDITOR_EXPORT char *toString(const KTextEditor::Range &range);
+template<>
+KTEXTEDITOR_EXPORT char *toString(const KTextEditor::Range &range);
 }
 
 #endif
