@@ -1,44 +1,25 @@
-/*  SPDX-License-Identifier: LGPL-2.0-or-later
+/*
+    SPDX-FileCopyrightText: KDE Developers
 
-    Copyright (C) KDE Developers
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "katenormalinputmode.h"
 #include "katecompletionwidget.h"
 #include "kateconfig.h"
+#include "katedocument.h"
+#include "katerenderer.h"
 #include "katesearchbar.h"
+#include "kateview.h"
 #include "kateviewinternal.h"
 
 #include <KLocalizedString>
 
 KateNormalInputMode::KateNormalInputMode(KateViewInternal *viewInternal)
     : KateAbstractInputMode(viewInternal)
-    , m_searchBar(nullptr)
-    , m_cmdLine(nullptr)
 {
 }
 
-KateNormalInputMode::~KateNormalInputMode()
-{
-    delete m_cmdLine;
-    delete m_searchBar;
-}
-
 void KateNormalInputMode::activate()
 {
     view()->activateEditActions();
@@ -178,21 +159,15 @@ void KateNormalInputMode::activateCommandLine()
 
 KateSearchBar *KateNormalInputMode::searchBar(const SearchBarMode mode)
 {
-    /**
-     * power mode wanted?
-     */
+    // power mode wanted?
     const bool wantPowerMode = (mode == PowerSearchBar);
 
-    /**
-     * create search bar is not there? use right mode
-     */
+    // create search bar is not there? use right mode
     if (!m_searchBar) {
-        m_searchBar = new KateSearchBar(wantPowerMode, view(), KateViewConfig::global());
+        m_searchBar.reset(new KateSearchBar(wantPowerMode, view(), KateViewConfig::global()));
     }
 
-    /**
-     * else: switch mode if needed!
-     */
+    // else: switch mode if needed!
     else if (mode != IncrementalSearchBarOrKeepMode) {
         if (wantPowerMode) {
             m_searchBar->enterPowerMode();
@@ -201,17 +176,17 @@ KateSearchBar *KateNormalInputMode::searchBar(const SearchBarMode mode)
         }
     }
 
-    return m_searchBar;
+    return m_searchBar.get();
 }
 
 KateCommandLineBar *KateNormalInputMode::cmdLineBar()
 {
     if (!m_cmdLine) {
-        m_cmdLine = new KateCommandLineBar(view(), view()->bottomViewBar());
-        view()->bottomViewBar()->addBarWidget(m_cmdLine);
+        m_cmdLine.reset(new KateCommandLineBar(view(), view()->bottomViewBar()));
+        view()->bottomViewBar()->addBarWidget(m_cmdLine.get());
     }
 
-    return m_cmdLine;
+    return m_cmdLine.get();
 }
 
 void KateNormalInputMode::updateRendererConfig()
@@ -227,10 +202,21 @@ bool KateNormalInputMode::keyPress(QKeyEvent *e)
     const int key = e->key() | (e->modifiers() & Qt::ShiftModifier);
 
     if (view()->isCompletionActive()) {
-        if (key == Qt::Key_Enter || key == Qt::Key_Return) {
-            view()->completionWidget()->execute();
-            e->accept();
-            return true;
+        if (key == Qt::Key_Tab || key == (Qt::SHIFT | Qt::Key_Backtab).toCombined() || key == Qt::Key_Backtab) {
+            if (KateViewConfig::global()->tabCompletion()) {
+                e->accept();
+                using W = KateCompletionWidget;
+                const auto direction = key == Qt::Key_Tab ? W::Down : W::Up;
+                view()->completionWidget()->tabCompletion(direction);
+                return true;
+            }
+        }
+
+        if (key == Qt::Key_Enter || key == Qt::Key_Return || key == Qt::Key_Tab) {
+            if (view()->completionWidget()->execute()) {
+                e->accept();
+                return true;
+            }
         }
     }
 
@@ -242,9 +228,9 @@ bool KateNormalInputMode::blinkCaret() const
     return true;
 }
 
-KateRenderer::caretStyles KateNormalInputMode::caretStyle() const
+KTextEditor::caretStyles KateNormalInputMode::caretStyle() const
 {
-    return view()->isOverwriteMode() ? KateRenderer::Block : KateRenderer::Line;
+    return view()->isOverwriteMode() ? KTextEditor::caretStyles::Block : KTextEditor::caretStyles::Line;
 }
 
 void KateNormalInputMode::toggleInsert()
