@@ -13,6 +13,7 @@
 #include "katedocument.h"
 #include "katehighlight.h"
 #include "katepartdebug.h"
+#include "katerenderer.h"
 #include "katetextfolding.h"
 #include "katetextlayout.h"
 #include "kateview.h"
@@ -79,9 +80,7 @@ PrintPainter::PrintPainter(KTextEditor::DocumentPrivate *doc, KTextEditor::ViewP
     , m_headerFormat()
     , m_footerFormat()
 {
-    m_folding = new Kate::TextFolding(m_doc->buffer());
-
-    m_renderer = new KateRenderer(m_doc, *m_folding, m_view);
+    m_renderer = new KateRenderer(m_doc, m_view->renderer()->folding(), m_view);
     m_renderer->setPrinterFriendly(true);
 
     updateCache();
@@ -90,7 +89,11 @@ PrintPainter::PrintPainter(KTextEditor::DocumentPrivate *doc, KTextEditor::ViewP
 PrintPainter::~PrintPainter()
 {
     delete m_renderer;
-    delete m_folding;
+}
+
+void PrintPainter::setTextFont(const QFont &font)
+{
+    m_renderer->config()->setFont(font);
 }
 
 void PrintPainter::setUseBox(const bool on)
@@ -178,6 +181,8 @@ void PrintPainter::paint(QPrinter *printer) const
     bool pageStarted = true;
     uint remainder = 0;
 
+    auto &f = m_view->renderer()->folding();
+
     // On to draw something :-)
     while (lineCount <= pl.lastline) {
         if (y + m_fontHeight > pl.maxHeight) {
@@ -198,11 +203,15 @@ void PrintPainter::paint(QPrinter *printer) const
             painter.translate(pl.xstart, y);
         }
 
-        if (m_printLineNumbers /*&& ! startCol*/) { // don't repeat!
+        const bool skipLine = m_dontPrintFoldedCode && !f.isLineVisible(lineCount);
+
+        if (!skipLine && m_printLineNumbers /*&& ! startCol*/) { // don't repeat!
             paintLineNumber(painter, lineCount, pl);
         }
 
-        paintLine(painter, lineCount, y, remainder, pl);
+        if (!skipLine) {
+            paintLine(painter, lineCount, y, remainder, pl);
+        }
 
         if (!remainder) {
             lineCount++;
@@ -244,7 +253,7 @@ void PrintPainter::configure(const QPrinter *printer, PageLayout &pl) const
         // none of these operations should be expensive,
         // and searching each tag in the format strings is avoided.
         QDateTime dt = QDateTime::currentDateTime();
-        QMap<QString, QString> tags;
+        std::map<QString, QString> tags;
 
         KUser u(KUser::UseRealUserID);
         tags[QStringLiteral("u")] = u.loginName();
@@ -262,7 +271,7 @@ void PrintPainter::configure(const QPrinter *printer, PageLayout &pl) const
             tags[QStringLiteral("U")].prepend(s);
         }
 
-        QRegularExpression reTags(QStringLiteral("%([dDfUhuyY])")); // TODO check for "%%<TAG>"
+        static const QRegularExpression reTags(QStringLiteral("%([dDfUhuyY])")); // TODO check for "%%<TAG>"
 
         if (m_useHeader) {
             pl.headerHeight = QFontMetrics(m_fhFont).height();
@@ -356,10 +365,10 @@ void PrintPainter::configure(const QPrinter *printer, PageLayout &pl) const
         int totalLines = 0;
         // TODO: right now ignores selection printing
         for (unsigned int i = pl.firstline; i <= pl.lastline; ++i) {
-            KateLineLayoutPtr rangeptr(new KateLineLayout(*m_renderer));
-            rangeptr->setLine(i);
-            m_renderer->layoutLine(rangeptr, (int)pl.maxWidth, false);
-            totalLines += rangeptr->viewLineCount();
+            KateLineLayout rangeptr(*m_renderer);
+            rangeptr.setLine(i);
+            m_renderer->layoutLine(&rangeptr, (int)pl.maxWidth, false);
+            totalLines += rangeptr.viewLineCount();
         }
 
         const int totalPages = (totalLines / linesPerPage) + ((totalLines % linesPerPage) > 0 ? 1 : 0);
@@ -508,7 +517,10 @@ void PrintPainter::paintGuide(QPainter &painter, uint &y, const PageLayout &pl)
     _titleFont.setBold(true);
     painter.setFont(_titleFont);
     QRect _r;
-    painter.drawText(QRect(_marg, y, pl.pageWidth - (2 * _marg), pl.maxHeight - y), Qt::AlignTop | Qt::AlignHCenter, i18n("Typographical Conventions for %1", _hlName), &_r);
+    painter.drawText(QRect(_marg, y, pl.pageWidth - (2 * _marg), pl.maxHeight - y),
+                     Qt::AlignTop | Qt::AlignHCenter,
+                     i18n("Typographical Conventions for %1", _hlName),
+                     &_r);
     const int _w = pl.pageWidth - (_marg * 2) - (pl.innerMargin * 2);
     const int _x = _marg + pl.innerMargin;
     y += _r.height() + pl.innerMargin;
@@ -516,7 +528,7 @@ void PrintPainter::paintGuide(QPainter &painter, uint &y, const PageLayout &pl)
     y += 1 + pl.innerMargin;
 
     int _widest(0);
-    for (const KTextEditor::Attribute::Ptr &attribute : qAsConst(_attributes)) {
+    for (const KTextEditor::Attribute::Ptr &attribute : std::as_const(_attributes)) {
         const QString _name = attribute->name().section(QLatin1Char(':'), 1, 1);
         _widest = qMax(QFontMetrics(attribute->font()).boundingRect(_name).width(), _widest);
     }
@@ -529,7 +541,7 @@ void PrintPainter::paintGuide(QPainter &painter, uint &y, const PageLayout &pl)
 
     _titleFont.setUnderline(true);
     QString _currentHlName;
-    for (const KTextEditor::Attribute::Ptr &attribute : qAsConst(_attributes)) {
+    for (const KTextEditor::Attribute::Ptr &attribute : std::as_const(_attributes)) {
         QString _hl = attribute->name().section(QLatin1Char(':'), 0, 0);
         QString _name = attribute->name().section(QLatin1Char(':'), 1, 1);
         if (_hl != _hlName && _hl != _currentHlName) {
@@ -603,7 +615,8 @@ void PrintPainter::paintBackground(QPainter &painter, const uint y, const PageLa
 {
     // If we have a box, or the header/footer has backgrounds, we want to paint
     // to the border of those. Otherwise just the contents area.
-    int _y = y, _h = pl.maxHeight - y;
+    int _y = y;
+    int _h = pl.maxHeight - y;
     if (m_useBox) {
         _y -= pl.innerMargin;
         _h += 2 * pl.innerMargin;
@@ -622,29 +635,29 @@ void PrintPainter::paintBackground(QPainter &painter, const uint y, const PageLa
 void PrintPainter::paintLine(QPainter &painter, const uint line, uint &y, uint &remainder, const PageLayout &pl) const
 {
     // HA! this is where we print [part of] a line ;]]
-    KateLineLayoutPtr rangeptr(new KateLineLayout(*m_renderer));
-    rangeptr->setLine(line);
-    m_renderer->layoutLine(rangeptr, (int)pl.maxWidth, false);
+    KateLineLayout rangeptr(*m_renderer);
+    rangeptr.setLine(line);
+    m_renderer->layoutLine(&rangeptr, (int)pl.maxWidth, false);
 
     // selectionOnly: clip non-selection parts and adjust painter position if needed
     int _xadjust = 0;
     if (pl.selectionOnly) {
         if (m_view && m_view->blockSelection()) {
-            int _x = m_renderer->cursorToX(rangeptr->viewLine(0), pl.selectionRange.start());
-            int _x1 = m_renderer->cursorToX(rangeptr->viewLine(rangeptr->viewLineCount() - 1), pl.selectionRange.end());
+            int _x = m_renderer->cursorToX(rangeptr.viewLine(0), pl.selectionRange.start());
+            int _x1 = m_renderer->cursorToX(rangeptr.viewLine(rangeptr.viewLineCount() - 1), pl.selectionRange.end());
             _xadjust = _x;
             painter.translate(-_xadjust, 0);
-            painter.setClipRegion(QRegion(_x, 0, _x1 - _x, rangeptr->viewLineCount() * m_fontHeight));
+            painter.setClipRegion(QRegion(_x, 0, _x1 - _x, rangeptr.viewLineCount() * m_fontHeight));
 
         } else if (line == pl.firstline || line == pl.lastline) {
-            QRegion region(0, 0, pl.maxWidth, rangeptr->viewLineCount() * m_fontHeight);
+            QRegion region(0, 0, pl.maxWidth, rangeptr.viewLineCount() * m_fontHeight);
 
             if (line == pl.firstline) {
-                region = region.subtracted(QRegion(0, 0, m_renderer->cursorToX(rangeptr->viewLine(0), pl.selectionRange.start()), m_fontHeight));
+                region = region.subtracted(QRegion(0, 0, m_renderer->cursorToX(rangeptr.viewLine(0), pl.selectionRange.start()), m_fontHeight));
             }
 
             if (line == pl.lastline) {
-                int _x = m_renderer->cursorToX(rangeptr->viewLine(rangeptr->viewLineCount() - 1), pl.selectionRange.end());
+                int _x = m_renderer->cursorToX(rangeptr.viewLine(rangeptr.viewLineCount() - 1), pl.selectionRange.end());
                 region = region.subtracted(QRegion(_x, 0, pl.maxWidth - _x, m_fontHeight));
             }
 
@@ -654,24 +667,32 @@ void PrintPainter::paintLine(QPainter &painter, const uint line, uint &y, uint &
 
     // If the line is too long (too many 'viewlines') to fit the remaining vertical space,
     // clip and adjust the painter position as necessary
-    int _lines = rangeptr->viewLineCount(); // number of "sublines" to paint.
+    int _lines = rangeptr.viewLineCount(); // number of "sublines" to paint.
 
     int proceedLines = _lines;
     if (remainder) {
         proceedLines = qMin((pl.maxHeight - y) / m_fontHeight, remainder);
 
         painter.translate(0, -(_lines - int(remainder)) * m_fontHeight + 1);
-        painter.setClipRect(0, (_lines - int(remainder)) * m_fontHeight + 1, pl.maxWidth, proceedLines * m_fontHeight); //### drop the crosspatch in printerfriendly mode???
+        painter.setClipRect(0,
+                            (_lines - int(remainder)) * m_fontHeight + 1,
+                            pl.maxWidth,
+                            proceedLines * m_fontHeight); // ### drop the crosspatch in printerfriendly mode???
         remainder -= proceedLines;
     } else if (y + m_fontHeight * _lines > pl.maxHeight) {
         remainder = _lines - ((pl.maxHeight - y) / m_fontHeight);
-        painter.setClipRect(0, 0, pl.maxWidth, (_lines - int(remainder)) * m_fontHeight + 1); //### drop the crosspatch in printerfriendly mode???
+        painter.setClipRect(0, 0, pl.maxWidth, (_lines - int(remainder)) * m_fontHeight + 1); // ### drop the crosspatch in printerfriendly mode???
     } else if (!pl.selectionOnly) {
         painter.setClipRegion(QRegion());
         painter.setClipping(false);
     }
 
-    m_renderer->paintTextLine(painter, rangeptr, 0, (int)pl.maxWidth);
+    KateRenderer::PaintTextLineFlags flags;
+    if (!m_dontPrintFoldedCode) {
+        flags.setFlag(KateRenderer::PaintTextLineFlag::SkipDrawFirstInvisibleLineUnderlined);
+    }
+
+    m_renderer->paintTextLine(painter, &rangeptr, 0, (int)pl.maxWidth, QRectF{}, nullptr, flags);
 
     painter.setClipping(false);
     painter.translate(_xadjust, (m_fontHeight * (_lines - remainder)));
@@ -686,7 +707,7 @@ void PrintPainter::paintLineNumber(QPainter &painter, const uint number, const P
     painter.save();
     painter.setFont(m_renderer->currentFont());
     painter.setPen(m_renderer->config()->lineNumberColor());
-    painter.drawText(left, 0, m_lineNumberWidth, m_fontHeight, Qt::AlignRight, QString::number(number + 1));
+    painter.drawText(left, 0, m_lineNumberWidth, m_fontHeight, Qt::AlignRight | Qt::AlignVCenter, QString::number(number + 1));
     painter.restore();
 }
 
