@@ -15,11 +15,13 @@
 #include "inlinenotedata.h"
 #include "katebuffer.h"
 #include "katedocument.h"
+#include "kateextendedattribute.h"
 #include "katehighlight.h"
 #include "katerenderrange.h"
 #include "katetextlayout.h"
 #include "kateview.h"
 
+#include "ktexteditor/attribute.h"
 #include "ktexteditor/inlinenote.h"
 #include "ktexteditor/inlinenoteprovider.h"
 
@@ -33,8 +35,6 @@
 #include <QStack>
 #include <QtMath> // qCeil
 
-#include <optional>
-
 static const QChar tabChar(QLatin1Char('\t'));
 static const QChar spaceChar(QLatin1Char(' '));
 static const QChar nbSpaceChar(0xa0); // non-breaking space
@@ -45,7 +45,7 @@ KateRenderer::KateRenderer(KTextEditor::DocumentPrivate *doc, Kate::TextFolding
     , m_view(view)
     , m_tabWidth(m_doc->config()->tabWidth())
     , m_indentWidth(m_doc->config()->indentationWidth())
-    , m_caretStyle(KateRenderer::Line)
+    , m_caretStyle(KTextEditor::caretStyles::Line)
     , m_drawCaret(true)
     , m_showSelections(true)
     , m_showTabs(true)
@@ -70,7 +70,7 @@ void KateRenderer::updateAttributes()
     m_attributes = m_doc->highlight()->attributes(config()->schema());
 }
 
-KTextEditor::Attribute::Ptr KateRenderer::attribute(uint pos) const
+const KTextEditor::Attribute::Ptr &KateRenderer::attribute(uint pos) const
 {
     if (pos < (uint)m_attributes.count()) {
         return m_attributes[pos];
@@ -93,7 +93,7 @@ void KateRenderer::setDrawCaret(bool drawCaret)
     m_drawCaret = drawCaret;
 }
 
-void KateRenderer::setCaretStyle(KateRenderer::caretStyles style)
+void KateRenderer::setCaretStyle(KTextEditor::caretStyles style)
 {
     m_caretStyle = style;
 }
@@ -127,8 +127,7 @@ void KateRenderer::setShowIndentLines(bool showIndentLines)
 {
     // invalidate our "active indent line" cached stuff
     m_currentBracketRange = KTextEditor::Range::invalid();
-    m_currentOpenBracketX = -1;
-    m_currentCloseBracketX = -1;
+    m_currentBracketX = -1;
 
     m_config->setShowIndentationLines(showIndentLines);
 }
@@ -179,7 +178,7 @@ void KateRenderer::setPrinterFriendly(bool printerFriendly)
     setDrawCaret(false);
 }
 
-void KateRenderer::paintTextLineBackground(QPainter &paint, KateLineLayoutPtr layout, int currentViewLine, int xStart, int xEnd)
+void KateRenderer::paintTextLineBackground(QPainter &paint, KateLineLayout *layout, int currentViewLine, int xStart, int xEnd)
 {
     if (isPrinterFriendly()) {
         return;
@@ -201,7 +200,7 @@ void KateRenderer::paintTextLineBackground(QPainter &paint, KateLineLayoutPtr la
     uint mrk = m_doc->mark(layout->line());
     if (mrk) {
         for (uint bit = 0; bit < 32; bit++) {
-            KTextEditor::MarkInterface::MarkTypes markType = (KTextEditor::MarkInterface::MarkTypes)(1U << bit);
+            KTextEditor::Document::MarkTypes markType = (KTextEditor::Document::MarkTypes)(1U << bit);
             if (mrk & markType) {
                 QColor markColor = config()->lineMarkerColor(markType);
 
@@ -335,69 +334,49 @@ void KateRenderer::paintNonPrintableSpaces(QPainter &paint, qreal x, qreal y, co
  * then use this data to color the indentation line differently.
  * @p view is current view
  * @p range is the current range from @ref paintTextLine
+ * @p spaceWidth width of space char
  * @p c is the position of cursor
- * @p openX will be X position of open bracket or -1 if not found
- * @p closeX will be X position of close bracket or -1 if not found
+ * @p bracketXPos will be X position of close bracket or -1 if not found
  */
-static KTextEditor::Range cursorAtBracket(KTextEditor::ViewPrivate *view, const KateLineLayoutPtr &range, KTextEditor::Cursor c, int &openX, int &closeX)
+static KTextEditor::Range cursorAtBracket(KTextEditor::ViewPrivate *view, const KateLineLayout *range, int spaceWidth, KTextEditor::Cursor c, int &bracketXPos)
 {
+    bracketXPos = -1;
     if (range->line() != c.line()) {
-        openX = closeX = -1;
         return KTextEditor::Range::invalid();
     }
 
     auto *doc = view->doc();
     // Avoid work if we are below tabwidth
     if (c.column() < doc->config()->tabWidth()) {
-        openX = closeX = -1;
         return KTextEditor::Range::invalid();
     }
 
     // We match these brackets only
-    static constexpr QChar brackets[] = {QLatin1Char('{'), QLatin1Char('}')};
+    static constexpr QChar brackets[] = {QLatin1Char('{'), QLatin1Char('}'), QLatin1Char('('), QLatin1Char(')')};
     // look for character in front
     QChar right = doc->characterAt(c);
     auto it = std::find(std::begin(brackets), std::end(brackets), right);
 
     KTextEditor::Range ret = KTextEditor::Range::invalid();
-    bool inFront = false;
     bool found = false;
     if (it != std::end(brackets)) {
         found = true;
-        inFront = true;
     } else {
         // look at previous character
         QChar left = doc->characterAt({c.line(), c.column() - 1});
         it = std::find(std::begin(brackets), std::end(brackets), left);
         if (it != std::end(brackets)) {
             found = true;
-            inFront = false;
         }
     }
 
     // We have a bracket
     if (found) {
-        ret = doc->findMatchingBracket(c, 150);
+        ret = doc->findMatchingBracket(c, 500);
         if (!ret.isValid()) {
-            openX = closeX = -1;
             return ret;
         }
-        // line for current pos
-        QTextLine line = range->layout()->lineForTextPosition(qMin(c.column(), range->length()));
-
-        if (ret.start().line() == c.line()) {
-            // Our cursor is at opening bracket
-            openX = line.cursorToX(c.column() + (inFront ? 0 : -1)) + 1;
-            QTextLine closeLine = view->textLayout(ret.end().line())->lineForTextPosition(ret.end().column());
-            closeX = closeLine.cursorToX(ret.end().column()) + 1;
-        } else {
-            // Our cursor is at closing bracket
-            closeX = line.cursorToX(c.column() + (inFront ? 0 : -1)) + 1;
-            QTextLine closeLine = view->textLayout(ret.start().line())->lineForTextPosition(ret.start().column());
-            openX = closeLine.cursorToX(ret.start().column()) + 1;
-        }
-    } else {
-        openX = closeX = -1;
+        bracketXPos = (ret.end().column() * spaceWidth) + 1;
     }
 
     return ret;
@@ -406,10 +385,10 @@ static KTextEditor::Range cursorAtBracket(KTextEditor::ViewPrivate *view, const
 void KateRenderer::paintIndentMarker(QPainter &paint, uint x, int line)
 {
     const QPen penBackup(paint.pen());
-    static const QVector<qreal> dashPattern = QVector<qreal>() << 1 << 1;
+    static const QList<qreal> dashPattern = QList<qreal>() << 1 << 1;
     QPen myPen;
 
-    const bool onBracket = m_currentOpenBracketX == (int)x || m_currentCloseBracketX == (int)x;
+    const bool onBracket = m_currentBracketX == (int)x;
     if (onBracket && m_currentBracketRange.containsLine(line)) {
         QColor c = view()->theme().textColor(KSyntaxHighlighting::Theme::Normal);
         c.setAlphaF(0.7);
@@ -454,7 +433,7 @@ static bool rangeLessThanForRenderer(const Kate::TextRange *a, const Kate::TextR
     return false;
 }
 
-QVector<QTextLayout::FormatRange> KateRenderer::decorationsForLine(const Kate::TextLine &textLine, int line, bool selectionsOnly) const
+QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine(const Kate::TextLine &textLine, int line, bool selectionsOnly) const
 {
     // limit number of attributes we can highlight in reasonable time
     const int limitOfRanges = 1024;
@@ -465,15 +444,15 @@ QVector<QTextLayout::FormatRange> KateRenderer::decorationsForLine(const Kate::T
 
     // Don't compute the highlighting if there isn't going to be any highlighting
     const auto &al = textLine->attributesList();
-    if (!(selectionsOnly || !al.isEmpty() || !rangesWithAttributes.isEmpty())) {
-        return QVector<QTextLayout::FormatRange>();
+    if (!(selectionsOnly || !al.empty() || !rangesWithAttributes.empty())) {
+        return QList<QTextLayout::FormatRange>();
     }
 
     // Add the inbuilt highlighting to the list, limit with limitOfRanges
     RenderRangeVector renderRanges;
-    if (!al.isEmpty()) {
+    if (!al.empty()) {
         auto &currentRange = renderRanges.pushNewRange();
-        for (int i = 0; i < std::min<int>(al.count(), limitOfRanges); ++i) {
+        for (int i = 0; i < std::min<int>(al.size(), limitOfRanges); ++i) {
             if (al[i].length > 0 && al[i].attributeValue > 0) {
                 currentRange.addRange(KTextEditor::Range(KTextEditor::Cursor(line, al[i].offset), al[i].length), specificAttribute(al[i].attributeValue));
             }
@@ -530,7 +509,7 @@ QVector<QTextLayout::FormatRange> KateRenderer::decorationsForLine(const Kate::T
         if (!hasCustomLineHeight()) {
             backgroundAttribute->setBackground(config()->selectionColor());
         }
-        backgroundAttribute->setForeground(attribute(KTextEditor::dsNormal)->selectedForeground().color());
+        backgroundAttribute->setForeground(attribute(KSyntaxHighlighting::Theme::TextStyle::Normal)->selectedForeground().color());
 
         // Create a range for the current selection
         if (m_view->blockSelection() && m_view->selectionRange().overlapsLine(line)) {
@@ -542,7 +521,7 @@ QVector<QTextLayout::FormatRange> KateRenderer::decorationsForLine(const Kate::T
 
     // no render ranges, nothing to do, else we loop below endless!
     if (renderRanges.isEmpty()) {
-        return QVector<QTextLayout::FormatRange>();
+        return QList<QTextLayout::FormatRange>();
     }
 
     // Calculate the range which we need to iterate in order to get the highlighting for just this line
@@ -570,7 +549,7 @@ QVector<QTextLayout::FormatRange> KateRenderer::decorationsForLine(const Kate::T
 
     // Main iterative loop.  This walks through each set of highlighting ranges, and stops each
     // time the highlighting changes.  It then creates the corresponding QTextLayout::FormatRanges.
-    QVector<QTextLayout::FormatRange> newHighlight;
+    QList<QTextLayout::FormatRange> newHighlight;
     while (currentPosition < endPosition) {
         renderRanges.advanceTo(currentPosition);
 
@@ -634,7 +613,7 @@ void KateRenderer::assignSelectionBrushesFromAttribute(QTextLayout::FormatRange
     }
 }
 
-void KateRenderer::paintTextBackground(QPainter &paint, KateLineLayoutPtr layout, const QVector<QTextLayout::FormatRange> &selRanges, const QBrush &brush) const
+void KateRenderer::paintTextBackground(QPainter &paint, KateLineLayout *layout, const QList<QTextLayout::FormatRange> &selRanges, const QBrush &brush) const
 {
     const bool rtl = layout->isRightToLeft();
 
@@ -696,7 +675,13 @@ void KateRenderer::paintTextBackground(QPainter &paint, KateLineLayoutPtr layout
     }
 }
 
-void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int xStart, int xEnd, const KTextEditor::Cursor *cursor, PaintTextLineFlags flags)
+void KateRenderer::paintTextLine(QPainter &paint,
+                                 KateLineLayout *range,
+                                 int xStart,
+                                 int xEnd,
+                                 const QRectF &textClipRect,
+                                 const KTextEditor::Cursor *cursor,
+                                 PaintTextLineFlags flags)
 {
     Q_ASSERT(range->isValid());
 
@@ -749,12 +734,12 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
             }
         }
 
-        QVector<QTextLayout::FormatRange> additionalFormats;
+        QList<QTextLayout::FormatRange> additionalFormats;
         if (range->length() > 0) {
             // We may have changed the pen, be absolutely sure it gets set back to
             // normal foreground color before drawing text for text that does not
             // set the pen color
-            paint.setPen(attribute(KTextEditor::dsNormal)->foreground().color());
+            paint.setPen(attribute(KSyntaxHighlighting::Theme::TextStyle::Normal)->foreground().color());
             // Draw the text :)
 
             if (range->layout()->textOption().textDirection() == Qt::RightToLeft) {
@@ -779,10 +764,10 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
                 if (hasCustomLineHeight()) {
                     paintTextBackground(paint, range, additionalFormats, config()->selectionColor());
                 }
-                range->layout()->draw(&paint, QPoint(-xStart, 0), additionalFormats);
+                range->layout()->draw(&paint, QPoint(-xStart, 0), additionalFormats, textClipRect);
 
             } else {
-                range->layout()->draw(&paint, QPoint(-xStart, 0));
+                range->layout()->draw(&paint, QPoint(-xStart, 0), QList<QTextLayout::FormatRange>{}, textClipRect);
             }
         }
 
@@ -793,7 +778,7 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
             auto cur = *cursor;
             cur.setColumn(cur.column() - 1);
             if (!m_currentBracketRange.boundaryAtCursor(*cursor) && m_currentBracketRange.end() != cur && m_currentBracketRange.start() != cur) {
-                m_currentBracketRange = cursorAtBracket(view(), range, *cursor, m_currentOpenBracketX, m_currentCloseBracketX);
+                m_currentBracketRange = cursorAtBracket(view(), range, spaceWidth(), *cursor, m_currentBracketX);
             }
         }
 
@@ -806,7 +791,10 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
                 const qreal w = spaceWidth();
                 const int lastIndentColumn = range->textLine()->indentDepth(m_tabWidth);
                 for (int x = m_indentWidth; x < lastIndentColumn; x += m_indentWidth) {
-                    paintIndentMarker(paint, x * w + 1 - xStart, range->line());
+                    auto xPos = x * w + 1 - xStart;
+                    if (xPos >= 0) {
+                        paintIndentMarker(paint, xPos, range->line());
+                    }
                 }
             }
 
@@ -845,8 +833,13 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
                 if (spaceIndex >= trailingPos) {
                     QVarLengthArray<int, 32> spacePositions;
                     // Adjust to visible contents
-                    spaceIndex = std::min(line.lineLayout().xToCursor(xEnd), spaceIndex);
-                    int visibleStart = line.lineLayout().xToCursor(xStart);
+                    const auto dir = range->layout()->textOption().textDirection();
+                    const bool isRTL = dir == Qt::RightToLeft && m_view->dynWordWrap();
+                    int start = isRTL ? xEnd : xStart;
+                    int end = isRTL ? xStart : xEnd;
+
+                    spaceIndex = std::min(line.lineLayout().xToCursor(end), spaceIndex);
+                    int visibleStart = line.lineLayout().xToCursor(start);
 
                     for (; spaceIndex >= line.startCol(); --spaceIndex) {
                         if (!text.at(spaceIndex).isSpace()) {
@@ -871,12 +864,29 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
                     // reverse because we want to look at the spaces at the beginning of line first
                     for (auto rit = spacePositions.rbegin(); rit != spacePositions.rend(); ++rit) {
                         const int spaceIdx = *rit;
-                        qreal x;
+                        qreal x = line.lineLayout().cursorToX(spaceIdx) - xStart;
+                        int dir = 1; // 1 == ltr, -1 == rtl
                         if (range->layout()->textOption().alignment() == Qt::AlignRight) {
-                            x = line.lineLayout().cursorToX(spaceIdx) - xStart - spaceWidth / 2.0;
+                            dir = -1;
+                            if (spaceIdx > 0) {
+                                QChar c = text.at(spaceIdx - 1);
+                                // line is LTR aligned, but is the char ltr or rtl?
+                                if (!isLineRightToLeft(QStringView(&c, 1))) {
+                                    dir = 1;
+                                }
+                            }
                         } else {
-                            x = (line.lineLayout().cursorToX(spaceIdx) - xStart) + (spaceWidth / 2.0);
+                            if (spaceIdx > 0) {
+                                // line is LTR aligned, but is the char ltr or rtl?
+                                QChar c = text.at(spaceIdx - 1);
+                                if (isLineRightToLeft(QStringView(&c, 1))) {
+                                    dir = -1;
+                                }
+                            }
                         }
+
+                        x += dir * (spaceWidth / 2.0);
+
                         const QPointF currentPoint(x, y);
                         if (!prev.isNull() && currentPoint == prev) {
                             break;
@@ -910,15 +920,15 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
             }
 
             // draw word-wrap-honor-indent filling
-            if ((i > 0) && range->shiftX() && (range->shiftX() > xStart)) {
+            if ((i > 0) && range->shiftX && (range->shiftX > xStart)) {
                 // fill background first with selection if we had selection from the previous line
                 if (drawSelection && !m_view->blockSelection() && m_view->selectionRange().start() < line.start()
                     && m_view->selectionRange().end() >= line.start()) {
-                    paint.fillRect(0, lineHeight() * i, range->shiftX() - xStart, lineHeight(), QBrush(config()->selectionColor()));
+                    paint.fillRect(0, lineHeight() * i, range->shiftX - xStart, lineHeight(), QBrush(config()->selectionColor()));
                 }
 
                 // paint the normal filling for the word wrap markers
-                paint.fillRect(0, lineHeight() * i, range->shiftX() - xStart, lineHeight(), QBrush(config()->wordWrapMarkerColor(), Qt::Dense4Pattern));
+                paint.fillRect(0, lineHeight() * i, range->shiftX - xStart, lineHeight(), QBrush(config()->wordWrapMarkerColor(), Qt::Dense4Pattern));
             }
         }
 
@@ -958,11 +968,23 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
         for (const auto &inlineNoteData : inlineNotes) {
             KTextEditor::InlineNote inlineNote(inlineNoteData);
             const int column = inlineNote.position().column();
-            int viewLine = range->viewLineForColumn(column);
+            const int viewLine = range->viewLineForColumn(column);
+            // We only consider a line "rtl" if dynamic wrap is enabled. If it is disabled, our
+            // text is always on the left side of the view
+            const auto dir = range->layout()->textOption().textDirection();
+            const bool isRTL = dir == Qt::RightToLeft && m_view->dynWordWrap();
 
             // Determine the position where to paint the note.
             // We start by getting the x coordinate of cursor placed to the column.
-            qreal x = range->viewLine(viewLine).lineLayout().cursorToX(column) - xStart;
+            // If the text is ltr or rtl + dyn wrap, get the X from column
+            qreal x;
+            if (dir == Qt::LeftToRight || (dir == Qt::RightToLeft && m_view->dynWordWrap())) {
+                x = range->viewLine(viewLine).lineLayout().cursorToX(column) - xStart;
+            } else /* rtl + dynWordWrap == false */ {
+                // if text is rtl and dynamic wrap is false, the x offsets are in the opposite
+                // direction i.e., [0] == biggest offset, [1] = next
+                x = range->viewLine(viewLine).lineLayout().cursorToX(range->length() - column) - xStart;
+            }
             int textLength = range->length();
             if (column == 0 || column < textLength) {
                 // If the note is inside text or at the beginning, then there is a hole in the text where the
@@ -972,7 +994,8 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
             } else {
                 // If the note is outside the text, then the X coordinate is located at the end of the line.
                 // Add appropriate amount of spaces to reach the required column.
-                x += spaceWidth() * (column - textLength);
+                const auto spaceToAdd = spaceWidth() * (column - textLength);
+                x += isRTL ? -spaceToAdd : spaceToAdd;
             }
 
             qreal y = lineHeight() * viewLine;
@@ -980,7 +1003,7 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
             // Paint the note
             paint.save();
             paint.translate(x, y);
-            inlineNote.provider()->paintInlineNote(inlineNote, paint);
+            inlineNote.provider()->paintInlineNote(inlineNote, paint, isRTL ? Qt::RightToLeft : Qt::LeftToRight);
             paint.restore();
         }
     }
@@ -988,9 +1011,7 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
 
 static void drawCursor(const QTextLayout &layout, QPainter *p, const QPointF &pos, int cursorPosition, int width, const int height)
 {
-    if (!layout.isValidCursorPosition(cursorPosition)) {
-        cursorPosition = 0;
-    }
+    cursorPosition = qBound(0, cursorPosition, layout.text().length());
     const QTextLine l = layout.lineForTextPosition(cursorPosition);
     Q_ASSERT(l.isValid());
     if (!l.isValid()) {
@@ -1008,7 +1029,7 @@ static void drawCursor(const QTextLayout &layout, QPainter *p, const QPointF &po
     p->setCompositionMode(origCompositionMode);
 }
 
-void KateRenderer::paintCaret(const KTextEditor::Cursor &cursor, const KateLineLayoutPtr &range, QPainter &paint, int xStart, int xEnd)
+void KateRenderer::paintCaret(KTextEditor::Cursor cursor, KateLineLayout *range, QPainter &paint, int xStart, int xEnd)
 {
     if (range->includesCursor(cursor)) {
         int caretWidth;
@@ -1017,10 +1038,10 @@ void KateRenderer::paintCaret(const KTextEditor::Cursor &cursor, const KateLineL
         QTextLine line = range->layout()->lineForTextPosition(qMin(cursor.column(), range->length()));
 
         // Determine the caret's style
-        caretStyles style = caretStyle();
+        KTextEditor::caretStyles style = caretStyle();
 
         // Make the caret the desired width
-        if (style == Line) {
+        if (style == KTextEditor::caretStyles::Line) {
             caretWidth = lineWidth;
         } else if (line.isValid() && cursor.column() < range->length()) {
             caretWidth = int(line.cursorToX(cursor.column() + 1) - line.cursorToX(cursor.column()));
@@ -1051,23 +1072,23 @@ void KateRenderer::paintCaret(const KTextEditor::Cursor &cursor, const KateLineL
             }
             // still no color found, fall back to default style
             if (!color.isValid()) {
-                color = attribute(KTextEditor::dsNormal)->foreground().color();
+                color = attribute(KSyntaxHighlighting::Theme::TextStyle::Normal)->foreground().color();
             }
         }
 
         paint.save();
         switch (style) {
-        case Line:
+        case KTextEditor::caretStyles::Line:
             paint.setPen(QPen(color, caretWidth));
             break;
-        case Block:
+        case KTextEditor::caretStyles::Block:
             // use a gray caret so it's possible to see the character
             color.setAlpha(128);
             paint.setPen(QPen(color, caretWidth));
             break;
-        case Underline:
+        case KTextEditor::caretStyles::Underline:
             break;
-        case Half:
+        case KTextEditor::caretStyles::Half:
             color.setAlpha(128);
             paint.setPen(QPen(color, caretWidth));
             break;
@@ -1083,7 +1104,7 @@ void KateRenderer::paintCaret(const KTextEditor::Cursor &cursor, const KateLineL
             for (const auto &inlineNoteData : inlineNotes) {
                 KTextEditor::InlineNote inlineNote(inlineNoteData);
                 if (inlineNote.position().column() == cursor.column()) {
-                    width = inlineNote.width() + (caretStyle() == Line ? 2.0 : 0.0);
+                    width = inlineNote.width() + (caretStyle() == KTextEditor::caretStyles::Line ? 2.0 : 0.0);
                 }
             }
             drawCursor(*range->layout(), &paint, QPoint(-xStart - width, 0), cursor.column(), caretWidth, lineHeight());
@@ -1215,7 +1236,7 @@ qreal KateRenderer::spaceWidth() const
     return m_fontMetrics.horizontalAdvance(spaceChar);
 }
 
-void KateRenderer::layoutLine(KateLineLayoutPtr lineLayout, int maxwidth, bool cacheLayout) const
+void KateRenderer::layoutLine(KateLineLayout *lineLayout, int maxwidth, bool cacheLayout) const
 {
     // if maxwidth == -1 we have no wrap
 
@@ -1251,7 +1272,11 @@ void KateRenderer::layoutLine(KateLineLayoutPtr lineLayout, int maxwidth, bool c
     // Qt's text renderer ("scribe") version 4.2 assumes a "higher-level protocol"
     // (such as KatePart) will specify the paragraph level, so it does not apply P2 & P3
     // by itself. If this ever change in Qt, the next code block could be removed.
-    if (isLineRightToLeft(lineLayout)) {
+    // -----
+    // Only force RTL direction if dynWordWrap is on. Otherwise the view has infinite width
+    // and the lines will never be forced RTL no matter what direction we set. The layout
+    // can't force a line to the right if it doesn't know where the "right" is
+    if (isLineRightToLeft(textLine->text()) || (view()->dynWordWrap() && view()->forceRTLDirection())) {
         opt.setAlignment(Qt::AlignRight);
         opt.setTextDirection(Qt::RightToLeft);
         // Must turn off this flag otherwise cursor placement
@@ -1269,7 +1294,7 @@ void KateRenderer::layoutLine(KateLineLayoutPtr lineLayout, int maxwidth, bool c
     l->setTextOption(opt);
 
     // Syntax highlighting, inbuilt and arbitrary
-    QVector<QTextLayout::FormatRange> decorations = decorationsForLine(textLine, lineLayout->line());
+    QList<QTextLayout::FormatRange> decorations = decorationsForLine(textLine, lineLayout->line());
 
     // Qt works badly if you have RTL text and formats set on that text.
     // It will shape the text according to the given format ranges which
@@ -1312,7 +1337,7 @@ void KateRenderer::layoutLine(KateLineLayoutPtr lineLayout, int maxwidth, bool c
                 firstLineOffset = width;
             } else if (column < l->text().length()) {
                 QTextCharFormat text_char_format;
-                const qreal caretWidth = caretStyle() == Line ? 2.0 : 0.0;
+                const qreal caretWidth = caretStyle() == KTextEditor::caretStyles::Line ? 2.0 : 0.0;
                 text_char_format.setFontLetterSpacing(width + caretWidth);
                 text_char_format.setFontLetterSpacingType(QFont::AbsoluteSpacing);
                 decorations.append(QTextLayout::FormatRange{column - 1, 1, text_char_format});
@@ -1363,7 +1388,7 @@ void KateRenderer::layoutLine(KateLineLayoutPtr lineLayout, int maxwidth, bool c
             // if shiftX > 0, the maxwidth has to adapted
             maxwidth -= shiftX;
 
-            lineLayout->setShiftX(shiftX);
+            lineLayout->shiftX = shiftX;
         }
 
         height += lineHeight();
@@ -1379,15 +1404,10 @@ void KateRenderer::layoutLine(KateLineLayoutPtr lineLayout, int maxwidth, bool c
 // 3) QString::isRightToLeft() does not seem to work on my setup
 // 4) isStringRightToLeft() should behave much better than QString::isRightToLeft() therefore:
 // 5) isStringRightToLeft() kicks ass
-bool KateRenderer::isLineRightToLeft(KateLineLayoutPtr lineLayout)
+bool KateRenderer::isLineRightToLeft(QStringView str)
 {
-    QString s = lineLayout->textLine()->text();
-    int i = 0;
-
     // borrowed from QString::updateProperties()
-    while (i != s.length()) {
-        QChar c = s.at(i);
-
+    for (auto c : str) {
         switch (c.direction()) {
         case QChar::DirL:
         case QChar::DirLRO:
@@ -1403,7 +1423,6 @@ bool KateRenderer::isLineRightToLeft(KateLineLayoutPtr lineLayout)
         default:
             break;
         }
-        i++;
     }
 
     return false;
@@ -1452,7 +1471,7 @@ KTextEditor::Cursor KateRenderer::xToCursor(const KateTextLayout &range, int x,
     }
     // TODO wrong for RTL lines?
     if (returnPastLine && range.endCol(true) == -1 && x > range.width() + range.xOffset()) {
-        ret.setColumn(ret.column() + ((x - (range.width() + range.xOffset())) / spaceWidth()));
+        ret.setColumn(ret.column() + round((x - (range.width() + range.xOffset())) / spaceWidth()));
     }
 
     return ret;
@@ -1493,13 +1512,13 @@ void KateRenderer::paintSelection(QPaintDevice *d, int startLine, int xStart, in
         }
 
         // compute layout WITHOUT cache to not poison it + render it
-        KateLineLayoutPtr lineLayout(new KateLineLayout(*this));
-        lineLayout->setLine(line, -1);
-        layoutLine(lineLayout, -1 /* no wrap */, false /* no layout cache */);
+        KateLineLayout lineLayout(*this);
+        lineLayout.setLine(line, -1);
+        layoutLine(&lineLayout, -1 /* no wrap */, false /* no layout cache */);
         KateRenderer::PaintTextLineFlags flags;
         flags.setFlag(KateRenderer::SkipDrawFirstInvisibleLineUnderlined);
         flags.setFlag(KateRenderer::SkipDrawLineSelection);
-        paintTextLine(paint, lineLayout, 0, 0, nullptr, flags);
+        paintTextLine(paint, &lineLayout, 0, 0, QRectF{}, nullptr, flags);
 
         // translate for next line
         paint.translate(0, lineHeight);
