@@ -1,12 +1,36 @@
 var katescript = {
     "author": "Dominik Haumann <dhdev@gmx.de>, Milian Wolff <mail@milianw.de>, Gerald Senarclens de Grancy <oss@senarclens.eu>, Alex Turbov <i.zaufi@gmail.com>, Pablo Rauzy <r_NOSPAM_@uzy.me>",
     "license": "LGPL-2.1+",
-    "revision": 9,
+    "revision": 11,
     "kate-version": "5.1",
-    "functions": ["sort", "moveLinesDown", "moveLinesUp", "natsort", "uniq", "rtrim", "ltrim", "trim", "join", "rmblank", "alignon", "unwrap", "each", "filter", "map", "duplicateLinesUp", "duplicateLinesDown", "rewrap", "encodeURISelection", "decodeURISelection", "fsel", "bsel"],
+    "functions": ["sort", "sortuniq", "moveLinesDown", "moveLinesUp", "natsort", "uniq", "rtrim", "ltrim", "trim", "join", "rmblank", "alignon", "unwrap", "each", "filter", "map", "duplicateLinesUp", "duplicateLinesDown", "rewrap", "encodeURISelection", "decodeURISelection", "fsel", "bsel"],
     "actions": [
         {   "function": "sort",
-            "name": "Sort Selected Text",
+            "name": "Sort Selected Text Alphabetically",
+            "category": "Editing"
+        },
+        {   "function": "sortuniq",
+            "name": "Remove Duplicates and Sort Selected Text Alphabetically",
+            "category": "Editing"
+        },
+        {   "function": "uniq",
+            "name": "Remove Duplicate Lines",
+            "category": "Editing"
+        },
+        {   "function": "rtrim",
+            "name": "Remove Trailing Spaces",
+            "category": "Editing"
+        },
+        {   "function": "ltrim",
+            "name": "Remove Leading Spaces",
+            "category": "Editing"
+        },
+        {   "function": "trim",
+            "name": "Remove Trailing and Leading Spaces",
+            "category": "Editing"
+        },
+        {   "function": "join",
+            "name": "Join Lines",
             "category": "Editing"
         },
         {   "function": "moveLinesDown",
@@ -34,6 +58,10 @@ var katescript = {
         {   "function": "decodeURISelection",
             "name": "URI-decode Selected Text",
             "category": "Editing"
+        },
+        {   "function": "rmblank",
+            "name": "Remove Empty Lines",
+            "category": "Editing"
         }
     ]
 }; // kate-script-header, must be at the start of the file without comments, pure json
@@ -61,6 +89,21 @@ function uniq()
     });
 }
 
+function sortuniq()
+{
+    each(function(lines) {
+        var uniq_lines = [];
+        var seen = new Set();
+        for ( var i = 0; i < lines.length; ++i ) {
+            if (!seen.has(lines[i])) {
+                seen.add(lines[i]);
+                uniq_lines.push(lines[i]);
+            }
+        }
+        return uniq_lines.sort();
+    });
+}
+
 function natsort()
 {
     each(function(lines){return lines.sort(natcompare);});
