@@ -33,6 +33,7 @@
 #include <QComboBox>
 #include <QCompleter>
 #include <QElapsedTimer>
+#include <QMenu>
 #include <QRegularExpression>
 #include <QShortcut>
 #include <QStringListModel>
@@ -56,16 +57,16 @@ namespace
 class AddMenuManager
 {
 private:
-    QVector<QString> m_insertBefore;
-    QVector<QString> m_insertAfter;
+    QList<QString> m_insertBefore;
+    QList<QString> m_insertAfter;
     QSet<QAction *> m_actionPointers;
     uint m_indexWalker;
     QMenu *m_menu;
 
 public:
     AddMenuManager(QMenu *parent, int expectedItemCount)
-        : m_insertBefore(QVector<QString>(expectedItemCount))
-        , m_insertAfter(QVector<QString>(expectedItemCount))
+        : m_insertBefore(QList<QString>(expectedItemCount))
+        , m_insertAfter(QList<QString>(expectedItemCount))
         , m_indexWalker(0)
         , m_menu(nullptr)
     {
@@ -120,7 +121,7 @@ public:
         const QString &before = m_insertBefore[index];
         const QString &after = m_insertAfter[index];
         lineEdit->insert(before + after);
-        lineEdit->setCursorPosition(cursorPos + before.count());
+        lineEdit->setCursorPosition(cursorPos + before.size());
         lineEdit->setFocus();
     }
 };
@@ -613,13 +614,13 @@ bool KateSearchBar::findOrReplace(SearchDirection searchDirection, const QString
     if (askWrap) {
         QString question =
             searchDirection == SearchForward ? i18n("Bottom of file reached. Continue from top?") : i18n("Top of file reached. Continue from bottom?");
-        wrap = (KMessageBox::questionYesNo(nullptr,
-                                           question,
-                                           i18n("Continue search?"),
-                                           KStandardGuiItem::yes(),
-                                           KStandardGuiItem::no(),
-                                           QStringLiteral("DoNotShowAgainContinueSearchDialog"))
-                == KMessageBox::Yes);
+        wrap = (KMessageBox::questionTwoActions(nullptr,
+                                                question,
+                                                i18n("Continue search?"),
+                                                KStandardGuiItem::cont(),
+                                                KStandardGuiItem::cancel(),
+                                                QStringLiteral("DoNotShowAgainContinueSearchDialog"))
+                == KMessageBox::PrimaryAction);
     }
     if (wrap) {
         m_view->showSearchWrappedHint(searchDirection == SearchBackward);
@@ -635,8 +636,10 @@ bool KateSearchBar::findOrReplace(SearchDirection searchDirection, const QString
         selectRange2(match.range());
     }
 
-    const MatchResult matchResult =
-        !match.isValid() ? MatchMismatch : !wrap ? MatchFound : searchDirection == SearchForward ? MatchWrappedForward : MatchWrappedBackward;
+    const MatchResult matchResult = !match.isValid() ? MatchMismatch
+        : !wrap                                      ? MatchFound
+        : searchDirection == SearchForward           ? MatchWrappedForward
+                                                     : MatchWrappedBackward;
     indicateMatch(matchResult);
 
     // highlight replacements if applicable
@@ -673,8 +676,8 @@ bool KateSearchBar::isPatternValid() const
     }
 
     return searchOptions().testFlag(WholeWords) ? searchPattern().trimmed() == searchPattern()
-                                                : searchOptions().testFlag(Regex)
-                                                ? QRegularExpression(searchPattern(), QRegularExpression::UseUnicodePropertiesOption).isValid() : true;
+        : searchOptions().testFlag(Regex)       ? QRegularExpression(searchPattern(), QRegularExpression::UseUnicodePropertiesOption).isValid()
+                                                : true;
 }
 
 void KateSearchBar::givePatternFeedback()
@@ -838,7 +841,7 @@ void KateSearchBar::findOrReplaceAll()
             Range lastRange;
             if (m_replaceMode) {
                 if (m_matchCounter == 0) {
-                    static_cast<KTextEditor::DocumentPrivate *>(m_view->document())->startEditing();
+                    static_cast<KTextEditor::DocumentPrivate *>(m_view->document())->editStart();
                 }
 
                 // Replace
@@ -908,19 +911,16 @@ void KateSearchBar::endFindOrReplaceAll()
     // After last match
     if (m_matchCounter > 0) {
         if (m_replaceMode) {
-            static_cast<KTextEditor::DocumentPrivate *>(m_view->document())->finishEditing();
+            static_cast<KTextEditor::DocumentPrivate *>(m_view->document())->editEnd();
         }
     }
 
     // Add ScrollBarMarks
     if (!m_highlightRanges.empty()) {
-        KTextEditor::MarkInterfaceV2 *iface = qobject_cast<KTextEditor::MarkInterfaceV2 *>(m_view->document());
-        if (iface) {
-            iface->setMarkDescription(KTextEditor::MarkInterface::SearchMatch, i18n("SearchHighLight"));
-            iface->setMarkIcon(KTextEditor::MarkInterface::SearchMatch, QIcon());
-            for (const Range &r : m_highlightRanges) {
-                iface->addMark(r.start().line(), KTextEditor::MarkInterface::SearchMatch);
-            }
+        m_view->document()->setMarkDescription(KTextEditor::Document::SearchMatch, i18n("SearchHighLight"));
+        m_view->document()->setMarkIcon(KTextEditor::Document::SearchMatch, QIcon());
+        for (const Range &r : m_highlightRanges) {
+            m_view->document()->addMark(r.start().line(), KTextEditor::Document::SearchMatch);
         }
     }
 
@@ -1053,9 +1053,9 @@ struct ParInfo {
     int captureNumber; // 1..9
 };
 
-QVector<QString> KateSearchBar::getCapturePatterns(const QString &pattern) const
+QList<QString> KateSearchBar::getCapturePatterns(const QString &pattern) const
 {
-    QVector<QString> capturePatterns;
+    QList<QString> capturePatterns;
     capturePatterns.reserve(9);
     QStack<ParInfo> parInfos;
 
@@ -1181,7 +1181,7 @@ void KateSearchBar::showExtendedContextMenu(bool forPattern, const QPoint &pos)
             addMenuManager.addSeparator();
             if (regexMode) {
                 const QString pattern = m_powerUi->pattern->currentText();
-                const QVector<QString> capturePatterns = getCapturePatterns(pattern);
+                const QList<QString> capturePatterns = getCapturePatterns(pattern);
 
                 const int captureCount = capturePatterns.count();
                 for (int i = 1; i <= 9; i++) {
@@ -1447,7 +1447,7 @@ void KateSearchBar::enterPowerMode()
         connect(m_powerUi->findPrev, &QToolButton::clicked, this, &KateSearchBar::findPrevious);
         connect(m_powerUi->replaceNext, &QPushButton::clicked, this, &KateSearchBar::replaceNext);
         connect(m_powerUi->replaceAll, &QPushButton::clicked, this, &KateSearchBar::replaceAll);
-        connect(m_powerUi->searchMode, qOverload<int>(&QComboBox::currentIndexChanged), this, &KateSearchBar::onPowerModeChanged);
+        connect(m_powerUi->searchMode, &QComboBox::currentIndexChanged, this, &KateSearchBar::onPowerModeChanged);
         connect(m_powerUi->matchCase, &QToolButton::toggled, this, &KateSearchBar::onMatchCaseToggled);
         connect(m_powerUi->findAll, &QPushButton::clicked, this, &KateSearchBar::findAll);
         connect(m_powerUi->cancel, &QPushButton::clicked, this, &KateSearchBar::onPowerCancelFindOrReplace);
@@ -1617,15 +1617,12 @@ void KateSearchBar::enterIncrementalMode()
 bool KateSearchBar::clearHighlights()
 {
     // Remove ScrollBarMarks
-    KTextEditor::MarkInterface *iface = qobject_cast<KTextEditor::MarkInterface *>(m_view->document());
-    if (iface) {
-        const QHash<int, KTextEditor::Mark *> marks = iface->marks();
-        QHashIterator<int, KTextEditor::Mark *> i(marks);
-        while (i.hasNext()) {
-            i.next();
-            if (i.value()->type & KTextEditor::MarkInterface::SearchMatch) {
-                iface->removeMark(i.value()->line, KTextEditor::MarkInterface::SearchMatch);
-            }
+    const QHash<int, KTextEditor::Mark *> &marks = m_view->document()->marks();
+    QHashIterator<int, KTextEditor::Mark *> i(marks);
+    while (i.hasNext()) {
+        i.next();
+        if (i.value()->type & KTextEditor::Document::SearchMatch) {
+            m_view->document()->removeMark(i.value()->line, KTextEditor::Document::SearchMatch);
         }
     }
 
@@ -1643,7 +1640,7 @@ bool KateSearchBar::clearHighlights()
 
 void KateSearchBar::updateHighlightColors()
 {
-    const QColor foregroundColor = m_view->defaultStyleAttribute(KTextEditor::dsNormal)->foreground().color();
+    const QColor foregroundColor = m_view->defaultStyleAttribute(KSyntaxHighlighting::Theme::TextStyle::Normal)->foreground().color();
     const QColor &searchColor = m_view->renderer()->config()->searchHighlightColor();
     const QColor &replaceColor = m_view->renderer()->config()->replaceHighlightColor();
 
@@ -1774,3 +1771,5 @@ void KateSearchBar::slotReadWriteChanged()
     m_powerUi->replaceNext->setEnabled(m_view->doc()->isReadWrite() && isPatternValid());
     m_powerUi->replaceAll->setEnabled(m_view->doc()->isReadWrite() && isPatternValid());
 }
+
+#include "moc_katesearchbar.cpp"
