@@ -112,7 +112,7 @@ void KateOnTheFlyChecker::handleRespellCheckBlock(int start, int end)
     m_modificationList.push_back(ModificationItem(TEXT_INSERTED, movingRange));
     ON_THE_FLY_DEBUG << "added" << *movingRange;
     if (listEmpty) {
-        QTimer::singleShot(0, this, SLOT(handleModifiedRanges()));
+        QTimer::singleShot(0, this, &KateOnTheFlyChecker::handleModifiedRanges);
     }
 }
 
@@ -146,7 +146,7 @@ void KateOnTheFlyChecker::textInserted(KTextEditor::Document *document, KTextEdi
     }
 
     if (listEmptyAtStart && !m_modificationList.isEmpty()) {
-        QTimer::singleShot(0, this, SLOT(handleModifiedRanges()));
+        QTimer::singleShot(0, this, &KateOnTheFlyChecker::handleModifiedRanges);
     }
 }
 
@@ -200,7 +200,7 @@ void KateOnTheFlyChecker::handleInsertedText(KTextEditor::Range range)
     queueSpellCheckVisibleRange(spellCheckRange);
 
     if (spellCheckInProgress || (emptyAtStart && !m_spellCheckQueue.isEmpty())) {
-        QTimer::singleShot(0, this, SLOT(performSpellCheck()));
+        QTimer::singleShot(0, this, &KateOnTheFlyChecker::performSpellCheck);
     }
 }
 
@@ -234,7 +234,7 @@ void KateOnTheFlyChecker::textRemoved(KTextEditor::Document *document, KTextEdit
         }
     }
     if (listEmptyAtStart && !m_modificationList.isEmpty()) {
-        QTimer::singleShot(0, this, SLOT(handleModifiedRanges()));
+        QTimer::singleShot(0, this, &KateOnTheFlyChecker::handleModifiedRanges);
     }
 }
 
@@ -310,7 +310,7 @@ void KateOnTheFlyChecker::handleRemovedText(KTextEditor::Range range)
 
     ON_THE_FLY_DEBUG << "finished";
     if (spellCheckInProgress || (emptyAtStart && !m_spellCheckQueue.isEmpty())) {
-        QTimer::singleShot(0, this, SLOT(performSpellCheck()));
+        QTimer::singleShot(0, this, &KateOnTheFlyChecker::performSpellCheck);
     }
 }
 
@@ -443,7 +443,7 @@ bool KateOnTheFlyChecker::removeRangeFromSpellCheckQueue(KTextEditor::MovingRang
 {
     if (removeRangeFromCurrentSpellCheck(range)) {
         if (!m_spellCheckQueue.isEmpty()) {
-            QTimer::singleShot(0, this, SLOT(performSpellCheck()));
+            QTimer::singleShot(0, this, &KateOnTheFlyChecker::performSpellCheck);
         }
         return true;
     }
@@ -471,18 +471,6 @@ void KateOnTheFlyChecker::rangeInvalid(KTextEditor::MovingRange *range)
     deleteMovingRange(range);
 }
 
-void KateOnTheFlyChecker::mouseEnteredRange(KTextEditor::MovingRange *range, KTextEditor::View *view)
-{
-    KTextEditor::ViewPrivate *kateView = static_cast<KTextEditor::ViewPrivate *>(view);
-    kateView->spellingMenu()->mouseEnteredMisspelledRange(range);
-}
-
-void KateOnTheFlyChecker::mouseExitedRange(KTextEditor::MovingRange *range, KTextEditor::View *view)
-{
-    KTextEditor::ViewPrivate *kateView = static_cast<KTextEditor::ViewPrivate *>(view);
-    kateView->spellingMenu()->mouseExitedMisspelledRange(range);
-}
-
 /**
  * It is not enough to use 'caret/Entered/ExitedRange' only as the cursor doesn't move when some
  * text has been selected.
@@ -615,16 +603,16 @@ void KateOnTheFlyChecker::spellCheckDone()
     deleteMovingRangeQuickly(movingRange);
 
     if (!m_spellCheckQueue.empty()) {
-        QTimer::singleShot(0, this, SLOT(performSpellCheck()));
+        QTimer::singleShot(0, this, &KateOnTheFlyChecker::performSpellCheck);
     }
 }
 
-QList<KTextEditor::MovingRange *> KateOnTheFlyChecker::installedMovingRanges(KTextEditor::Range range)
+QList<KTextEditor::MovingRange *> KateOnTheFlyChecker::installedMovingRanges(KTextEditor::Range range) const
 {
     ON_THE_FLY_DEBUG << range;
     MovingRangeList toReturn;
 
-    for (QList<SpellCheckItem>::iterator i = m_misspelledList.begin(); i != m_misspelledList.end(); ++i) {
+    for (QList<SpellCheckItem>::const_iterator i = m_misspelledList.begin(); i != m_misspelledList.end(); ++i) {
         KTextEditor::MovingRange *movingRange = (*i).first;
         if (movingRange->overlaps(range)) {
             toReturn.push_back(movingRange);
@@ -664,13 +652,13 @@ void KateOnTheFlyChecker::viewDestroyed(QObject *obj)
 {
     ON_THE_FLY_DEBUG;
     KTextEditor::View *view = static_cast<KTextEditor::View *>(obj);
-    m_displayRangeMap.remove(view);
+    m_displayRangeMap.erase(view);
 }
 
 void KateOnTheFlyChecker::removeView(KTextEditor::View *view)
 {
     ON_THE_FLY_DEBUG;
-    m_displayRangeMap.remove(view);
+    m_displayRangeMap.erase(view);
 }
 
 void KateOnTheFlyChecker::updateInstalledMovingRanges(KTextEditor::ViewPrivate *view)
@@ -721,7 +709,7 @@ void KateOnTheFlyChecker::updateInstalledMovingRanges(KTextEditor::ViewPrivate *
             }
         }
         if (emptyAtStart && !m_spellCheckQueue.isEmpty()) {
-            QTimer::singleShot(0, this, SLOT(performSpellCheck()));
+            QTimer::singleShot(0, this, &KateOnTheFlyChecker::performSpellCheck);
         }
     }
 }
