@@ -14,6 +14,7 @@
 #include "katehighlight.h"
 
 #include "katedocument.h"
+#include "kateextendedattribute.h"
 #include "katesyntaxmanager.h"
 // END
 
@@ -97,7 +98,7 @@ KateHighlighting::KateHighlighting(const KSyntaxHighlighting::Definition &def)
         for (const auto &format : formats) {
             // register format id => internal attributes, we want no clashs
             const auto nextId = m_formats.size();
-            m_formatsIdToIndex.insert(std::make_pair(format.id(), nextId));
+            m_formatsIdToIndex.insert(std::make_pair(format.id(), int(nextId)));
             m_formats.push_back(format);
             m_propertiesForFormat.push_back(&properties);
         }
@@ -178,12 +179,11 @@ void KateHighlighting::applyFolding(int offset, int length, KSyntaxHighlighting:
     // WE ATM assume ascending offset order, we add the length to the offset for the folding ends to have ranges spanning the full folding region
     Q_ASSERT(m_textLineToHighlight);
     Q_ASSERT(region.isValid());
-    const int foldingValue = (region.type() == KSyntaxHighlighting::FoldingRegion::Begin) ? int(region.id()) : -int(region.id());
-    m_textLineToHighlight->addFolding(offset + ((region.type() == KSyntaxHighlighting::FoldingRegion::Begin) ? 0 : length), length, foldingValue);
+    m_textLineToHighlight->addFolding(offset + ((region.type() == KSyntaxHighlighting::FoldingRegion::Begin) ? 0 : length), length, region);
 
     // for each end region, decrement counter for that type, erase if count reaches 0!
-    if (foldingValue < 0) {
-        QHash<int, int>::iterator end = m_foldingStartToCount.find(-foldingValue);
+    if (region.type() == KSyntaxHighlighting::FoldingRegion::End) {
+        QHash<int, int>::iterator end = m_foldingStartToCount.find(region.id());
         if (end != m_foldingStartToCount.end()) {
             if (end.value() > 1) {
                 --(end.value());
@@ -194,8 +194,8 @@ void KateHighlighting::applyFolding(int offset, int length, KSyntaxHighlighting:
     }
 
     // increment counter for each begin region!
-    if (foldingValue > 0) {
-        ++m_foldingStartToCount[foldingValue];
+    else {
+        ++m_foldingStartToCount[region.id()];
     }
 }
 
@@ -228,9 +228,9 @@ bool KateHighlighting::attributeRequiresSpellchecking(int attr)
     return m_formats[sanitizeFormatIndex(attr)].spellCheck();
 }
 
-KTextEditor::DefaultStyle KateHighlighting::defaultStyleForAttribute(int attr) const
+KSyntaxHighlighting::Theme::TextStyle KateHighlighting::defaultStyleForAttribute(int attr) const
 {
-    return textStyleToDefaultStyle(m_formats[sanitizeFormatIndex(attr)].textStyle());
+    return m_formats[sanitizeFormatIndex(attr)].textStyle();
 }
 
 QString KateHighlighting::nameForAttrib(int attrib) const
@@ -251,7 +251,7 @@ bool KateHighlighting::canBreakAt(QChar c, int attrib) const
     return m_propertiesForFormat.at(sanitizeFormatIndex(attrib))->definition.isWordWrapDelimiter(c) && c != QLatin1Char('"') && c != QLatin1Char('\'');
 }
 
-const QVector<QRegularExpression> &KateHighlighting::emptyLines(int attrib) const
+const QList<QRegularExpression> &KateHighlighting::emptyLines(int attrib) const
 {
     return m_propertiesForFormat.at(sanitizeFormatIndex(attrib))->emptyLines;
 }
@@ -296,15 +296,15 @@ void KateHighlighting::clearAttributeArrays()
     m_attributeArrays.clear();
 }
 
-QVector<KTextEditor::Attribute::Ptr> KateHighlighting::attributesForDefinition(const QString &schema) const
+QList<KTextEditor::Attribute::Ptr> KateHighlighting::attributesForDefinition(const QString &schema) const
 {
     // create list of known attributes based on highlighting format & wanted theme
-    QVector<KTextEditor::Attribute::Ptr> array;
+    QList<KTextEditor::Attribute::Ptr> array;
     array.reserve(m_formats.size());
     const auto currentTheme = KateHlManager::self()->repository().theme(schema);
     for (const auto &format : m_formats) {
         // create a KTextEditor attribute matching the given format
-        KTextEditor::Attribute::Ptr newAttribute(new KTextEditor::Attribute(nameForAttrib(array.size()), textStyleToDefaultStyle(format.textStyle())));
+        KTextEditor::Attribute::Ptr newAttribute(new KTextEditor::Attribute(nameForAttrib(array.size()), format.textStyle()));
 
         if (const auto color = format.textColor(currentTheme).rgba()) {
             newAttribute->setForeground(QColor::fromRgba(color));
@@ -347,7 +347,7 @@ QVector<KTextEditor::Attribute::Ptr> KateHighlighting::attributesForDefinition(c
     return array;
 }
 
-QVector<KTextEditor::Attribute::Ptr> KateHighlighting::attributes(const QString &schema)
+QList<KTextEditor::Attribute::Ptr> KateHighlighting::attributes(const QString &schema)
 {
     // query cache first
     if (m_attributeArrays.contains(schema)) {
@@ -378,7 +378,7 @@ bool KateHighlighting::isEmptyLine(const Kate::TextLineData *textline) const
     }
 
     for (const QRegularExpression &re : l) {
-        const QRegularExpressionMatch match = re.match(txt, 0, QRegularExpression::NormalMatch, QRegularExpression::AnchoredMatchOption);
+        const QRegularExpressionMatch match = re.match(txt, 0, QRegularExpression::NormalMatch, QRegularExpression::AnchorAtOffsetMatchOption);
         if (match.hasMatch() && match.capturedLength() == txt.length()) {
             return true;
         }
@@ -406,7 +406,7 @@ int KateHighlighting::attributeForLocation(KTextEditor::DocumentPrivate *doc, co
     if (cursor.column() < tl->length()) {
         return sanitizeFormatIndex(tl->attribute(cursor.column()));
     } else if (cursor.column() >= tl->length()) {
-        if (!tl->attributesList().isEmpty()) {
+        if (!tl->attributesList().empty()) {
             return sanitizeFormatIndex(tl->attributesList().back().attributeValue);
         }
     }
