@@ -14,13 +14,13 @@
 #include "katesyntaxmanager.h"
 #include "kateview.h"
 
-#include <KCharsets>
 #include <KConfigGroup>
 
 #include <QGuiApplication>
 #include <QSettings>
+#include <QStringDecoder>
+#include <QStringEncoder>
 #include <QStringListModel>
-#include <QTextCodec>
 
 #include <Sonnet/GuessLanguage>
 #include <Sonnet/Speller>
@@ -40,8 +40,11 @@ void KateConfig::addConfigEntry(ConfigEntry &&entry)
     // shall only be called for toplevel config
     Q_ASSERT(isGlobal());
 
-    // there shall be no gaps in the entries
+    // There shall be no gaps in the entries; i.e. in KateViewConfig constructor
+    // addConfigEntry() is called on each value from the ConfigEntryTypes enum in
+    // the same order as the enumrators.
     // we might later want to use a vector
+    // qDebug() << m_configEntries.size() << entry.enumKey;
     Q_ASSERT(m_configEntries.size() == static_cast<size_t>(entry.enumKey));
 
     // add new element
@@ -94,8 +97,6 @@ void KateConfig::configStart()
     if (configSessionNumber > 1) {
         return;
     }
-
-    configIsRunning = true;
 }
 
 void KateConfig::configEnd()
@@ -110,8 +111,6 @@ void KateConfig::configEnd()
         return;
     }
 
-    configIsRunning = false;
-
     updateConfig();
 }
 
@@ -215,9 +214,7 @@ KateRendererConfig *KateRendererConfig::s_global = nullptr;
  */
 static bool isEncodingOk(const QString &name)
 {
-    bool found = false;
-    auto codec = KCharsets::charsets()->codecForName(name, found);
-    return found && codec;
+    return QStringDecoder(name.toUtf8().constData()).isValid() && QStringEncoder(name.toUtf8().constData()).isValid();
 }
 
 static bool inBounds(const int min, const QVariant &value, const int max)
@@ -241,18 +238,30 @@ KateGlobalConfig::KateGlobalConfig()
     Q_ASSERT(isGlobal());
     s_global = this;
 
+    // avoid updateConfig effects like config write in constructor, see bug 377067
+    Q_ASSERT(configSessionNumber == 0);
+    ++configSessionNumber;
+
     // init all known config entries
     addConfigEntry(ConfigEntry(EncodingProberType, "Encoding Prober Type", QString(), KEncodingProber::Universal));
-    addConfigEntry(ConfigEntry(FallbackEncoding, "Fallback Encoding", QString(), QStringLiteral("ISO 8859-15"), [](const QVariant &value) {
-        return isEncodingOk(value.toString());
-    }));
+    addConfigEntry(ConfigEntry(FallbackEncoding,
+                               "Fallback Encoding",
+                               QString(),
+                               QString::fromUtf8(QStringConverter::nameForEncoding(QStringConverter::Latin1)),
+                               [](const QVariant &value) {
+                                   return isEncodingOk(value.toString());
+                               }));
 
     // finalize the entries, e.g. hashs them
     finalizeConfigEntries();
 
     // init with defaults from config or really hardcoded ones
-    KConfigGroup cg(KTextEditor::EditorPrivate::config(), "KTextEditor Editor");
+    KConfigGroup cg(KTextEditor::EditorPrivate::config(), QStringLiteral("KTextEditor Editor"));
     readConfig(cg);
+
+    // avoid updateConfig effects like config write in constructor, see bug 377067
+    Q_ASSERT(configSessionNumber == 1);
+    --configSessionNumber;
 }
 
 void KateGlobalConfig::readConfig(const KConfigGroup &config)
@@ -276,25 +285,13 @@ void KateGlobalConfig::writeConfig(KConfigGroup &config)
 void KateGlobalConfig::updateConfig()
 {
     // write config
-    KConfigGroup cg(KTextEditor::EditorPrivate::config(), "KTextEditor Editor");
+    KConfigGroup cg(KTextEditor::EditorPrivate::config(), QStringLiteral("KTextEditor Editor"));
     writeConfig(cg);
     KTextEditor::EditorPrivate::config()->sync();
 
     // trigger emission of KTextEditor::Editor::configChanged
     KTextEditor::EditorPrivate::self()->triggerConfigChanged();
 }
-
-QTextCodec *KateGlobalConfig::fallbackCodec() const
-{
-    // query stored encoding, always fallback to ISO 8859-15 if nothing valid set
-    const auto encoding = value(FallbackEncoding).toString();
-    if (encoding.isEmpty()) {
-        return QTextCodec::codecForName("ISO 8859-15");
-    }
-
-    // use configured encoding
-    return KCharsets::charsets()->codecForName(encoding);
-}
 // END
 
 // BEGIN KateDocumentConfig
@@ -304,6 +301,10 @@ KateDocumentConfig::KateDocumentConfig()
     Q_ASSERT(isGlobal());
     s_global = this;
 
+    // avoid updateConfig effects like config write in constructor, see bug 377067
+    Q_ASSERT(configSessionNumber == 0);
+    ++configSessionNumber;
+
     // init all known config entries
     addConfigEntry(ConfigEntry(TabWidth, "Tab Width", QStringLiteral("tab-width"), 4, [](const QVariant &value) {
         return value.toInt() >= 1;
@@ -312,7 +313,7 @@ KateDocumentConfig::KateDocumentConfig()
         return value.toInt() >= 1;
     }));
     addConfigEntry(ConfigEntry(OnTheFlySpellCheck, "On-The-Fly Spellcheck", QStringLiteral("on-the-fly-spellcheck"), false));
-    addConfigEntry(ConfigEntry(IndentOnTextPaste, "Indent On Text Paste", QStringLiteral("indent-pasted-text"), false));
+    addConfigEntry(ConfigEntry(IndentOnTextPaste, "Indent On Text Paste", QStringLiteral("indent-pasted-text"), true));
     addConfigEntry(ConfigEntry(ReplaceTabsWithSpaces, "ReplaceTabsDyn", QStringLiteral("replace-tabs"), true));
     addConfigEntry(ConfigEntry(BackupOnSaveLocal, "Backup Local", QStringLiteral("backup-on-save-local"), false));
     addConfigEntry(ConfigEntry(BackupOnSaveRemote, "Backup Remote", QStringLiteral("backup-on-save-remote"), false));
@@ -337,9 +338,10 @@ KateDocumentConfig::KateDocumentConfig()
     }));
     addConfigEntry(ConfigEntry(NewlineAtEOF, "Newline at End of File", QString(), true));
     addConfigEntry(ConfigEntry(OverwriteMode, "Overwrite Mode", QString(), false));
-    addConfigEntry(ConfigEntry(Encoding, "Encoding", QString(), QStringLiteral("UTF-8"), [](const QVariant &value) {
-        return isEncodingOk(value.toString());
-    }));
+    addConfigEntry(
+        ConfigEntry(Encoding, "Encoding", QString(), QString::fromUtf8(QStringConverter::nameForEncoding(QStringConverter::Utf8)), [](const QVariant &value) {
+            return isEncodingOk(value.toString());
+        }));
     addConfigEntry(ConfigEntry(EndOfLine, "End of Line", QString(), 0));
     addConfigEntry(ConfigEntry(AllowEndOfLineDetection, "Allow End of Line Detection", QString(), true));
     addConfigEntry(ConfigEntry(ByteOrderMark, "BOM", QString(), false));
@@ -350,12 +352,24 @@ KateDocumentConfig::KateDocumentConfig()
     addConfigEntry(ConfigEntry(CamelCursor, "Camel Cursor", QString(), true));
     addConfigEntry(ConfigEntry(AutoDetectIndent, "Auto Detect Indent", QString(), true));
 
+    // Auto save and co.
+    addConfigEntry(ConfigEntry(AutoSave, "Auto Save", QString(), false));
+    addConfigEntry(ConfigEntry(AutoSaveOnFocusOut, "Auto Save On Focus Out", QString(), false));
+    addConfigEntry(ConfigEntry(AutoSaveInteral, "Auto Save Interval", QString(), 0));
+
+    // Shall we do auto reloading for stuff e.g. in Git?
+    addConfigEntry(ConfigEntry(AutoReloadIfStateIsInVersionControl, "Auto Reload If State Is In Version Control", QString(), true));
+
     // finalize the entries, e.g. hashs them
     finalizeConfigEntries();
 
     // init with defaults from config or really hardcoded ones
-    KConfigGroup cg(KTextEditor::EditorPrivate::config(), "KTextEditor Document");
+    KConfigGroup cg(KTextEditor::EditorPrivate::config(), QStringLiteral("KTextEditor Document"));
     readConfig(cg);
+
+    // avoid updateConfig effects like config write in constructor, see bug 377067
+    Q_ASSERT(configSessionNumber == 1);
+    --configSessionNumber;
 }
 
 KateDocumentConfig::KateDocumentConfig(KTextEditor::DocumentPrivate *doc)
@@ -421,12 +435,13 @@ void KateDocumentConfig::updateConfig()
     }
 
     if (isGlobal()) {
-        for (int z = 0; z < KTextEditor::EditorPrivate::self()->kateDocuments().size(); ++z) {
-            (KTextEditor::EditorPrivate::self()->kateDocuments())[z]->updateConfig();
+        const auto docs = KTextEditor::EditorPrivate::self()->documents();
+        for (auto doc : docs) {
+            static_cast<KTextEditor::DocumentPrivate *>(doc)->updateConfig();
         }
 
         // write config
-        KConfigGroup cg(KTextEditor::EditorPrivate::config(), "KTextEditor Document");
+        KConfigGroup cg(KTextEditor::EditorPrivate::config(), QStringLiteral("KTextEditor Document"));
         writeConfig(cg);
         KTextEditor::EditorPrivate::config()->sync();
 
@@ -435,19 +450,7 @@ void KateDocumentConfig::updateConfig()
     }
 }
 
-QTextCodec *KateDocumentConfig::codec() const
-{
-    // query stored encoding, always fallback to UTF-8 if nothing valid set
-    const auto encoding = value(Encoding).toString();
-    if (encoding.isEmpty()) {
-        return QTextCodec::codecForName("UTF-8");
-    }
-
-    // use configured encoding
-    return KCharsets::charsets()->codecForName(encoding);
-}
-
-QString KateDocumentConfig::eolString()
+QString KateDocumentConfig::eolString() const
 {
     switch (eol()) {
     case KateDocumentConfig::eolDos:
@@ -465,26 +468,32 @@ QString KateDocumentConfig::eolString()
 // BEGIN KateViewConfig
 KateViewConfig::KateViewConfig()
 {
+    // register this as our global instance
+    Q_ASSERT(isGlobal());
     s_global = this;
 
+    // avoid updateConfig effects like config write in constructor, see bug 377067
+    Q_ASSERT(configSessionNumber == 0);
+    ++configSessionNumber;
+
     // Init all known config entries
     // NOTE: Ensure to keep the same order as listed in enum ConfigEntryTypes or it will later assert!
     // addConfigEntry(ConfigEntry(<EnumKey>, <ConfigKey>, <CommandName>, <DefaultValue>,  [<ValidatorFunction>]))
     addConfigEntry(ConfigEntry(AllowMarkMenu, "Allow Mark Menu", QStringLiteral("allow-mark-menu"), true));
-    addConfigEntry(ConfigEntry(AutoBrackets, "Auto Brackets", QStringLiteral("auto-brackets"), false));
+    addConfigEntry(ConfigEntry(AutoBrackets, "Auto Brackets", QStringLiteral("auto-brackets"), true));
     addConfigEntry(ConfigEntry(AutoCenterLines, "Auto Center Lines", QStringLiteral("auto-center-lines"), 0));
     addConfigEntry(ConfigEntry(AutomaticCompletionInvocation, "Auto Completion", QString(), true));
     addConfigEntry(ConfigEntry(AutomaticCompletionPreselectFirst, "Auto Completion Preselect First Entry", QString(), true));
     addConfigEntry(ConfigEntry(BackspaceRemoveComposedCharacters, "Backspace Remove Composed Characters", QString(), false));
     addConfigEntry(ConfigEntry(BookmarkSorting, "Bookmark Menu Sorting", QString(), 0));
     addConfigEntry(ConfigEntry(CharsToEncloseSelection, "Chars To Enclose Selection", QStringLiteral("enclose-selection"), QStringLiteral("<>(){}[]'\"")));
-    addConfigEntry(ConfigEntry(DefaultMarkType,
-                               "Default Mark Type",
-                               QStringLiteral("default-mark-type"),
-                               KTextEditor::MarkInterface::markType01,
-                               [](const QVariant &value) {
-                                   return isPositive(value);
-                               }));
+    addConfigEntry(ConfigEntry(ClipboardHistoryEntries, "Max Clipboard History Entries", QString(), 20, [](const QVariant &value) {
+        return inBounds(1, value, 999);
+    }));
+    addConfigEntry(
+        ConfigEntry(DefaultMarkType, "Default Mark Type", QStringLiteral("default-mark-type"), KTextEditor::Document::markType01, [](const QVariant &value) {
+            return isPositive(value);
+        }));
     addConfigEntry(ConfigEntry(DynWordWrapAlignIndent, "Dynamic Word Wrap Align Indent", QString(), 80, [](const QVariant &value) {
         return inBounds(0, value, 100);
     }));
@@ -509,6 +518,7 @@ KateViewConfig::KateViewConfig()
     }));
     addConfigEntry(ConfigEntry(ScrollPastEnd, "Scroll Past End", QString(), false));
     addConfigEntry(ConfigEntry(SearchFlags, "Search/Replace Flags", QString(), IncFromCursor | PowerMatchCase | PowerModePlainText));
+    addConfigEntry(ConfigEntry(TabCompletion, "Enable Tab completion", QString(), false));
     addConfigEntry(ConfigEntry(ShowBracketMatchPreview, "Bracket Match Preview", QStringLiteral("bracket-match-preview"), false));
     addConfigEntry(ConfigEntry(ShowFoldingBar, "Folding Bar", QStringLiteral("folding-bar"), true));
     addConfigEntry(ConfigEntry(ShowFoldingPreview, "Folding Preview", QStringLiteral("folding-preview"), true));
@@ -530,19 +540,40 @@ KateViewConfig::KateViewConfig()
     addConfigEntry(ConfigEntry(ViInputModeStealKeys, "Vi Input Mode Steal Keys", QString(), false));
     addConfigEntry(ConfigEntry(ViRelativeLineNumbers, "Vi Relative Line Numbers", QString(), false));
     addConfigEntry(ConfigEntry(WordCompletion, "Word Completion", QString(), true));
-    addConfigEntry(ConfigEntry(WordCompletionMinimalWordLength, "Word Completion Minimal Word Length", QString(), 3, [](const QVariant &value) {
-        return inBounds(0, value, 99);
-    }));
+    addConfigEntry(ConfigEntry(WordCompletionMinimalWordLength,
+                               "Word Completion Minimal Word Length",
+                               QStringLiteral("word-completion-minimal-word-length"),
+                               3,
+                               [](const QVariant &value) {
+                                   return inBounds(0, value, 99);
+                               }));
     addConfigEntry(ConfigEntry(WordCompletionRemoveTail, "Word Completion Remove Tail", QString(), true));
     addConfigEntry(ConfigEntry(ShowFocusFrame, "Show Focus Frame Around Editor", QString(), true));
     addConfigEntry(ConfigEntry(ShowDocWithCompletion, "Show Documentation With Completion", QString(), true));
+    addConfigEntry(ConfigEntry(MultiCursorModifier, "Multiple Cursor Modifier", QString(), (int)Qt::AltModifier));
+    addConfigEntry(ConfigEntry(ShowFoldingOnHoverOnly, "Show Folding Icons On Hover Only", QString(), true));
+
+    // Statusbar stuff
+    addConfigEntry(ConfigEntry(ShowStatusbarLineColumn, "Show Statusbar Line Column", QString(), true));
+    addConfigEntry(ConfigEntry(ShowStatusbarDictionary, "Show Statusbar Dictionary", QString(), true));
+    addConfigEntry(ConfigEntry(ShowStatusbarInputMode, "Show Statusbar Input Mode", QString(), true));
+    addConfigEntry(ConfigEntry(ShowStatusbarHighlightingMode, "Show Statusbar Highlighting Mode", QString(), true));
+    addConfigEntry(ConfigEntry(ShowStatusbarTabSettings, "Show Statusbar Tab Settings", QString(), true));
+    addConfigEntry(ConfigEntry(ShowStatusbarFileEncoding, "Show File Encoding", QString(), true));
+    addConfigEntry(ConfigEntry(StatusbarLineColumnCompact, "Statusbar Line Column Compact Mode", QString(), true));
+    addConfigEntry(ConfigEntry(ShowStatusbarEOL, "Shoe Line Ending Type in Statusbar", QString(), false));
+    addConfigEntry(ConfigEntry(EnableAccessibility, "Enable Accessibility", QString(), true));
 
     // Never forget to finalize or the <CommandName> becomes not available
     finalizeConfigEntries();
 
     // init with defaults from config or really hardcoded ones
-    KConfigGroup config(KTextEditor::EditorPrivate::config(), "KTextEditor View");
+    KConfigGroup config(KTextEditor::EditorPrivate::config(), QStringLiteral("KTextEditor View"));
     readConfig(config);
+
+    // avoid updateConfig effects like config write in constructor, see bug 377067
+    Q_ASSERT(configSessionNumber == 1);
+    --configSessionNumber;
 }
 
 KateViewConfig::KateViewConfig(KTextEditor::ViewPrivate *view)
@@ -577,13 +608,12 @@ void KateViewConfig::updateConfig()
     }
 
     if (isGlobal()) {
-        const auto allViews = KTextEditor::EditorPrivate::self()->views();
-        for (KTextEditor::ViewPrivate *view : allViews) {
+        for (KTextEditor::ViewPrivate *view : KTextEditor::EditorPrivate::self()->views()) {
             view->updateConfig();
         }
 
         // write config
-        KConfigGroup cg(KTextEditor::EditorPrivate::config(), "KTextEditor View");
+        KConfigGroup cg(KTextEditor::EditorPrivate::config(), QStringLiteral("KTextEditor View"));
         writeConfig(cg);
         KTextEditor::EditorPrivate::config()->sync();
 
@@ -595,7 +625,7 @@ void KateViewConfig::updateConfig()
 
 // BEGIN KateRendererConfig
 KateRendererConfig::KateRendererConfig()
-    : m_lineMarkerColor(KTextEditor::MarkInterface::reservedMarkersCount())
+    : m_lineMarkerColor(KTextEditor::Document::reservedMarkersCount())
     , m_schemaSet(false)
     , m_fontSet(false)
     , m_wordWrapMarkerSet(false)
@@ -625,8 +655,14 @@ KateRendererConfig::KateRendererConfig()
     // init bitarray
     m_lineMarkerColorSet.fill(true);
 
+    // register this as our global instance
+    Q_ASSERT(isGlobal());
     s_global = this;
 
+    // avoid updateConfig effects like config write in constructor, see bug 377067
+    Q_ASSERT(configSessionNumber == 0);
+    ++configSessionNumber;
+
     // Init all known config entries
     addConfigEntry(ConfigEntry(AutoColorThemeSelection, "Auto Color Theme Selection", QString(), true));
 
@@ -634,13 +670,17 @@ KateRendererConfig::KateRendererConfig()
     finalizeConfigEntries();
 
     // init with defaults from config or really hardcoded ones
-    KConfigGroup config(KTextEditor::EditorPrivate::config(), "KTextEditor Renderer");
+    KConfigGroup config(KTextEditor::EditorPrivate::config(), QStringLiteral("KTextEditor Renderer"));
     readConfig(config);
+
+    // avoid updateConfig effects like config write in constructor, see bug 377067
+    Q_ASSERT(configSessionNumber == 1);
+    --configSessionNumber;
 }
 
 KateRendererConfig::KateRendererConfig(KateRenderer *renderer)
     : KateConfig(s_global)
-    , m_lineMarkerColor(KTextEditor::MarkInterface::reservedMarkersCount())
+    , m_lineMarkerColor(KTextEditor::Document::reservedMarkersCount())
     , m_schemaSet(false)
     , m_fontSet(false)
     , m_wordWrapMarkerSet(false)
@@ -675,12 +715,13 @@ KateRendererConfig::~KateRendererConfig() = default;
 
 namespace
 {
-const char KEY_FONT[] = "Font";
+const char KEY_FONT[] = "Text Font";
 const char KEY_COLOR_THEME[] = "Color Theme";
 const char KEY_WORD_WRAP_MARKER[] = "Word Wrap Marker";
 const char KEY_SHOW_INDENTATION_LINES[] = "Show Indentation Lines";
 const char KEY_SHOW_WHOLE_BRACKET_EXPRESSION[] = "Show Whole Bracket Expression";
 const char KEY_ANIMATE_BRACKET_MATCHING[] = "Animate Bracket Matching";
+const char KEY_LINE_HEIGHT_MULTIPLIER[] = "Line Height Multiplier";
 }
 
 void KateRendererConfig::readConfig(const KConfigGroup &config)
@@ -704,6 +745,8 @@ void KateRendererConfig::readConfig(const KConfigGroup &config)
 
     setAnimateBracketMatching(config.readEntry(KEY_ANIMATE_BRACKET_MATCHING, false));
 
+    setLineHeightMultiplier(config.readEntry<qreal>(KEY_LINE_HEIGHT_MULTIPLIER, 1.0));
+
     configEnd();
 }
 
@@ -723,6 +766,8 @@ void KateRendererConfig::writeConfig(KConfigGroup &config)
     config.writeEntry(KEY_SHOW_WHOLE_BRACKET_EXPRESSION, showWholeBracketExpression());
 
     config.writeEntry(KEY_ANIMATE_BRACKET_MATCHING, animateBracketMatching());
+
+    config.writeEntry<qreal>(KEY_LINE_HEIGHT_MULTIPLIER, lineHeightMultiplier());
 }
 
 void KateRendererConfig::updateConfig()
@@ -733,12 +778,12 @@ void KateRendererConfig::updateConfig()
     }
 
     if (isGlobal()) {
-        for (int z = 0; z < KTextEditor::EditorPrivate::self()->views().size(); ++z) {
-            (KTextEditor::EditorPrivate::self()->views())[z]->renderer()->updateConfig();
+        for (auto view : KTextEditor::EditorPrivate::self()->views()) {
+            view->renderer()->updateConfig();
         }
 
         // write config
-        KConfigGroup cg(KTextEditor::EditorPrivate::config(), "KTextEditor Renderer");
+        KConfigGroup cg(KTextEditor::EditorPrivate::config(), QStringLiteral("KTextEditor Renderer"));
         writeConfig(cg);
         KTextEditor::EditorPrivate::config()->sync();
 
@@ -779,8 +824,7 @@ void KateRendererConfig::reloadSchema()
 {
     if (isGlobal()) {
         setSchemaInternal(m_schema);
-        const auto allViews = KTextEditor::EditorPrivate::self()->views();
-        for (KTextEditor::ViewPrivate *view : allViews) {
+        for (KTextEditor::ViewPrivate *view : KTextEditor::EditorPrivate::self()->views()) {
             view->renderer()->config()->reloadSchema();
         }
     }
@@ -995,7 +1039,7 @@ void KateRendererConfig::setHighlightedLineColor(const QColor &col)
     configEnd();
 }
 
-const QColor &KateRendererConfig::lineMarkerColor(KTextEditor::MarkInterface::MarkTypes type) const
+const QColor &KateRendererConfig::lineMarkerColor(KTextEditor::Document::MarkTypes type) const
 {
     int index = 0;
     if (type > 0) {
@@ -1003,7 +1047,7 @@ const QColor &KateRendererConfig::lineMarkerColor(KTextEditor::MarkInterface::Ma
     }
     index -= 1;
 
-    if (index < 0 || index >= KTextEditor::MarkInterface::reservedMarkersCount()) {
+    if (index < 0 || index >= KTextEditor::Document::reservedMarkersCount()) {
         static QColor dummy;
         return dummy;
     }
@@ -1373,6 +1417,13 @@ void KateRendererConfig::setReplaceHighlightColor(const QColor &col)
     configEnd();
 }
 
+void KateRendererConfig::setLineHeightMultiplier(qreal value)
+{
+    configStart();
+    m_lineHeightMultiplier = value;
+    configEnd();
+}
+
 bool KateRendererConfig::showIndentationLines() const
 {
     if (m_showIndentationLinesSet || isGlobal()) {
@@ -1419,7 +1470,7 @@ void KateRendererConfig::setShowWholeBracketExpression(bool on)
     configEnd();
 }
 
-bool KateRendererConfig::animateBracketMatching() const
+bool KateRendererConfig::animateBracketMatching()
 {
     return s_global->m_animateBracketMatching;
 }
