@@ -9,7 +9,7 @@
 
 #include <ktexteditor_export.h>
 
-#include <ktexteditor/markinterface.h>
+#include <ktexteditor/document.h>
 #include <ktexteditor/view.h>
 
 #include <KEncodingProber>
@@ -18,10 +18,10 @@
 #include <map>
 #include <memory>
 
-#include <QBitRef>
+#include <QBitArray>
 #include <QColor>
+#include <QList>
 #include <QObject>
-#include <QVector>
 
 class KConfigGroup;
 namespace KTextEditor
@@ -41,8 +41,6 @@ class EditorPrivate;
 
 class KConfig;
 
-class QTextCodec;
-
 /**
  * Base Class for the Kate Config Classes
  * Current childs are KateDocumentConfig/KateDocumentConfig/KateDocumentConfig
@@ -250,16 +248,6 @@ private:
      */
     const KateConfig *const m_parent = nullptr;
 
-    /**
-     * recursion depth
-     */
-    uint configSessionNumber = 0;
-
-    /**
-     * is a config session running
-     */
-    bool configIsRunning = false;
-
     /**
      * two cases:
      *   - we have m_parent == nullptr => this contains all known config entries
@@ -278,6 +266,12 @@ private:
      * Hash of config keys => config entry, filled only for the object with m_parent == nullptr
      */
     std::unique_ptr<QHash<QString, const ConfigEntry *>> m_configKeyToEntry;
+
+protected:
+    /**
+     * recursion depth
+     */
+    int configSessionNumber = 0;
 };
 
 class KTEXTEDITOR_EXPORT KateGlobalConfig : public KateConfig
@@ -336,13 +330,6 @@ public:
         return setValue(EncodingProberType, type);
     }
 
-    /**
-     * Fallback codec.
-     * Based on fallback encoding.
-     * @return fallback codec
-     */
-    QTextCodec *fallbackCodec() const;
-
     QString fallbackEncoding() const
     {
         return value(FallbackEncoding).toString();
@@ -362,7 +349,6 @@ class KTEXTEDITOR_EXPORT KateDocumentConfig : public KateConfig
 private:
     friend class KTextEditor::EditorPrivate;
 
-    KTEXTEDITOR_NO_EXPORT
     KateDocumentConfig();
 
 public:
@@ -838,13 +824,6 @@ public:
         return value(IndentOnTab).toBool();
     }
 
-    /**
-     * Get current text codec.
-     * Based on current set encoding
-     * @return current text codec.
-     */
-    QTextCodec *codec() const;
-
     QString encoding() const
     {
         return value(Encoding).toString();
@@ -1068,6 +1047,7 @@ public:
         ShowStatusbarFileEncoding,
         StatusbarLineColumnCompact,
         ShowStatusbarEOL,
+        EnableAccessibility,
     };
 
 public:
@@ -1461,7 +1441,7 @@ public:
     const QColor &highlightedLineColor() const;
     void setHighlightedLineColor(const QColor &col);
 
-    const QColor &lineMarkerColor(KTextEditor::MarkInterface::MarkTypes type = KTextEditor::MarkInterface::markType01) const; // markType01 == Bookmark
+    const QColor &lineMarkerColor(KTextEditor::Document::MarkTypes type = KTextEditor::Document::markType01) const; // markType01 == Bookmark
 
     const QColor &highlightedBracketColor() const;
     void setHighlightedBracketColor(const QColor &col);
@@ -1534,6 +1514,7 @@ private:
     KTEXTEDITOR_NO_EXPORT
     void setSchemaInternal(const QString &schema);
 
+private:
     QString m_schema;
     QFont m_font;
     QColor m_backgroundColor;
@@ -1549,7 +1530,7 @@ private:
     QColor m_currentLineNumberColor;
     QColor m_separatorColor;
     QColor m_spellingMistakeLineColor;
-    QVector<QColor> m_lineMarkerColor;
+    std::vector<QColor> m_lineMarkerColor;
 
     QColor m_templateBackgroundColor;
     QColor m_templateEditablePlaceholderColor;
