@@ -21,12 +21,9 @@
 #include "command.h"
 #include "inlinenote.h"
 #include "inlinenotedata.h"
-#include "inlinenoteinterface.h"
 #include "inlinenoteprovider.h"
 #include "katerenderer.h"
 #include "katevariableexpansionmanager.h"
-#include "markinterface.h"
-#include "modificationinterface.h"
 #include "sessionconfiginterface.h"
 #include "texthintinterface.h"
 #include "variable.h"
@@ -41,12 +38,7 @@
 
 using namespace KTextEditor;
 
-Cursor Cursor::fromString(const QStringRef &str) Q_DECL_NOEXCEPT
-{
-    return fromString(QStringView(str));
-}
-
-Cursor Cursor::fromString(QStringView str) Q_DECL_NOEXCEPT
+Cursor Cursor::fromString(QStringView str) noexcept
 {
     // parse format "(line, column)"
     const int startIndex = str.indexOf(QLatin1Char('('));
@@ -60,8 +52,8 @@ Cursor Cursor::fromString(QStringView str) Q_DECL_NOEXCEPT
     bool ok1 = false;
     bool ok2 = false;
 
-    const int line = str.mid(startIndex + 1, commaIndex - startIndex - 1).toString().toInt(&ok1); // FIXME KF6, Qt 5.15.2 and higher
-    const int column = str.mid(commaIndex + 1, endIndex - commaIndex - 1).toString().toInt(&ok2); // FIXME KF6, Qt 5.15.2 and higher
+    const int line = str.mid(startIndex + 1, commaIndex - startIndex - 1).toInt(&ok1);
+    const int column = str.mid(commaIndex + 1, endIndex - commaIndex - 1).toInt(&ok2);
 
     if (!ok1 || !ok2) {
         return invalid();
@@ -70,6 +62,22 @@ Cursor Cursor::fromString(QStringView str) Q_DECL_NOEXCEPT
     return {line, column};
 }
 
+QString Cursor::toString() const
+{
+    return QStringLiteral("(%1, %2)").arg(m_line).arg(m_column);
+}
+
+QDebug operator<<(QDebug s, KTextEditor::Cursor cursor)
+{
+    s.nospace() << "(" << cursor.line() << ", " << cursor.column() << ")";
+    return s.space();
+}
+
+size_t qHash(KTextEditor::Cursor cursor, size_t seed) noexcept
+{
+    return qHash(qMakePair(cursor.line(), cursor.column()), seed);
+}
+
 Editor::Editor(EditorPrivate *impl)
     : QObject()
     , d(impl)
@@ -102,12 +110,7 @@ bool Editor::registerVariablePrefix(const QString &prefix, const QString &descri
     return d->variableExpansionManager()->addVariable(var);
 }
 
-bool Editor::unregisterVariableMatch(const QString &variable)
-{
-    return d->variableExpansionManager()->removeVariable(variable);
-}
-
-bool Editor::unregisterVariablePrefix(const QString &variable)
+bool Editor::unregisterVariable(const QString &variable)
 {
     return d->variableExpansionManager()->removeVariable(variable);
 }
@@ -117,12 +120,12 @@ bool Editor::expandVariable(const QString &variable, KTextEditor::View *view, QS
     return d->variableExpansionManager()->expandVariable(variable, view, output);
 }
 
-void Editor::expandText(const QString &text, KTextEditor::View *view, QString &output) const
+QString Editor::expandText(const QString &text, KTextEditor::View *view) const
 {
-    output = d->variableExpansionManager()->expandText(text, view);
+    return d->variableExpansionManager()->expandText(text, view);
 }
 
-void Editor::addVariableExpansion(const QVector<QWidget *> &widgets, const QStringList &variables) const
+void Editor::addVariableExpansion(const QList<QWidget *> &widgets, const QStringList &variables) const
 {
     d->variableExpansionManager()->showDialog(widgets, variables);
 }
@@ -168,21 +171,36 @@ void View::setStatusBarEnabled(bool enable)
     d->toggleStatusBar();
 }
 
-bool View::insertTemplate(const KTextEditor::Cursor &insertPosition, const QString &templateString, const QString &script)
+bool View::insertTemplate(KTextEditor::Cursor insertPosition, const QString &templateString, const QString &script)
 {
     return d->insertTemplateInternal(insertPosition, templateString, script);
 }
 
-void View::setViewInputMode(InputMode inputMode)
-{
-    d->setInputMode(inputMode);
-}
-
 KSyntaxHighlighting::Theme View::theme() const
 {
     return KateHlManager::self()->repository().theme(d->renderer()->config()->schema());
 }
 
+void View::setCursorPositions(const QList<KTextEditor::Cursor> &positions)
+{
+    d->setCursors(positions);
+}
+
+QList<KTextEditor::Cursor> View::cursorPositions() const
+{
+    return d->cursors();
+}
+
+void View::setSelections(const QList<KTextEditor::Range> &ranges)
+{
+    d->setSelections(ranges);
+}
+
+QList<KTextEditor::Range> View::selectionRanges() const
+{
+    return d->selectionRanges();
+}
+
 ConfigPage::ConfigPage(QWidget *parent)
     : QWidget(parent)
     , d(nullptr)
@@ -228,37 +246,21 @@ ConfigPage *Plugin::configPage(int, QWidget *)
     return nullptr;
 }
 
-MarkInterface::MarkInterface() = default;
-
-MarkInterface::~MarkInterface() = default;
-
-ModificationInterface::ModificationInterface() = default;
-
-ModificationInterface::~ModificationInterface() = default;
-
 SessionConfigInterface::SessionConfigInterface() = default;
 
 SessionConfigInterface::~SessionConfigInterface() = default;
 
-TextHintInterface::TextHintInterface() = default;
-
-TextHintInterface::~TextHintInterface() = default;
-
 TextHintProvider::TextHintProvider() = default;
 
 TextHintProvider::~TextHintProvider() = default;
 
-InlineNoteInterface::InlineNoteInterface() = default;
-
-InlineNoteInterface::~InlineNoteInterface() = default;
-
 InlineNoteProvider::InlineNoteProvider() = default;
 
 InlineNoteProvider::~InlineNoteProvider() = default;
 
 KateInlineNoteData::KateInlineNoteData(KTextEditor::InlineNoteProvider *provider,
                                        const KTextEditor::View *view,
-                                       const KTextEditor::Cursor &position,
+                                       const KTextEditor::Cursor position,
                                        int index,
                                        bool underMouse,
                                        const QFont &font,
@@ -378,7 +380,7 @@ void Command::processText(KTextEditor::View *, const QString &)
 {
 }
 
-void View::setScrollPosition(KTextEditor::Cursor &cursor)
+void View::setScrollPosition(KTextEditor::Cursor cursor)
 {
     d->setScrollPositionInternal(cursor);
 }
@@ -432,3 +434,19 @@ AbstractAnnotationItemDelegate::AbstractAnnotationItemDelegate(QObject *parent)
 }
 
 AbstractAnnotationItemDelegate::~AbstractAnnotationItemDelegate() = default;
+
+AnnotationModel::~AnnotationModel() = default;
+
+#include "moc_abstractannotationitemdelegate.cpp"
+#include "moc_annotationinterface.cpp"
+#include "moc_application.cpp"
+#include "moc_codecompletionmodel.cpp"
+#include "moc_command.cpp"
+#include "moc_configpage.cpp"
+#include "moc_document.cpp"
+#include "moc_editor.cpp"
+#include "moc_inlinenoteprovider.cpp"
+#include "moc_mainwindow.cpp"
+#include "moc_message.cpp"
+#include "moc_plugin.cpp"
+#include "moc_view.cpp"
