@@ -10,22 +10,12 @@
 #ifndef kate_view_h
 #define kate_view_h
 
-#include <ktexteditor/annotationinterface.h>
-#include <ktexteditor/codecompletioninterface.h>
-#include <ktexteditor/configinterface.h>
-#include <ktexteditor/inlinenoteinterface.h>
 #include <ktexteditor/linerange.h>
 #include <ktexteditor/mainwindow.h>
-#include <ktexteditor/texthintinterface.h>
 #include <ktexteditor/view.h>
 
 #include <QJsonDocument>
-#include <QMenu>
-#include <QModelIndex>
 #include <QPointer>
-#include <QScopedPointer>
-#include <QSpacerItem>
-#include <QTextLayout>
 #include <QTimer>
 
 #include <array>
@@ -71,27 +61,19 @@ class KToggleAction;
 class KSelectAction;
 
 class QAction;
+class QTextLayout;
+class QSpacerItem;
+class QMenu;
+class QActionGroup;
 
 namespace KTextEditor
 {
 //
 // Kate KTextEditor::View class ;)
 //
-class KTEXTEDITOR_EXPORT ViewPrivate final : public KTextEditor::View,
-                                             public KTextEditor::TextHintInterface,
-                                             public KTextEditor::CodeCompletionInterfaceV2,
-                                             public KTextEditor::ConfigInterface,
-                                             public KTextEditor::InlineNoteInterface,
-                                             public KTextEditor::AnnotationViewInterfaceV2
+class KTEXTEDITOR_EXPORT ViewPrivate final : public KTextEditor::View
 {
     Q_OBJECT
-    Q_INTERFACES(KTextEditor::TextHintInterface)
-    Q_INTERFACES(KTextEditor::ConfigInterface)
-    Q_INTERFACES(KTextEditor::CodeCompletionInterface)
-    Q_INTERFACES(KTextEditor::CodeCompletionInterfaceV2)
-    Q_INTERFACES(KTextEditor::AnnotationViewInterface)
-    Q_INTERFACES(KTextEditor::AnnotationViewInterfaceV2)
-    Q_INTERFACES(KTextEditor::InlineNoteInterface)
 
     friend class KTextEditor::View;
     friend class ::KateViewInternal;
@@ -119,6 +101,11 @@ public:
     InputMode viewInputMode() const override;
     QString viewInputModeHuman() const override;
 
+    void setViewInputMode(InputMode mode) override
+    {
+        setInputMode(mode);
+    }
+
     void setInputMode(InputMode mode, const bool rememberInConfig = true);
 
 public:
@@ -134,6 +121,7 @@ public Q_SLOTS:
     void paste(const QString *textToPaste = nullptr);
     void cut();
     void copy() const;
+    void screenshot();
 
 private Q_SLOTS:
     /**
@@ -178,7 +166,7 @@ public:
 
     KTextEditor::Cursor cursorPositionVirtual() const override;
 
-    QPoint cursorToCoordinate(const KTextEditor::Cursor &cursor) const override;
+    QPoint cursorToCoordinate(KTextEditor::Cursor cursor) const override;
 
     KTextEditor::Cursor coordinatesToCursor(const QPoint &coord) const override;
 
@@ -186,6 +174,9 @@ public:
 
     bool setCursorPositionVisual(const KTextEditor::Cursor position);
 
+    QScrollBar *verticalScrollBar() const override;
+    QScrollBar *horizontalScrollBar() const override;
+
     /**
      * Return the virtual cursor column, each tab is expanded into the
      * document's tabWidth characters. If word wrap is off, the cursor may be
@@ -224,7 +215,7 @@ public:
             return l.cursor() < r.cursor();
         }
 
-        friend bool operator<(const SecondaryCursor &l, const KTextEditor::Cursor &r)
+        friend bool operator<(const SecondaryCursor &l, KTextEditor::Cursor r)
         {
             return l.cursor() < r;
         }
@@ -245,12 +236,12 @@ public:
     bool isMulticursorNotAllowed() const;
 
     // Adds a secondary cursor
-    void addSecondaryCursor(const KTextEditor::Cursor &cursor);
-    void setSecondaryCursors(const QVector<KTextEditor::Cursor> &positions);
+    void addSecondaryCursor(KTextEditor::Cursor cursor);
+    void setSecondaryCursors(const QList<KTextEditor::Cursor> &positions);
 
     const std::vector<SecondaryCursor> &secondaryCursors() const;
-    QVector<PlainSecondaryCursor> plainSecondaryCursors() const;
-    void addSecondaryCursorsWithSelection(const QVector<PlainSecondaryCursor> &cursorsWithSelection);
+    QList<PlainSecondaryCursor> plainSecondaryCursors() const;
+    void addSecondaryCursorsWithSelection(const QList<PlainSecondaryCursor> &cursorsWithSelection);
 
     void clearSecondaryCursors();
     void clearSecondarySelections();
@@ -260,28 +251,35 @@ public:
     void ensureUniqueCursors(bool matchLine = false);
 
     // For multicursor external api
-    QVector<KTextEditor::Cursor> cursors() const;
-    QVector<KTextEditor::Range> selectionRanges() const;
+    QList<KTextEditor::Cursor> cursors() const;
+    QList<KTextEditor::Range> selectionRanges() const;
 
-    void setCursors(const QVector<KTextEditor::Cursor> &cursorPositions);
-    void setSelections(const QVector<KTextEditor::Range> &selectionRanges);
+    void setCursors(const QList<KTextEditor::Cursor> &cursorPositions);
+    void setSelections(const QList<KTextEditor::Range> &selectionRanges);
 
     // Returns true if primary or secondary cursors have selection
     bool hasSelections() const;
 
 private:
+    KTEXTEDITOR_NO_EXPORT
     bool removeSecondaryCursors(const std::vector<KTextEditor::Cursor> &cursorToRemove, bool removeIfOverlapsSelection = false);
+    KTEXTEDITOR_NO_EXPORT
     Kate::TextRange *newSecondarySelectionRange(KTextEditor::Range);
+    KTEXTEDITOR_NO_EXPORT
     void sortCursors();
+    KTEXTEDITOR_NO_EXPORT
     void paintCursors();
 
     std::vector<SecondaryCursor> m_secondaryCursors;
     bool m_skipCurrentSelection = false;
 
     void addSecondaryCursorDown();
+    // exported for multicursortest
     void addSecondaryCursorUp();
+    // exported for multicursortest
 
 private:
+    KTEXTEDITOR_NO_EXPORT
     void notifyMousePositionChanged(const KTextEditor::Cursor newPosition);
 
     // Internal
@@ -325,12 +323,10 @@ public:
      */
     bool toggleFoldingsInRange(int line);
 
-    //
-    // KTextEditor::CodeCompletionInterfaceV2
-    //
+    // Code Completion Interface
 public:
     bool isCompletionActive() const override;
-    void startCompletion(const KTextEditor::Range &word, KTextEditor::CodeCompletionModel *model) override;
+    void startCompletion(KTextEditor::Range word, KTextEditor::CodeCompletionModel *model) override;
     void startCompletion(const Range &word,
                          const QList<KTextEditor::CodeCompletionModel *> &models = QList<KTextEditor::CodeCompletionModel *>(),
                          KTextEditor::CodeCompletionModel::InvocationType invocationType = KTextEditor::CodeCompletionModel::ManualInvocation) override;
@@ -344,7 +340,7 @@ public:
     void setAutomaticInvocationEnabled(bool enabled = true) override;
 
 Q_SIGNALS:
-    void completionExecuted(KTextEditor::View *view, const KTextEditor::Cursor &position, KTextEditor::CodeCompletionModel *model, const QModelIndex &);
+    void completionExecuted(KTextEditor::View *view, KTextEditor::Cursor position, KTextEditor::CodeCompletionModel *model, const QModelIndex &);
     void completionAborted(KTextEditor::View *view);
 
 public Q_SLOTS:
@@ -392,7 +388,7 @@ private Q_SLOTS:
     // KTextEditor::SelectionInterface stuff
     //
 public Q_SLOTS:
-    bool setSelection(const KTextEditor::Range &selection) override;
+    bool setSelection(KTextEditor::Range selection) override;
 
     bool removeSelection() override
     {
@@ -491,11 +487,6 @@ public:
     bool uniformAnnotationItemSizes() const override;
 
 Q_SIGNALS:
-    void annotationContextMenuAboutToShow(KTextEditor::View *view, QMenu *menu, int line) override;
-    void annotationActivated(KTextEditor::View *view, int line) override;
-    // KF6: fix View -> KTextEditor::View
-    void annotationBorderVisibilityChanged(View *view, bool visible) override;
-
     void navigateLeft();
     void navigateRight();
     void navigateUp();
@@ -541,7 +532,6 @@ public:
 
     bool isLineRTL(int line) const;
 
-    QTextLayout *textLayout(int line) const;
     QTextLayout *textLayout(const KTextEditor::Cursor pos) const;
 
 public Q_SLOTS:
@@ -771,6 +761,7 @@ public:
     bool scrollBarMiniMapAll();
     int dynWrapIndicators();
     bool foldingMarkersOn();
+    bool forceRTLDirection();
 
 private Q_SLOTS:
     /**
@@ -842,11 +833,18 @@ public Q_SLOTS:
     void slotToggleFoldingsInRange();
 
 private:
+    KTEXTEDITOR_NO_EXPORT
     void setupLayout();
+    KTEXTEDITOR_NO_EXPORT
     void setupConnections();
+    KTEXTEDITOR_NO_EXPORT
     void setupActions();
+    KTEXTEDITOR_NO_EXPORT
     void setupEditActions();
+    KTEXTEDITOR_NO_EXPORT
     void setupCodeFolding();
+    KTEXTEDITOR_NO_EXPORT
+    void setupSpeechActions();
 
     std::vector<QAction *> m_editActions;
     QAction *m_editUndo;
@@ -879,14 +877,18 @@ private:
     QAction *m_swapWithClipboard;
     QAction *m_selectAll;
     QAction *m_deSelect;
+    QAction *m_screenshotSelection = nullptr;
 
     QActionGroup *m_inputModeActions;
 
     KToggleAction *m_toggleBlockSelection;
     KToggleAction *m_toggleInsert;
     KToggleAction *m_toggleWriteLock;
+    KToggleAction *m_forceRTLDirection;
 
     bool m_hasWrap;
+    bool m_forceRTL = false;
+    bool m_accessibilityEnabled = false;
 
     KTextEditor::DocumentPrivate *const m_doc;
     Kate::TextFolding m_textFolding;
@@ -945,6 +947,7 @@ public:
     KateDictionaryBar *dictionaryBar();
 
 private:
+    KTEXTEDITOR_NO_EXPORT
     KateGotoBar *gotoBar();
 
     /**
@@ -1007,10 +1010,13 @@ public:
     void clearFoldingState();
 
 private:
+    KTEXTEDITOR_NO_EXPORT
     void clearHighlights();
+    KTEXTEDITOR_NO_EXPORT
     void createHighlights();
 
 private:
+    KTEXTEDITOR_NO_EXPORT
     void selectionChangedForHighlights();
 
     /**
@@ -1104,7 +1110,7 @@ public:
     /**
      * Used by Document::postMessage().
      */
-    void postMessage(KTextEditor::Message *message, QList<QSharedPointer<QAction>> actions);
+    void postMessage(KTextEditor::Message *message, QList<std::shared_ptr<QAction>> actions);
 
 private:
     /**
@@ -1190,7 +1196,7 @@ public:
     /**
      * Returns the attribute for the default style \p defaultStyle.
      */
-    Attribute::Ptr defaultStyleAttribute(DefaultStyle defaultStyle) const override;
+    Attribute::Ptr defaultStyleAttribute(KSyntaxHighlighting::Theme::TextStyle defaultStyle) const override;
 
     /**
      * Get the list of AttributeBlocks for a given \p line in the document.
@@ -1223,7 +1229,7 @@ private:
      * destruction before other QObject auto-cleanup as it
      * manage sub objects on its own that have this view as parent
      */
-    QScopedPointer<KateScriptActionMenu> m_scriptActionMenu;
+    std::unique_ptr<KateScriptActionMenu> m_scriptActionMenu;
 
     // for showSearchWrappedHint()
     QPointer<KTextEditor::Message> m_wrappedMessage;
