@@ -12,8 +12,10 @@
 #include "kateglobal.h"
 #include "katelayoutcache.h"
 #include "katerenderer.h"
+#include "kateview.h"
 #include "kateviewinternal.h"
 #include "kateviinputmode.h"
+#include "ktexteditor/message.h"
 #include <vimode/globalstate.h>
 #include <vimode/inputmodemanager.h>
 #include <vimode/jumps.h>
@@ -48,7 +50,7 @@ void ModeBase::yankToClipBoard(QChar chosen_register, const QString &text)
     // textlength > 1 and there is something else then whitespace
     if ((chosen_register == QLatin1Char('0') || chosen_register == QLatin1Char('-') || chosen_register == PrependNumberedRegister) && text.length() > 1
         && !text.trimmed().isEmpty()) {
-        KTextEditor::EditorPrivate::self()->copyToClipboard(text);
+        KTextEditor::EditorPrivate::self()->copyToClipboard(text, m_view->doc()->url().fileName());
     }
 }
 
@@ -68,19 +70,19 @@ bool ModeBase::deleteRange(Range &r, OperationMode mode, bool addToRegister)
         res = doc()->removeText(r.toEditorRange(), mode == Block);
     }
 
-    // the BlackHoleRegister here is only a placeholder to signify that no register was selected
+    // the UnnamedRegister here is only a placeholder to signify that no register was selected
     // this is needed because the fallback register depends on whether the deleted text spans a line/lines
-    QChar chosenRegister = getChosenRegister(BlackHoleRegister);
+    QChar chosenRegister = getChosenRegister(UnnamedRegister);
     if (addToRegister) {
         fillRegister(chosenRegister, removedText, mode);
     }
 
-    const QChar lastChar = removedText.count() > 0 ? removedText.back() : QLatin1Char('\0');
-    if (r.startLine != r.endLine || lastChar == QLatin1Char('\n') || lastChar == QLatin1Char('\r')) {
+    const QChar lastChar = removedText.size() > 0 ? removedText.back() : QLatin1Char('\0');
+    if (chosenRegister != BlackHoleRegister && (r.startLine != r.endLine || lastChar == QLatin1Char('\n') || lastChar == QLatin1Char('\r'))) {
         // for deletes spanning a line/lines, always prepend to the numbered registers
         fillRegister(PrependNumberedRegister, removedText, mode);
         chosenRegister = PrependNumberedRegister;
-    } else if (chosenRegister == BlackHoleRegister) {
+    } else if (chosenRegister == UnnamedRegister) {
         // only set the SmallDeleteRegister when no register was selected
         fillRegister(SmallDeleteRegister, removedText, mode);
         chosenRegister = SmallDeleteRegister;
@@ -119,7 +121,7 @@ const QString ModeBase::getRange(Range &r, OperationMode mode) const
 
 const QString ModeBase::getLine(int line) const
 {
-    return (line < 0) ? m_view->currentTextLine() : doc()->line(line);
+    return (line < 0) ? m_view->doc()->line(m_view->cursorPosition().line()) : doc()->line(line);
 }
 
 const QChar ModeBase::getCharUnderCursor() const
@@ -186,7 +188,9 @@ KTextEditor::Cursor ModeBase::findNextWordStart(int fromLine, int fromColumn, bo
 
     const QRegularExpression startOfWord(startOfWordPattern, QRegularExpression::UseUnicodePropertiesOption); // start of a word
     static const QRegularExpression nonSpaceAfterSpace(QStringLiteral("\\s\\S"), QRegularExpression::UseUnicodePropertiesOption); // non-space right after space
-    static const QRegularExpression nonWordAfterWord(QStringLiteral("\\b(?!\\s)\\W"), QRegularExpression::UseUnicodePropertiesOption); // word-boundary followed by a non-word which is not a space
+    static const QRegularExpression nonWordAfterWord(
+        QStringLiteral("\\b(?!\\s)\\W"),
+        QRegularExpression::UseUnicodePropertiesOption); // word-boundary followed by a non-word which is not a space
 
     int l = fromLine;
     int c = fromColumn;
@@ -366,7 +370,9 @@ KTextEditor::Cursor ModeBase::findPrevWordStart(int fromLine, int fromColumn, bo
 
     const QRegularExpression startOfWord(startOfWordPattern, QRegularExpression::UseUnicodePropertiesOption); // start of a word
     static const QRegularExpression nonSpaceAfterSpace(QStringLiteral("\\s\\S"), QRegularExpression::UseUnicodePropertiesOption); // non-space right after space
-    static const QRegularExpression nonWordAfterWord(QStringLiteral("\\b(?!\\s)\\W"), QRegularExpression::UseUnicodePropertiesOption); // word-boundary followed by a non-word which is not a space
+    static const QRegularExpression nonWordAfterWord(
+        QStringLiteral("\\b(?!\\s)\\W"),
+        QRegularExpression::UseUnicodePropertiesOption); // word-boundary followed by a non-word which is not a space
     static const QRegularExpression startOfLine(QStringLiteral("^\\S"), QRegularExpression::UseUnicodePropertiesOption); // non-space at start of line
 
     int l = fromLine;
@@ -769,7 +775,7 @@ int ModeBase::findLineStartingWitchChar(const QChar &c, int count, bool forward)
     return -1;
 }
 
-void ModeBase::updateCursor(const KTextEditor::Cursor &c) const
+void ModeBase::updateCursor(const KTextEditor::Cursor c) const
 {
     m_viInputModeManager->updateCursor(c);
 }
@@ -904,7 +910,8 @@ Range ModeBase::goVisualLineUpDown(int lines)
         // Find the beginning of the visual line "lines" visual lines down.
         while (count > 0) {
             finishVisualLine++;
-            if (finishVisualLine >= cache->line(finishRealLine)->viewLineCount()) {
+            const KateLineLayout *lineLayout = cache->line(finishRealLine);
+            if (lineLayout && finishVisualLine >= lineLayout->viewLineCount()) {
                 finishRealLine++;
                 finishVisualLine = 0;
             }
@@ -924,7 +931,12 @@ Range ModeBase::goVisualLineUpDown(int lines)
                     invalidPos = true;
                     break;
                 }
-                finishVisualLine = cache->line(finishRealLine)->viewLineCount() - 1;
+                const auto lineLayout = cache->line(finishRealLine);
+                if (!lineLayout) {
+                    finishVisualLine = 0;
+                    continue;
+                }
+                finishVisualLine = lineLayout->viewLineCount() - 1;
             }
             count--;
         }
@@ -947,8 +959,14 @@ Range ModeBase::goVisualLineUpDown(int lines)
         // Adjust for the fact that if the portion of the line before wrapping is indented,
         // the continuations are also "invisibly" (i.e. without any spaces in the text itself) indented.
         const bool isWrappedContinuation = (cache->textLayout(startRealLine, startVisualLine).lineLayout().lineNumber() != 0);
-        const int numInvisibleIndentChars =
-            isWrappedContinuation ? startLine->toVirtualColumn(cache->line(startRealLine)->textLine()->nextNonSpaceChar(0), tabstop) : 0;
+        const int numInvisibleIndentChars = [&] {
+            if (isWrappedContinuation) {
+                if (auto l = doc()->plainKateTextLine(startRealLine)) {
+                    return startLine->toVirtualColumn(l->nextNonSpaceChar(0), tabstop);
+                }
+            }
+            return 0;
+        }();
 
         const int realLineStartColumn = cache->textLayout(startRealLine, startVisualLine).startCol();
         const int lineStartVirtualColumn = startLine->toVirtualColumn(realLineStartColumn, tabstop);
@@ -964,8 +982,14 @@ Range ModeBase::goVisualLineUpDown(int lines)
     // Adjust for the fact that if the portion of the line before wrapping is indented,
     // the continuations are also "invisibly" (i.e. without any spaces in the text itself) indented.
     const bool isWrappedContinuation = (cache->textLayout(finishRealLine, finishVisualLine).lineLayout().lineNumber() != 0);
-    const int numInvisibleIndentChars =
-        isWrappedContinuation ? endLine->toVirtualColumn(cache->line(finishRealLine)->textLine()->nextNonSpaceChar(0), tabstop) : 0;
+    const int numInvisibleIndentChars = [&] {
+        if (isWrappedContinuation) {
+            if (auto l = doc()->plainKateTextLine(finishRealLine)) {
+                return endLine->toVirtualColumn(l->nextNonSpaceChar(0), tabstop);
+            }
+        }
+        return 0;
+    }();
     if (m_stickyColumn == (unsigned int)KateVi::EOL) {
         const int visualEndColumn = cache->textLayout(finishRealLine, finishVisualLine).lineLayout().textLength() - 1;
         r.endColumn = endLine->fromVirtualColumn(visualEndColumn + realLineStartColumn - numInvisibleIndentChars, tabstop);
@@ -1099,7 +1123,7 @@ QString ModeBase::getVerbatimKeys() const
     return m_keysVerbatim;
 }
 
-const QChar ModeBase::getCharAtVirtualColumn(const QString &line, int virtualColumn, int tabWidth) const
+const QChar ModeBase::getCharAtVirtualColumn(const QString &line, int virtualColumn, int tabWidth)
 {
     int column = 0;
     int tempCol = 0;
@@ -1196,14 +1220,15 @@ void ModeBase::addToNumberUnderCursor(int count)
     } else if (base == 8) {
         basePrefix = QStringLiteral("0");
     }
-    const QStringRef withoutBase = numberAsString.midRef(basePrefix.length());
+
+    const int withoutBaseLength = numberAsString.length() - basePrefix.length();
 
     const int newNumber = originalNumber + count;
 
     // Create the new text string to be inserted. Prepend with “0x” if in base 16, and "0" if base 8.
     // For non-decimal numbers, try to keep the length of the number the same (including leading 0's).
     const QString newNumberPadded =
-        (base == 10) ? QStringLiteral("%1").arg(newNumber, 0, base) : QStringLiteral("%1").arg(newNumber, withoutBase.length(), base, QLatin1Char('0'));
+        (base == 10) ? QStringLiteral("%1").arg(newNumber, 0, base) : QStringLiteral("%1").arg(newNumber, withoutBaseLength, base, QLatin1Char('0'));
     const QString newNumberText = basePrefix + newNumberPadded;
 
     // Replace the old number string with the new.
@@ -1216,9 +1241,8 @@ void ModeBase::addToNumberUnderCursor(int count)
 
 void ModeBase::switchView(Direction direction)
 {
-    QList<KTextEditor::ViewPrivate *> visible_views;
-    const auto views = KTextEditor::EditorPrivate::self()->views();
-    for (KTextEditor::ViewPrivate *view : views) {
+    std::vector<KTextEditor::ViewPrivate *> visible_views;
+    for (KTextEditor::ViewPrivate *view : KTextEditor::EditorPrivate::self()->views()) {
         if (view->isVisible()) {
             visible_views.push_back(view);
         }
@@ -1235,12 +1259,17 @@ void ModeBase::switchView(Direction direction)
     int curr_cursor_x = globalPos.x();
 
     KTextEditor::ViewPrivate *bestview = nullptr;
-    int best_x1 = -1, best_x2 = -1, best_y1 = -1, best_y2 = -1, best_center_y = -1, best_center_x = -1;
+    int best_x1 = -1;
+    int best_x2 = -1;
+    int best_y1 = -1;
+    int best_y2 = -1;
+    int best_center_y = -1;
+    int best_center_x = -1;
 
-    if (direction == Next && visible_views.count() != 1) {
-        for (int i = 0; i < visible_views.count(); i++) {
+    if (direction == Next && visible_views.size() != 1) {
+        for (size_t i = 0; i < visible_views.size(); i++) {
             if (visible_views.at(i) == m_view) {
-                if (i != visible_views.count() - 1) {
+                if (i != visible_views.size() - 1) {
                     bestview = visible_views.at(i + 1);
                 } else {
                     bestview = visible_views.at(0);
@@ -1248,7 +1277,7 @@ void ModeBase::switchView(Direction direction)
             }
         }
     } else {
-        for (KTextEditor::ViewPrivate *view : qAsConst(visible_views)) {
+        for (KTextEditor::ViewPrivate *view : visible_views) {
             QPoint point = view->mapToGlobal(view->pos());
             int x1 = point.x();
             int x2 = point.x() + view->width();
